@@ -1,23 +1,23 @@
 /*************************************************************************
- *									 *
- * Open Dynamics Engine, Copyright (C) 2001, 2002 Russell L. Smith.	 *
- * All rights reserved.  Email: russ@q12.org   Web: www.q12.org 	 *
- *									 *
- * This library is free software; you can redistribute it and/or	 *
- * modify it under the terms of EITHER: 				 *
+ *                                                                       *
+ * Open Dynamics Engine, Copyright (C) 2001, 2002 Russell L. Smith.      *
+ * All rights reserved.  Email: russ@q12.org   Web: www.q12.org          *
+ *                                                                       *
+ * This library is free software; you can redistribute it and/or         *
+ * modify it under the terms of EITHER:                                  *
  *   (1) The GNU Lesser General Public License as published by the Free  *
- *	 Software Foundation; either version 2.1 of the License, or (at  *
- *	 your option) any later version. The text of the GNU Lesser	 *
- *	 General Public License is included with this library in the	 *
- *	 file LICENSE.TXT.						 *
- *   (2) The BSD-style license that is included with this library in	 *
- *	 the file LICENSE-BSD.TXT.					 *
- *									 *
- * This library is distributed in the hope that it will be useful, 	 *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of	 *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files	 *
- * LICENSE.TXT and LICENSE-BSD.TXT for more details.			 *
- *									 *
+ *   Software Foundation; either version 2.1 of the License, or (at      *
+ *   your option) any later version. The text of the GNU Lesser          *
+ *   General Public License is included with this library in the         *
+ *   file LICENSE.TXT.                                                   *
+ *   (2) The BSD-style license that is included with this library in     *
+ *   the file LICENSE-BSD.TXT.                                           *
+ *                                                                       *
+ * This library is distributed in the hope that it will be useful,       *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files    *
+ * LICENSE.TXT and LICENSE-BSD.TXT for more details.                     *
+ *                                                                       *
  *************************************************************************/
 
 /* C++ interface for non-collision stuff */
@@ -35,22 +35,22 @@
 
 class dWorldSimpleIDContainer {
 protected:
-	dWorldID _id;
+  dWorldID _id;
 
-	dWorldSimpleIDContainer(): _id(0) {}
-	~dWorldSimpleIDContainer() { destroy(); }
+  dWorldSimpleIDContainer(): _id(0) {}
+  ~dWorldSimpleIDContainer() { destroy(); }
 
-	void destroy() { 
-		if (_id) {
-			dWorldDestroy(_id); 
-			_id = 0;
-		}
-	}
+  void destroy() {
+    if (_id) {
+      dWorldDestroy(_id);
+      _id = 0;
+    }
+  }
 };
 
 class dWorldDynamicIDContainer: public dWorldSimpleIDContainer {
 protected:
-	virtual ~dWorldDynamicIDContainer() {}
+  virtual ~dWorldDynamicIDContainer() {}
 };
 
 template <class dWorldTemplateBase>
@@ -126,24 +126,32 @@ public:
     { return dWorldGetQuickStepInertiaRatioReduction (get_id()); }
   dReal getQuickStepContactResidualSmoothing() const
     { return dWorldGetQuickStepContactResidualSmoothing (get_id()); }
+  bool getQuickStepThreadPositionCorrection() const
+    { return dWorldGetQuickStepThreadPositionCorrection (get_id()); }
   bool getQuickStepExperimentalRowReordering() const
     { return dWorldGetQuickStepExperimentalRowReordering (get_id()); }
   dReal getQuickStepWarmStartFactor() const
     { return dWorldGetQuickStepWarmStartFactor (get_id()); }
   int getQuickStepExtraFrictionIterations() const
     { return dWorldGetQuickStepExtraFrictionIterations (get_id()); }
+  int getQuickStepFrictionModel() const
+    { return dWorldGetQuickStepFrictionModel(get_id()); }
   void setQuickStepInertiaRatioReduction(bool irr)
     { dWorldSetQuickStepInertiaRatioReduction (get_id(), irr); }
   void setQuickStepContactResidualSmoothing(dReal smoo)
     { dWorldSetQuickStepContactResidualSmoothing (get_id(), smoo); }
+  void setQuickStepThreadPositionCorrection(bool thread)
+    { dWorldSetQuickStepThreadPositionCorrection (get_id(), thread); }
   void setQuickStepExperimentalRowReordering(bool order)
     { dWorldSetQuickStepExperimentalRowReordering (get_id(), order); }
   void setQuickStepWarmStartFactor(dReal warm)
     { dWorldSetQuickStepWarmStartFactor (get_id(), warm); }
   void setQuickStepExtraFrictionIterations(int iters)
     { dWorldSetQuickStepExtraFrictionIterations (get_id(), iters); }
+  void setQuickStepFrictionModel(int fricmodel)
+    { dWorldSetQuickStepFrictionModel(get_id(), fricmodel); }
 
-  void  setAutoDisableLinearThreshold (dReal threshold) 
+  void  setAutoDisableLinearThreshold (dReal threshold)
     { dWorldSetAutoDisableLinearThreshold (get_id(), threshold); }
   dReal getAutoDisableLinearThreshold() const
     { return dWorldGetAutoDisableLinearThreshold (get_id()); }
@@ -193,30 +201,30 @@ public:
   dReal getContactSurfaceLayer() const
     { return dWorldGetContactSurfaceLayer (get_id()); }
 
-  void impulseToForce (dReal stepsize, dReal ix, dReal iy, dReal iz, 
-		       dVector3 force)
+  void impulseToForce (dReal stepsize, dReal ix, dReal iy, dReal iz,
+           dVector3 force)
     { dWorldImpulseToForce (get_id(), stepsize, ix, iy, iz, force); }
 };
 
 
 class dBodySimpleIDContainer {
 protected:
-	dBodyID _id;
+  dBodyID _id;
 
-	dBodySimpleIDContainer(): _id(0) {}
-	~dBodySimpleIDContainer() { destroy(); }
+  dBodySimpleIDContainer(): _id(0) {}
+  ~dBodySimpleIDContainer() { destroy(); }
 
-	void destroy() { 
-		if (_id) {
-			dBodyDestroy(_id); 
-			_id = 0;
-		}
-	}
+  void destroy() {
+    if (_id) {
+      dBodyDestroy(_id);
+      _id = 0;
+    }
+  }
 };
 
 class dBodyDynamicIDContainer: public dBodySimpleIDContainer {
 protected:
-	virtual ~dBodyDynamicIDContainer() {}
+  virtual ~dBodyDynamicIDContainer() {}
 };
 
 template <class dBodyTemplateBase, class dWorldTemplateBase>
@@ -316,26 +324,26 @@ public:
   void addRelTorque (const dVector3 t)
     { addRelTorque (t[0], t[1], t[2]); }
 
-  void addForceAtPos (dReal fx, dReal fy, dReal fz, 
-		      dReal px, dReal py, dReal pz)
+  void addForceAtPos (dReal fx, dReal fy, dReal fz,
+          dReal px, dReal py, dReal pz)
     { dBodyAddForceAtPos (get_id(), fx, fy, fz, px, py, pz); }
   void addForceAtPos (const dVector3 f, const dVector3 p)
     { addForceAtPos (f[0], f[1], f[2], p[0], p[1], p[2]); }
 
-  void addForceAtRelPos (dReal fx, dReal fy, dReal fz, 
+  void addForceAtRelPos (dReal fx, dReal fy, dReal fz,
                          dReal px, dReal py, dReal pz)
     { dBodyAddForceAtRelPos (get_id(), fx, fy, fz, px, py, pz); }
   void addForceAtRelPos (const dVector3 f, const dVector3 p)
     { addForceAtRelPos (f[0], f[1], f[2], p[0], p[1], p[2]); }
 
-  void addRelForceAtPos (dReal fx, dReal fy, dReal fz, 
-			 dReal px, dReal py, dReal pz)
+  void addRelForceAtPos (dReal fx, dReal fy, dReal fz,
+       dReal px, dReal py, dReal pz)
     { dBodyAddRelForceAtPos (get_id(), fx, fy, fz, px, py, pz); }
   void addRelForceAtPos (const dVector3 f, const dVector3 p)
     { addRelForceAtPos (f[0], f[1], f[2], p[0], p[1], p[2]); }
 
-  void addRelForceAtRelPos (dReal fx, dReal fy, dReal fz, 
-			    dReal px, dReal py, dReal pz)
+  void addRelForceAtRelPos (dReal fx, dReal fy, dReal fz,
+          dReal px, dReal py, dReal pz)
     { dBodyAddRelForceAtRelPos (get_id(), fx, fy, fz, px, py, pz); }
   void addRelForceAtRelPos (const dVector3 f, const dVector3 p)
     { addRelForceAtRelPos (f[0], f[1], f[2], p[0], p[1], p[2]); }
@@ -423,7 +431,7 @@ public:
   bool isConnectedTo (dBodyID body) const
     { return dAreConnected (get_id(), body) != 0; }
 
-  void  setAutoDisableLinearThreshold (dReal threshold) 
+  void  setAutoDisableLinearThreshold (dReal threshold)
     { dBodySetAutoDisableLinearThreshold (get_id(), threshold); }
   dReal getAutoDisableLinearThreshold() const
     { return dBodyGetAutoDisableLinearThreshold (get_id()); }
@@ -480,22 +488,22 @@ public:
 
 class dJointGroupSimpleIDContainer {
 protected:
-	dJointGroupID _id;
+  dJointGroupID _id;
 
-	dJointGroupSimpleIDContainer(): _id(0) {}
-	~dJointGroupSimpleIDContainer() { destroy(); }
+  dJointGroupSimpleIDContainer(): _id(0) {}
+  ~dJointGroupSimpleIDContainer() { destroy(); }
 
-	void destroy() { 
-		if (_id) {
-			dJointGroupDestroy(_id); 
-			_id = 0;
-		}
-	}
+  void destroy() {
+    if (_id) {
+      dJointGroupDestroy(_id);
+      _id = 0;
+    }
+  }
 };
 
 class dJointGroupDynamicIDContainer: public dJointGroupSimpleIDContainer {
 protected:
-	virtual ~dJointGroupDynamicIDContainer() {}
+  virtual ~dJointGroupDynamicIDContainer() {}
 };
 
 template <class dJointGroupTemplateBase>
@@ -513,7 +521,7 @@ protected:
 public:
   dJointGroupTemplate ()
     { set_id(dJointGroupCreate(0)); }
-  
+
   void create () {
     destroy();
     set_id(dJointGroupCreate(0));
@@ -533,22 +541,22 @@ public:
 
 class dJointSimpleIDContainer {
 protected:
-	dJointID _id;
+  dJointID _id;
 
-	dJointSimpleIDContainer(): _id(0) {}
-	~dJointSimpleIDContainer() { destroy(); }
+  dJointSimpleIDContainer(): _id(0) {}
+  ~dJointSimpleIDContainer() { destroy(); }
 
-	void destroy() { 
-		if (_id) {
-			dJointDestroy (_id); 
-			_id = 0;
-		}
-	}
+  void destroy() {
+    if (_id) {
+      dJointDestroy (_id);
+      _id = 0;
+    }
+  }
 };
 
 class dJointDynamicIDContainer: public dJointSimpleIDContainer {
 protected:
-	virtual ~dJointDynamicIDContainer() {}
+  virtual ~dJointDynamicIDContainer() {}
 };
 
 template <class dJointTemplateBase, class dWorldTemplateBase, class dBodyTemplateBase>
@@ -684,7 +692,7 @@ public:
   }
   void create (dWorldTemplate<dWorldTemplateBase>& world, dJointGroupID group=0)
     { create(world.id(), group); }
-  
+
   void setAnchor (dReal x, dReal y, dReal z)
     { dJointSetHingeAnchor (get_id(), x, y, z); }
   void setAnchor (const dVector3 a)
@@ -713,7 +721,7 @@ public:
   // TODO: expose params through methods
 
   void addTorque (dReal torque)
-	{ dJointAddHingeTorque(get_id(), torque); }
+  { dJointAddHingeTorque(get_id(), torque); }
 };
 
 
@@ -765,7 +773,7 @@ public:
   // TODO: expose params through methods
 
   void addForce (dReal force)
-	{ dJointAddSliderForce(get_id(), force); }
+  { dJointAddSliderForce(get_id(), force); }
 };
 
 
@@ -817,7 +825,7 @@ public:
   // TODO: expose params through methods
 
   void addForce (dReal force)
-	{ dJointAddScrewForce(get_id(), force); }
+  { dJointAddScrewForce(get_id(), force); }
 };
 
 
@@ -877,7 +885,7 @@ public:
   virtual dReal getParam (int parameter) const
     { return dJointGetUniversalParam (get_id(), parameter); }
   // TODO: expose params through methods
-  
+
   void getAngles(dReal *angle1, dReal *angle2) const
     { dJointGetUniversalAngles (get_id(), angle1, angle2); }
 
@@ -891,7 +899,7 @@ public:
     { return dJointGetUniversalAngle2Rate (get_id()); }
 
   void addTorques (dReal torque1, dReal torque2)
-	{ dJointAddUniversalTorques(get_id(), torque1, torque2); }
+  { dJointAddUniversalTorques(get_id(), torque1, torque2); }
 };
 
 
@@ -936,7 +944,7 @@ public:
     { dJointSetHinge2Axis2 (get_id(), x, y, z); }
   void setAxis2 (const dVector3 a)
     { setAxis2 (a[0], a[1], a[2]); }
-    
+
   void getAnchor (dVector3 result) const
     { dJointGetHinge2Anchor (get_id(), result); }
   void getAnchor2 (dVector3 result) const
@@ -960,7 +968,7 @@ public:
   // TODO: expose params through methods
 
   void addTorques(dReal torque1, dReal torque2)
-	{ dJointAddHinge2Torques(get_id(), torque1, torque2); }
+  { dJointAddHinge2Torques(get_id(), torque1, torque2); }
 };
 
 
@@ -1249,7 +1257,7 @@ public:
     destroy();
     set_id(dJointCreateContact(world, group, contact));
   }
-  
+
   void create (dWorldTemplate<dWorldTemplateBase>& world, dJointGroupID group, dContact *contact)
     { create(world.id(), group, contact); }
 };
@@ -1348,7 +1356,7 @@ public:
   // TODO: expose params through methods
 
   void addTorques(dReal torque1, dReal torque2, dReal torque3)
-	{ dJointAddAMotorTorques(get_id(), torque1, torque2, torque3); }
+  { dJointAddAMotorTorques(get_id(), torque1, torque2, torque3); }
 };
 
 
