@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -85,22 +85,28 @@ std::string Mesh::GetName() const
 //////////////////////////////////////////////////
 math::Vector3 Mesh::GetMax() const
 {
-  math::Vector3 max;
+  return this->Max();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Mesh::Max() const
+{
+  ignition::math::Vector3d max;
   std::vector<SubMesh*>::const_iterator iter;
 
-  max.x = -FLT_MAX;
-  max.y = -FLT_MAX;
-  max.z = -FLT_MAX;
+  max.X(-FLT_MAX);
+  max.Y(-FLT_MAX);
+  max.Z(-FLT_MAX);
 
   for (iter = this->submeshes.begin(); iter != this->submeshes.end(); ++iter)
   {
     if ((*iter)->GetVertexCount() <= 2)
       continue;
 
-    math::Vector3 smax = (*iter)->GetMax();
-    max.x = std::max(max.x, smax.x);
-    max.y = std::max(max.y, smax.y);
-    max.z = std::max(max.z, smax.z);
+    ignition::math::Vector3d smax = (*iter)->Max();
+    max.X(std::max(max.X(), smax.X()));
+    max.Y(std::max(max.Y(), smax.Y()));
+    max.Z(std::max(max.Z(), smax.Z()));
   }
 
   return max;
@@ -109,22 +115,28 @@ math::Vector3 Mesh::GetMax() const
 //////////////////////////////////////////////////
 math::Vector3 Mesh::GetMin() const
 {
-  math::Vector3 min;
+  return this->Min();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Mesh::Min() const
+{
+  ignition::math::Vector3d min;
   std::vector<SubMesh *>::const_iterator iter;
 
-  min.x = FLT_MAX;
-  min.y = FLT_MAX;
-  min.z = FLT_MAX;
+  min.X(FLT_MAX);
+  min.Y(FLT_MAX);
+  min.Z(FLT_MAX);
 
   for (iter = this->submeshes.begin(); iter != this->submeshes.end(); ++iter)
   {
     if ((*iter)->GetVertexCount() <= 2)
       continue;
 
-    math::Vector3 smin = (*iter)->GetMin();
-    min.x = std::min(min.x, smin.x);
-    min.y = std::min(min.y, smin.y);
-    min.z = std::min(min.z, smin.z);
+    ignition::math::Vector3d smin = (*iter)->Min();
+    min.X(std::min(min.X(), smin.X()));
+    min.Y(std::min(min.Y(), smin.Y()));
+    min.Z(std::min(min.Z(), smin.Z()));
   }
 
   return min;
@@ -259,6 +271,18 @@ const Material *Mesh::GetMaterial(int index) const
   return NULL;
 }
 
+//////////////////////////////////////////////////
+int Mesh::GetMaterialIndex(const Material *_mat) const
+{
+  for (unsigned int i = 0; i < this->materials.size(); ++i)
+  {
+    if (this->materials[i] == _mat)
+      return i;
+  }
+
+  return -1;
+}
+
 //////////////////////////////////////////////////
 void Mesh::FillArrays(float **_vertArr, int **_indArr) const
 {
@@ -299,7 +323,7 @@ void Mesh::FillArrays(float **_vertArr, int **_indArr) const
 
     memcpy(vPtr, vertTmp, sizeof(vertTmp[0])*(*iter)->GetVertexCount()*3);
 
-    for (unsigned int i = 0; i < (*iter)->GetIndexCount(); i++)
+    for (unsigned int i = 0; i < (*iter)->GetIndexCount(); ++i)
     {
       (*_indArr)[index++] = (*iter)->GetIndex(i) + offset;
     }
@@ -352,6 +376,12 @@ void Mesh::Scale(double _factor)
 
 //////////////////////////////////////////////////
 void Mesh::SetScale(const math::Vector3 &_factor)
+{
+  this->SetScale(_factor.Ign());
+}
+
+//////////////////////////////////////////////////
+void Mesh::SetScale(const ignition::math::Vector3d &_factor)
 {
   std::vector<SubMesh*>::iterator iter;
   for (iter = this->submeshes.begin(); iter != this->submeshes.end(); ++iter)
@@ -360,6 +390,12 @@ void Mesh::SetScale(const math::Vector3 &_factor)
 
 //////////////////////////////////////////////////
 void Mesh::GenSphericalTexCoord(const math::Vector3 &_center)
+{
+  this->GenSphericalTexCoord(_center.Ign());
+}
+
+//////////////////////////////////////////////////
+void Mesh::GenSphericalTexCoord(const ignition::math::Vector3d &_center)
 {
   std::vector<SubMesh*>::iterator siter;
   for (siter = this->submeshes.begin(); siter != this->submeshes.end(); ++siter)
@@ -369,9 +405,15 @@ void Mesh::GenSphericalTexCoord(const math::Vector3 &_center)
 //////////////////////////////////////////////////
 void Mesh::Center(const math::Vector3 &_center)
 {
-  math::Vector3 min, max, half;
-  min = this->GetMin();
-  max = this->GetMax();
+  this->Center(_center.Ign());
+}
+
+//////////////////////////////////////////////////
+void Mesh::Center(const ignition::math::Vector3d &_center)
+{
+  ignition::math::Vector3d min, max, half;
+  min = this->Min();
+  max = this->Max();
   half = (max - min) * 0.5;
 
   this->Translate(_center - (min + half));
@@ -379,6 +421,12 @@ void Mesh::Center(const math::Vector3 &_center)
 
 //////////////////////////////////////////////////
 void Mesh::Translate(const math::Vector3 &_vec)
+{
+  this->Translate(_vec.Ign());
+}
+
+//////////////////////////////////////////////////
+void Mesh::Translate(const ignition::math::Vector3d &_vec)
 {
   std::vector<SubMesh*>::iterator iter;
 
@@ -404,6 +452,12 @@ SubMesh::SubMesh()
 //////////////////////////////////////////////////
 SubMesh::SubMesh(const SubMesh *_mesh)
 {
+  if (!_mesh)
+  {
+    gzerr << "Submesh is NULL." << std::endl;
+    return;
+  }
+
   this->name = _mesh->name;
   this->materialIndex = _mesh->materialIndex;
   this->primitiveType = _mesh->primitiveType;
@@ -443,6 +497,15 @@ SubMesh::PrimitiveType SubMesh::GetPrimitiveType() const
 
 //////////////////////////////////////////////////
 void SubMesh::CopyVertices(const std::vector<math::Vector3> &_verts)
+{
+  std::vector<ignition::math::Vector3d> verts;
+  for (auto const &vert : _verts)
+    verts.push_back(vert.Ign());
+  this->CopyVertices(verts);
+}
+
+//////////////////////////////////////////////////
+void SubMesh::CopyVertices(const std::vector<ignition::math::Vector3d> &_verts)
 {
   this->vertices.clear();
   this->vertices.resize(_verts.size());
@@ -451,14 +514,23 @@ void SubMesh::CopyVertices(const std::vector<math::Vector3> &_verts)
 
 //////////////////////////////////////////////////
 void SubMesh::CopyNormals(const std::vector<math::Vector3> &_norms)
+{
+  std::vector<ignition::math::Vector3d> norms;
+  for (auto const &norm : _norms)
+    norms.push_back(norm.Ign());
+  this->CopyNormals(norms);
+}
+
+//////////////////////////////////////////////////
+void SubMesh::CopyNormals(const std::vector<ignition::math::Vector3d> &_norms)
 {
   this->normals.clear();
   this->normals.resize(_norms.size());
-  for (unsigned int i = 0; i < _norms.size(); i++)
+  for (unsigned int i = 0; i < _norms.size(); ++i)
   {
     this->normals[i] = _norms[i];
     this->normals[i].Normalize();
-    if (math::equal(this->normals[i].GetLength(), 0.0))
+    if (ignition::math::equal(this->normals[i].Length(), 0.0))
     {
       this->normals[i].Set(0, 0, 1);
     }
@@ -497,6 +569,12 @@ void SubMesh::AddIndex(unsigned int _i)
 
 //////////////////////////////////////////////////
 void SubMesh::AddVertex(const math::Vector3 &_v)
+{
+  this->AddVertex(_v.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::AddVertex(const ignition::math::Vector3d &_v)
 {
   this->vertices.push_back(_v);
 }
@@ -504,11 +582,17 @@ void SubMesh::AddVertex(const math::Vector3 &_v)
 //////////////////////////////////////////////////
 void SubMesh::AddVertex(double _x, double _y, double _z)
 {
-  this->AddVertex(math::Vector3(_x, _y, _z));
+  this->AddVertex(ignition::math::Vector3d(_x, _y, _z));
 }
 
 //////////////////////////////////////////////////
 void SubMesh::AddNormal(const math::Vector3 &_n)
+{
+  this->AddNormal(_n.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::AddNormal(const ignition::math::Vector3d &_n)
 {
   this->normals.push_back(_n);
 }
@@ -516,13 +600,13 @@ void SubMesh::AddNormal(const math::Vector3 &_n)
 //////////////////////////////////////////////////
 void SubMesh::AddNormal(double _x, double _y, double _z)
 {
-  this->AddNormal(math::Vector3(_x, _y, _z));
+  this->AddNormal(ignition::math::Vector3d(_x, _y, _z));
 }
 
 //////////////////////////////////////////////////
 void SubMesh::AddTexCoord(double _u, double _v)
 {
-  this->texCoords.push_back(math::Vector2d(_u, _v));
+  this->texCoords.push_back(ignition::math::Vector2d(_u, _v));
 }
 
 //////////////////////////////////////////////////
@@ -539,6 +623,12 @@ void SubMesh::AddNodeAssignment(unsigned int _vertex, unsigned int _node,
 
 //////////////////////////////////////////////////
 math::Vector3 SubMesh::GetVertex(unsigned int _i) const
+{
+  return this->Vertex(_i);
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d SubMesh::Vertex(unsigned int _i) const
 {
   if (_i >= this->vertices.size())
     gzthrow("Index too large");
@@ -548,6 +638,12 @@ math::Vector3 SubMesh::GetVertex(unsigned int _i) const
 
 //////////////////////////////////////////////////
 void SubMesh::SetVertex(unsigned int _i, const math::Vector3 &_v)
+{
+  this->SetVertex(_i, _v.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::SetVertex(unsigned int _i, const ignition::math::Vector3d &_v)
 {
   if (_i >= this->vertices.size())
     gzthrow("Index too large");
@@ -557,6 +653,12 @@ void SubMesh::SetVertex(unsigned int _i, const math::Vector3 &_v)
 
 //////////////////////////////////////////////////
 math::Vector3 SubMesh::GetNormal(unsigned int _i) const
+{
+  return this->Normal(_i);
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d SubMesh::Normal(unsigned int _i) const
 {
   if (_i >= this->normals.size())
     gzthrow("Index too large");
@@ -566,6 +668,12 @@ math::Vector3 SubMesh::GetNormal(unsigned int _i) const
 
 //////////////////////////////////////////////////
 void SubMesh::SetNormal(unsigned int _i, const math::Vector3 &_n)
+{
+  this->SetNormal(_i, _n.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::SetNormal(unsigned int _i, const ignition::math::Vector3d &_n)
 {
   if (_i >= this->normals.size())
     gzthrow("Index too large");
@@ -575,6 +683,12 @@ void SubMesh::SetNormal(unsigned int _i, const math::Vector3 &_n)
 
 //////////////////////////////////////////////////
 math::Vector2d SubMesh::GetTexCoord(unsigned int _i) const
+{
+  return this->TexCoord(_i);
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector2d SubMesh::TexCoord(unsigned int _i) const
 {
   if (_i >= this->texCoords.size())
     gzthrow("Index too large");
@@ -593,6 +707,12 @@ NodeAssignment SubMesh::GetNodeAssignment(unsigned int _i) const
 
 //////////////////////////////////////////////////
 void SubMesh::SetTexCoord(unsigned int _i, const math::Vector2d &_t)
+{
+  return this->SetTexCoord(_i, _t.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::SetTexCoord(unsigned int _i, const ignition::math::Vector2d &_t)
 {
   if (_i >= this->texCoords.size())
     gzthrow("Index too large");
@@ -612,18 +732,24 @@ unsigned int SubMesh::GetIndex(unsigned int _i) const
 //////////////////////////////////////////////////
 math::Vector3 SubMesh::GetMax() const
 {
-  math::Vector3 max;
-  std::vector<math::Vector3>::const_iterator iter;
+  return this->Max();
+}
 
-  max.x = -FLT_MAX;
-  max.y = -FLT_MAX;
-  max.z = -FLT_MAX;
+//////////////////////////////////////////////////
+ignition::math::Vector3d SubMesh::Max() const
+{
+  ignition::math::Vector3d max;
+  std::vector<ignition::math::Vector3d>::const_iterator iter;
+
+  max.X(-FLT_MAX);
+  max.Y(-FLT_MAX);
+  max.Z(-FLT_MAX);
 
   for (iter = this->vertices.begin(); iter != this->vertices.end(); ++iter)
   {
-    max.x = std::max(max.x, (*iter).x);
-    max.y = std::max(max.y, (*iter).y);
-    max.z = std::max(max.z, (*iter).z);
+    max.X(std::max(max.X(), (*iter).X()));
+    max.Y(std::max(max.Y(), (*iter).Y()));
+    max.Z(std::max(max.Z(), (*iter).Z()));
   }
 
   return max;
@@ -632,18 +758,24 @@ math::Vector3 SubMesh::GetMax() const
 //////////////////////////////////////////////////
 math::Vector3 SubMesh::GetMin() const
 {
-  math::Vector3 min;
-  std::vector<math::Vector3>::const_iterator iter;
+  return this->Min();
+}
 
-  min.x = FLT_MAX;
-  min.y = FLT_MAX;
-  min.z = FLT_MAX;
+//////////////////////////////////////////////////
+ignition::math::Vector3d SubMesh::Min() const
+{
+  ignition::math::Vector3d min;
+  std::vector<ignition::math::Vector3d>::const_iterator iter;
+
+  min.X(FLT_MAX);
+  min.Y(FLT_MAX);
+  min.Z(FLT_MAX);
 
   for (iter = this->vertices.begin(); iter != this->vertices.end(); ++iter)
   {
-    min.x = std::min(min.x, (*iter).x);
-    min.y = std::min(min.y, (*iter).y);
-    min.z = std::min(min.z, (*iter).z);
+    min.X(std::min(min.X(), (*iter).X()));
+    min.Y(std::min(min.Y(), (*iter).Y()));
+    min.Z(std::min(min.Z(), (*iter).Z()));
   }
 
   return min;
@@ -706,7 +838,13 @@ unsigned int SubMesh::GetMaterialIndex() const
 //////////////////////////////////////////////////
 bool SubMesh::HasVertex(const math::Vector3 &_v) const
 {
-  std::vector< math::Vector3 >::const_iterator iter;
+  return this->HasVertex(_v.Ign());
+}
+
+//////////////////////////////////////////////////
+bool SubMesh::HasVertex(const ignition::math::Vector3d &_v) const
+{
+  std::vector< ignition::math::Vector3d >::const_iterator iter;
 
   for (iter = this->vertices.begin(); iter != this->vertices.end(); ++iter)
     if (_v.Equal(*iter))
@@ -718,7 +856,13 @@ bool SubMesh::HasVertex(const math::Vector3 &_v) const
 //////////////////////////////////////////////////
 unsigned int SubMesh::GetVertexIndex(const math::Vector3 &_v) const
 {
-  std::vector< math::Vector3 >::const_iterator iter;
+  return this->GetVertexIndex(_v.Ign());
+}
+
+//////////////////////////////////////////////////
+unsigned int SubMesh::GetVertexIndex(const ignition::math::Vector3d &_v) const
+{
+  std::vector< ignition::math::Vector3d >::const_iterator iter;
 
   for (iter = this->vertices.begin(); iter != this->vertices.end(); ++iter)
     if (_v.Equal(*iter))
@@ -733,8 +877,8 @@ void SubMesh::FillArrays(float **_vertArr, int **_indArr) const
   if (this->vertices.empty() || this->indices.empty())
     gzerr << "No vertices or indices\n";
 
-  std::vector< math::Vector3 >::const_iterator viter;
-  std::vector< unsigned int >::const_iterator iiter;
+  std::vector<ignition::math::Vector3d>::const_iterator viter;
+  std::vector<unsigned int>::const_iterator iiter;
   unsigned int i;
 
   if (*_vertArr)
@@ -749,14 +893,16 @@ void SubMesh::FillArrays(float **_vertArr, int **_indArr) const
   for (viter = this->vertices.begin(), i = 0; viter != this->vertices.end();
       ++viter)
   {
-    (*_vertArr)[i++] = static_cast<float>((*viter).x);
-    (*_vertArr)[i++] = static_cast<float>((*viter).y);
-    (*_vertArr)[i++] = static_cast<float>((*viter).z);
+    (*_vertArr)[i++] = static_cast<float>((*viter).X());
+    (*_vertArr)[i++] = static_cast<float>((*viter).Y());
+    (*_vertArr)[i++] = static_cast<float>((*viter).Z());
   }
 
   for (iiter = this->indices.begin(), i = 0;
       iiter != this->indices.end(); ++iiter)
+  {
     (*_indArr)[i++] = (*iiter);
+  }
 }
 
 //////////////////////////////////////////////////
@@ -767,7 +913,7 @@ void SubMesh::RecalculateNormals()
     return;
 
   // Reset all the normals
-  for (i = 0; i < this->normals.size(); i++)
+  for (i = 0; i < this->normals.size(); ++i)
     this->normals[i].Set(0, 0, 0);
 
   if (this->normals.size() != this->vertices.size())
@@ -776,12 +922,12 @@ void SubMesh::RecalculateNormals()
   // For each face, which is defined by three indices, calculate the normals
   for (i = 0; i < this->indices.size(); i+= 3)
   {
-    math::Vector3 v1 = this->vertices[this->indices[i]];
-    math::Vector3 v2 = this->vertices[this->indices[i+1]];
-    math::Vector3 v3 = this->vertices[this->indices[i+2]];
-    math::Vector3 n = math::Vector3::GetNormal(v1, v2, v3);
+    ignition::math::Vector3d v1 = this->vertices[this->indices[i]];
+    ignition::math::Vector3d v2 = this->vertices[this->indices[i+1]];
+    ignition::math::Vector3d v3 = this->vertices[this->indices[i+2]];
+    ignition::math::Vector3d n = ignition::math::Vector3d::Normal(v1, v2, v3);
 
-    for (unsigned int j = 0; j< this->vertices.size(); j++)
+    for (unsigned int j = 0; j< this->vertices.size(); ++j)
     {
       if (this->vertices[j] == v1 ||
           this->vertices[j] == v2 ||
@@ -793,56 +939,74 @@ void SubMesh::RecalculateNormals()
   }
 
   // Normalize the results
-  for (i = 0; i < this->normals.size(); i++)
+  for (i = 0; i < this->normals.size(); ++i)
   {
     this->normals[i].Normalize();
   }
 }
 
 //////////////////////////////////////////////////
-void Mesh::GetAABB(math::Vector3 &_center, math::Vector3 &_min_xyz,
-                   math::Vector3 &_max_xyz) const
+void Mesh::GetAABB(math::Vector3 &_center, math::Vector3 &_minXYZ,
+                   math::Vector3 &_maxXYZ) const
+{
+  ignition::math::Vector3d center, min, max;
+  this->GetAABB(center, min, max);
+  _center = center;
+  _minXYZ = min;
+  _maxXYZ = max;
+}
+
+//////////////////////////////////////////////////
+void Mesh::GetAABB(ignition::math::Vector3d &_center,
+                   ignition::math::Vector3d &_minXYZ,
+                   ignition::math::Vector3d &_maxXYZ) const
 {
   // find aabb center
-  _min_xyz.x = 1e15;
-  _max_xyz.x = -1e15;
-  _min_xyz.y = 1e15;
-  _max_xyz.y = -1e15;
-  _min_xyz.z = 1e15;
-  _max_xyz.z = -1e15;
-  _center.x = 0;
-  _center.y = 0;
-  _center.z = 0;
+  _minXYZ.X(1e15);
+  _maxXYZ.X(-1e15);
+  _minXYZ.Y(1e15);
+  _maxXYZ.Y(-1e15);
+  _minXYZ.Z(1e15);
+  _maxXYZ.Z(-1e15);
+  _center.X(0);
+  _center.Y(0);
+  _center.Z(0);
 
   std::vector<SubMesh*>::const_iterator siter;
   for (siter = this->submeshes.begin(); siter != this->submeshes.end(); ++siter)
   {
-    math::Vector3 max = (*siter)->GetMax();
-    math::Vector3 min = (*siter)->GetMin();
-    _min_xyz.x = std::min(_min_xyz.x, min.x);
-    _max_xyz.x = std::max(_max_xyz.x, max.x);
-    _min_xyz.y = std::min(_min_xyz.y, min.y);
-    _max_xyz.y = std::max(_max_xyz.y, max.y);
-    _min_xyz.z = std::min(_min_xyz.z, min.z);
-    _max_xyz.z = std::max(_max_xyz.z, max.z);
-  }
-  _center.x = 0.5*(_min_xyz.x+_max_xyz.x);
-  _center.y = 0.5*(_min_xyz.y+_max_xyz.y);
-  _center.z = 0.5*(_min_xyz.z+_max_xyz.z);
-}
+    ignition::math::Vector3d max = (*siter)->Max();
+    ignition::math::Vector3d min = (*siter)->Min();
 
+    _minXYZ.X(std::min(_minXYZ.X(), min.X()));
+    _maxXYZ.X(std::max(_maxXYZ.X(), max.X()));
+    _minXYZ.Y(std::min(_minXYZ.Y(), min.Y()));
+    _maxXYZ.Y(std::max(_maxXYZ.Y(), max.Y()));
+    _minXYZ.Z(std::min(_minXYZ.Z(), min.Z()));
+    _maxXYZ.Z(std::max(_maxXYZ.Z(), max.Z()));
+  }
+  _center.X(0.5 * (_minXYZ.X() + _maxXYZ.X()));
+  _center.Y(0.5 * (_minXYZ.Y() + _maxXYZ.Y()));
+  _center.Z(0.5 * (_minXYZ.Z() + _maxXYZ.Z()));
+}
 
 //////////////////////////////////////////////////
 void SubMesh::GenSphericalTexCoord(const math::Vector3 &_center)
 {
-  std::vector<math::Vector3>::const_iterator viter;
+  this->GenSphericalTexCoord(_center.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::GenSphericalTexCoord(const ignition::math::Vector3d &_center)
+{
+  std::vector<ignition::math::Vector3d>::const_iterator viter;
   for (viter = this->vertices.begin(); viter != this->vertices.end(); ++viter)
   {
     // generate projected texture coordinates, projected from center
     // get x, y, z for computing texture coordinate projections
-    double x = (*viter).x - _center.x;
-    double y = (*viter).y - _center.y;
-    double z = (*viter).z - _center.z;
+    double x = (*viter).X() - _center.X();
+    double y = (*viter).Y() - _center.Y();
+    double z = (*viter).Z() - _center.Z();
 
     double r = std::max(0.000001, sqrt(x*x+y*y+z*z));
     double s = std::min(1.0, std::max(-1.0, z/r));
@@ -856,31 +1020,35 @@ void SubMesh::GenSphericalTexCoord(const math::Vector3 &_center)
 //////////////////////////////////////////////////
 void SubMesh::Scale(double _factor)
 {
-  for (std::vector<math::Vector3>::iterator iter = this->vertices.begin();
-       iter != this->vertices.end(); ++iter)
-  {
-    (*iter) *= _factor;
-  }
+  for (auto &vert : this->vertices)
+    vert *= _factor;
 }
 
 //////////////////////////////////////////////////
 void SubMesh::SetScale(const math::Vector3 &_factor)
 {
-  for (std::vector<math::Vector3>::iterator iter = this->vertices.begin();
-       iter != this->vertices.end(); ++iter)
-  {
-    (*iter).x *= _factor.x;
-    (*iter).y *= _factor.y;
-    (*iter).z *= _factor.z;
-  }
+  this->SetScale(_factor.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::SetScale(const ignition::math::Vector3d &_factor)
+{
+  for (auto &vert : this->vertices)
+    vert *= _factor;
 }
 
 //////////////////////////////////////////////////
 void SubMesh::Center(const math::Vector3 &_center)
 {
-  math::Vector3 min, max, half;
-  min = this->GetMin();
-  max = this->GetMax();
+  this->Center(_center.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::Center(const ignition::math::Vector3d &_center)
+{
+  ignition::math::Vector3d min, max, half;
+  min = this->Min();
+  max = this->Max();
   half = (max - min) * 0.5;
 
   this->Translate(_center - (min + half));
@@ -889,11 +1057,14 @@ void SubMesh::Center(const math::Vector3 &_center)
 //////////////////////////////////////////////////
 void SubMesh::Translate(const math::Vector3 &_vec)
 {
-  for (std::vector<math::Vector3>::iterator iter = this->vertices.begin();
-       iter != this->vertices.end(); ++iter)
-  {
-    (*iter) += _vec;
-  }
+  this->Translate(_vec.Ign());
+}
+
+//////////////////////////////////////////////////
+void SubMesh::Translate(const ignition::math::Vector3d &_vec)
+{
+  for (auto &vert : this->vertices)
+    vert += _vec;
 }
 
 //////////////////////////////////////////////////
