@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 */
 
 #include <tinyxml.h>
+#ifndef _WIN32
 #include <libtar.h>
+#endif
 #include <curl/curl.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -95,8 +97,6 @@ void ModelDatabase::Start(bool _fetchImmediately)
 /////////////////////////////////////////////////
 void ModelDatabase::Fini()
 {
-  this->dataPtr->deprecatedCallbacks.clear();
-
   // Stop the update thread.
   this->dataPtr->stop = true;
   this->dataPtr->updateCacheCompleteCondition.notify_all();
@@ -292,15 +292,8 @@ void ModelDatabase::UpdateModelCache(bool _fetchImmediately)
     else
     {
       boost::mutex::scoped_lock lock2(this->dataPtr->callbacksMutex);
-
-      for (std::list<ModelDatabasePrivate::CallbackFunc>::iterator iter =
-          this->dataPtr->deprecatedCallbacks.begin();
-          iter != this->dataPtr->deprecatedCallbacks.end(); ++iter)
-      {
-        (*iter)(this->dataPtr->modelCache);
-      }
-      this->dataPtr->deprecatedCallbacks.clear();
-
+      if (this->dataPtr->stop)
+        break;
       this->dataPtr->modelDBUpdated(this->dataPtr->modelCache);
     }
     this->dataPtr->updateCacheCompleteCondition.notify_all();
@@ -511,6 +504,7 @@ std::string ModelDatabase::GetModelPath(const std::string &_uri,
         continue;
       }
 
+#ifndef _WIN32
       TAR *tar;
       tar_open(&tar, const_cast<char*>(tarfilename.c_str()),
           NULL, O_RDONLY, 0644, TAR_GNU);
@@ -522,6 +516,7 @@ std::string ModelDatabase::GetModelPath(const std::string &_uri,
       path = outputPath + "/" + modelName;
 
       ModelDatabase::DownloadDependencies(path);
+#endif
     }
 
     curl_easy_cleanup(curl);
