@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,12 @@
  * limitations under the License.
  *
 */
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <math.h>
 
 #include "gazebo/common/Assert.hh"
@@ -33,8 +39,9 @@
 #include "gazebo/rendering/UserCamera.hh"
 #include "gazebo/rendering/OrbitViewController.hh"
 #include "gazebo/rendering/FPSViewController.hh"
-#include "gazebo/rendering/SelectionObj.hh"
 
+#include "gazebo/gui/ModelAlign.hh"
+#include "gazebo/gui/ModelSnap.hh"
 #include "gazebo/gui/ModelManipulator.hh"
 #include "gazebo/gui/MouseEventHandler.hh"
 #include "gazebo/gui/KeyEventHandler.hh"
@@ -56,29 +63,28 @@ GLWidget::GLWidget(QWidget *_parent)
 {
   this->setObjectName("GLWidget");
   this->state = "select";
-  this->sceneCreated = false;
+  this->copyEntityName = "";
+  this->modelEditorEnabled = false;
 
   this->setFocusPolicy(Qt::StrongFocus);
 
   this->windowId = -1;
 
-  setAttribute(Qt::WA_OpaquePaintEvent, true);
-  setAttribute(Qt::WA_PaintOnScreen, true);
+  this->setAttribute(Qt::WA_OpaquePaintEvent, true);
+  this->setAttribute(Qt::WA_PaintOnScreen, true);
 
   this->renderFrame = new QFrame;
   this->renderFrame->setFrameShape(QFrame::NoFrame);
   this->renderFrame->setSizePolicy(QSizePolicy::Expanding,
                                    QSizePolicy::Expanding);
+  this->renderFrame->setContentsMargins(0, 0, 0, 0);
   this->renderFrame->show();
+
   QVBoxLayout *mainLayout = new QVBoxLayout;
   mainLayout->addWidget(this->renderFrame);
   mainLayout->setContentsMargins(0, 0, 0, 0);
   this->setLayout(mainLayout);
 
-  this->connections.push_back(
-      rendering::Events::ConnectCreateScene(
-        boost::bind(&GLWidget::OnCreateScene, this, _1)));
-
   this->connections.push_back(
       rendering::Events::ConnectRemoveScene(
         boost::bind(&GLWidget::OnRemoveScene, this, _1)));
@@ -107,6 +113,10 @@ GLWidget::GLWidget(QWidget *_parent)
      event::Events::ConnectSetSelectedEntity(
        boost::bind(&GLWidget::OnSetSelectedEntity, this, _1, _2)));
 
+  this->connections.push_back(
+      gui::Events::ConnectAlignMode(
+        boost::bind(&GLWidget::OnAlignMode, this, _1, _2, _3, _4)));
+
   this->renderFrame->setMouseTracking(true);
   this->setMouseTracking(true);
 
@@ -117,16 +127,17 @@ GLWidget::GLWidget(QWidget *_parent)
   this->modelPub = this->node->Advertise<msgs::Model>("~/model/modify");
 
   this->factoryPub = this->node->Advertise<msgs::Factory>("~/factory");
-  this->selectionSub = this->node->Subscribe("~/selection",
-      &GLWidget::OnSelectionMsg, this);
+
+  // Publishes information about user selections.
+  this->selectionPub =
+    this->node->Advertise<msgs::Selection>("~/selection");
+
   this->requestSub = this->node->Subscribe("~/request",
       &GLWidget::OnRequest, this);
 
   this->installEventFilter(this);
   this->keyModifiers = 0;
 
-  this->selectedVis.reset();
-
   MouseEventHandler::Instance()->AddPressFilter("glwidget",
       boost::bind(&GLWidget::OnMousePress, this, _1));
 
@@ -138,17 +149,64 @@ GLWidget::GLWidget(QWidget *_parent)
 
   MouseEventHandler::Instance()->AddDoubleClickFilter("glwidget",
       boost::bind(&GLWidget::OnMouseDoubleClick, this, _1));
+
+  connect(g_copyAct, SIGNAL(triggered()), this, SLOT(OnCopy()));
+  connect(g_pasteAct, SIGNAL(triggered()), this, SLOT(OnPaste()));
+
+  connect(g_editModelAct, SIGNAL(toggled(bool)), this,
+      SLOT(OnModelEditor(bool)));
+
+  // Connect the ortho action
+  connect(g_cameraOrthoAct, SIGNAL(triggered()), this,
+          SLOT(OnOrtho()));
+
+  // Connect the perspective action
+  connect(g_cameraPerspectiveAct, SIGNAL(triggered()), this,
+          SLOT(OnPerspective()));
+
+  // Create the scene. This must be done in the constructor so that
+  // we can then create a user camera.
+  this->scene = rendering::create_scene(gui::get_world(), true);
+
+  if (!this->scene)
+  {
+    gzerr << "GLWidget could not create a scene. This will likely result "
+      << "in a blank screen.\n";
+  }
+  else
+  {
+    // This will ultimately create a user camera. We need to create a user
+    // camera in the constructor so that communications (such as via the
+    // ~/gui topic) can work properly (see MainWindow::OnGUI).
+    //
+    // All of this means that we must have a GL Context by this point. So,
+    // we have to create a dummy 1x1 window in RenderEngine::Load.
+    this->OnCreateScene(this->scene->GetName());
+  }
 }
 
 /////////////////////////////////////////////////
 GLWidget::~GLWidget()
 {
+  MouseEventHandler::Instance()->RemovePressFilter("glwidget");
+  MouseEventHandler::Instance()->RemoveReleaseFilter("glwidget");
+  MouseEventHandler::Instance()->RemoveMoveFilter("glwidget");
+  MouseEventHandler::Instance()->RemoveDoubleClickFilter("glwidget");
+
   this->connections.clear();
   this->node.reset();
   this->modelPub.reset();
-  this->selectionSub.reset();
+  this->selectionPub.reset();
+
+  ModelManipulator::Instance()->Clear();
+  ModelSnap::Instance()->Clear();
+  ModelAlign::Instance()->Clear();
+
+  if (this->userCamera)
+    this->userCamera->Fini();
 
   this->userCamera.reset();
+  this->scene.reset();
 }
 
 /////////////////////////////////////////////////
@@ -166,15 +224,25 @@ bool GLWidget::eventFilter(QObject * /*_obj*/, QEvent *_event)
 /////////////////////////////////////////////////
 void GLWidget::showEvent(QShowEvent *_event)
 {
+  // These two functions are most applicable for Linux.
   QApplication::flush();
-  this->windowId = rendering::RenderEngine::Instance()->GetWindowManager()->
-    CreateWindow(this->GetOgreHandle(), this->width(), this->height());
+  QApplication::syncX();
 
+  // Get the window handle in a form that OGRE can use.
+  std::string winHandle = this->GetOgreHandle();
+
+  // Create the OGRE render window
+  this->windowId = rendering::RenderEngine::Instance()->GetWindowManager()->
+    CreateWindow(winHandle, this->width(), this->height());
+
+  // Attach the user camera to the window
+  rendering::RenderEngine::Instance()->GetWindowManager()->SetCamera(
+      this->windowId, this->userCamera);
+
+  // Let QT continue processing the show event.
   QWidget::showEvent(_event);
 
-  if (this->userCamera)
-    rendering::RenderEngine::Instance()->GetWindowManager()->SetCamera(
-        this->windowId, this->userCamera);
+  // Grab focus.
   this->setFocus();
 }
 
@@ -198,11 +266,6 @@ void GLWidget::moveEvent(QMoveEvent *_e)
 /////////////////////////////////////////////////
 void GLWidget::paintEvent(QPaintEvent *_e)
 {
-  // Timing may cause GLWidget to miss the OnCreateScene event. So, we check
-  // here to make sure it's handled.
-  if (!this->sceneCreated && rendering::get_scene())
-    this->OnCreateScene(rendering::get_scene()->GetName());
-
   rendering::UserCameraPtr cam = gui::get_active_camera();
   if (cam && cam->GetInitialized())
   {
@@ -213,6 +276,12 @@ void GLWidget::paintEvent(QPaintEvent *_e)
 
     event::Events::postRender();
   }
+  else
+  {
+    event::Events::preRender();
+  }
+
+  this->update();
 
   _e->accept();
 }
@@ -220,14 +289,13 @@ void GLWidget::paintEvent(QPaintEvent *_e)
 /////////////////////////////////////////////////
 void GLWidget::resizeEvent(QResizeEvent *_e)
 {
-  if (!this->scene)
-    return;
-
   if (this->windowId >= 0)
   {
     rendering::RenderEngine::Instance()->GetWindowManager()->Resize(
         this->windowId, _e->size().width(), _e->size().height());
-    this->userCamera->Resize(_e->size().width(), _e->size().height());
+
+    if (this->userCamera)
+      this->userCamera->Resize(_e->size().width(), _e->size().height());
   }
 }
 
@@ -237,13 +305,14 @@ void GLWidget::keyPressEvent(QKeyEvent *_event)
   if (!this->scene)
     return;
 
-  if (_event->isAutoRepeat())
+  if (_event->isAutoRepeat() && !KeyEventHandler::Instance()->GetAutoRepeat())
     return;
 
   this->keyText = _event->text().toStdString();
   this->keyModifiers = _event->modifiers();
 
   this->keyEvent.key = _event->key();
+  this->keyEvent.text = this->keyText;
 
   // Toggle full screen
   if (_event->key() == Qt::Key_F11)
@@ -254,8 +323,26 @@ void GLWidget::keyPressEvent(QKeyEvent *_event)
 
   // Trigger a model delete if the Delete key was pressed, and a model
   // is currently selected.
-  if (_event->key() == Qt::Key_Delete && this->selectedVis)
-    g_deleteAct->Signal(this->selectedVis->GetName());
+  if (_event->key() == Qt::Key_Delete &&
+      this->selectionLevel == SelectionLevels::MODEL)
+  {
+    boost::mutex::scoped_lock lock(this->selectedVisMutex);
+    while (!this->selectedVisuals.empty())
+    {
+      std::string name = this->selectedVisuals.back()->GetName();
+      int id = this->selectedVisuals.back()->GetId();
+      this->selectedVisuals.pop_back();
+
+      // Publish message about visual deselection
+      msgs::Selection msg;
+      msg.set_id(id);
+      msg.set_name(name);
+      msg.set_selected(false);
+      this->selectionPub->Publish(msg);
+
+      g_deleteAct->Signal(name);
+    }
+  }
 
   if (_event->key() == Qt::Key_Escape)
   {
@@ -267,19 +354,43 @@ void GLWidget::keyPressEvent(QKeyEvent *_event)
     }
   }
 
-  this->mouseEvent.control =
+  this->keyEvent.control =
     this->keyModifiers & Qt::ControlModifier ? true : false;
-  this->mouseEvent.shift =
+  this->keyEvent.shift =
     this->keyModifiers & Qt::ShiftModifier ? true : false;
-  this->mouseEvent.alt =
+  this->keyEvent.alt =
     this->keyModifiers & Qt::AltModifier ? true : false;
 
-  ModelManipulator::Instance()->OnKeyPressEvent(this->keyEvent);
+  this->mouseEvent.SetControl(this->keyEvent.control);
+  this->mouseEvent.SetShift(this->keyEvent.shift);
+  this->mouseEvent.SetAlt(this->keyEvent.alt);
 
-  this->userCamera->HandleKeyPressEvent(this->keyText);
+  if (this->mouseEvent.Control())
+  {
+    if (_event->key() == Qt::Key_C && !this->selectedVisuals.empty()
+       && !this->modelEditorEnabled)
+    {
+      g_copyAct->trigger();
+    }
+    else if (_event->key() == Qt::Key_V && !this->copyEntityName.empty()
+       && !this->modelEditorEnabled)
+    {
+      g_pasteAct->trigger();
+    }
+  }
 
   // Process Key Events
-  KeyEventHandler::Instance()->HandlePress(this->keyEvent);
+  if (!KeyEventHandler::Instance()->HandlePress(this->keyEvent))
+  {
+    // model editor exit pop-up message is modal so can block event propagation.
+    // So using hotkeys to exit will leave the control variable in a bad state.
+    // Manually override and reset the control value.
+    if (this->modelEditorEnabled && this->mouseEvent.Control())
+      this->mouseEvent.SetControl(false);
+
+    ModelManipulator::Instance()->OnKeyPressEvent(this->keyEvent);
+    this->userCamera->HandleKeyPressEvent(this->keyText);
+  }
 }
 
 /////////////////////////////////////////////////
@@ -288,7 +399,8 @@ void GLWidget::keyReleaseEvent(QKeyEvent *_event)
   if (!this->scene)
     return;
 
-  if (_event->isAutoRepeat())
+  // this shouldn't happen, but in case it does...
+  if (_event->isAutoRepeat() && !KeyEventHandler::Instance()->GetAutoRepeat())
     return;
 
   this->keyModifiers = _event->modifiers();
@@ -308,17 +420,22 @@ void GLWidget::keyReleaseEvent(QKeyEvent *_event)
       g_translateAct->trigger();
     else if (_event->key() == Qt::Key_S)
       g_scaleAct->trigger();
+    else if (_event->key() == Qt::Key_N)
+      g_snapAct->trigger();
     else if (_event->key() == Qt::Key_Escape)
       g_arrowAct->trigger();
   }
 
-  this->mouseEvent.control =
+  this->keyEvent.control =
     this->keyModifiers & Qt::ControlModifier ? true : false;
-  this->mouseEvent.shift =
+  this->keyEvent.shift =
     this->keyModifiers & Qt::ShiftModifier ? true : false;
-  this->mouseEvent.alt =
+  this->keyEvent.alt =
     this->keyModifiers & Qt::AltModifier ? true : false;
 
+  this->mouseEvent.SetControl(this->keyEvent.control);
+  this->mouseEvent.SetShift(this->keyEvent.shift);
+  this->mouseEvent.SetAlt(this->keyEvent.alt);
 
   ModelManipulator::Instance()->OnKeyReleaseEvent(this->keyEvent);
   this->keyText = "";
@@ -335,28 +452,18 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *_event)
   if (!this->scene)
     return;
 
-  this->mouseEvent.pressPos.Set(_event->pos().x(), _event->pos().y());
-  this->mouseEvent.prevPos = this->mouseEvent.pressPos;
+  this->mouseEvent.SetPressPos(_event->pos().x(), _event->pos().y());
+  this->mouseEvent.SetPrevPos(this->mouseEvent.PressPos());
 
   /// Set the button which cause the press event
-  if (_event->button() == Qt::LeftButton)
-    this->mouseEvent.button = common::MouseEvent::LEFT;
-  else if (_event->button() == Qt::RightButton)
-    this->mouseEvent.button = common::MouseEvent::RIGHT;
-  else if (_event->button() == Qt::MidButton)
-    this->mouseEvent.button = common::MouseEvent::MIDDLE;
+  this->SetMouseEventButton(_event->button());
 
-  this->mouseEvent.buttons = common::MouseEvent::NO_BUTTON;
-  this->mouseEvent.type = common::MouseEvent::PRESS;
+  this->mouseEvent.SetButtons(common::MouseEvent::NO_BUTTON);
+  this->mouseEvent.SetType(common::MouseEvent::PRESS);
 
-  this->mouseEvent.buttons |= _event->buttons() & Qt::LeftButton ?
-    common::MouseEvent::LEFT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::RightButton ?
-    common::MouseEvent::RIGHT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::MidButton ?
-    common::MouseEvent::MIDDLE : 0x0;
+  this->SetMouseEventButtons(_event->buttons());
 
-  this->mouseEvent.dragging = false;
+  this->mouseEvent.SetDragging(false);
 
   // Process Mouse Events
   MouseEventHandler::Instance()->HandleDoubleClick(this->mouseEvent);
@@ -368,28 +475,18 @@ void GLWidget::mousePressEvent(QMouseEvent *_event)
   if (!this->scene)
     return;
 
-  this->mouseEvent.pressPos.Set(_event->pos().x(), _event->pos().y());
-  this->mouseEvent.prevPos = this->mouseEvent.pressPos;
+  this->mouseEvent.SetPressPos(_event->pos().x(), _event->pos().y());
+  this->mouseEvent.SetPrevPos(this->mouseEvent.PressPos());
 
   /// Set the button which cause the press event
-  if (_event->button() == Qt::LeftButton)
-    this->mouseEvent.button = common::MouseEvent::LEFT;
-  else if (_event->button() == Qt::RightButton)
-    this->mouseEvent.button = common::MouseEvent::RIGHT;
-  else if (_event->button() == Qt::MidButton)
-    this->mouseEvent.button = common::MouseEvent::MIDDLE;
+  this->SetMouseEventButton(_event->button());
 
-  this->mouseEvent.buttons = common::MouseEvent::NO_BUTTON;
-  this->mouseEvent.type = common::MouseEvent::PRESS;
+  this->mouseEvent.SetButtons(common::MouseEvent::NO_BUTTON);
+  this->mouseEvent.SetType(common::MouseEvent::PRESS);
 
-  this->mouseEvent.buttons |= _event->buttons() & Qt::LeftButton ?
-    common::MouseEvent::LEFT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::RightButton ?
-    common::MouseEvent::RIGHT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::MidButton ?
-    common::MouseEvent::MIDDLE : 0x0;
+  this->SetMouseEventButtons(_event->buttons());
 
-  this->mouseEvent.dragging = false;
+  this->mouseEvent.SetDragging(false);
 
   // Process Mouse Events
   MouseEventHandler::Instance()->HandlePress(this->mouseEvent);
@@ -405,6 +502,8 @@ bool GLWidget::OnMousePress(const common::MouseEvent & /*_event*/)
   else if (this->state == "translate" || this->state == "rotate"
       || this->state == "scale")
     ModelManipulator::Instance()->OnMousePressEvent(this->mouseEvent);
+  else if (this->state == "snap")
+    ModelSnap::Instance()->OnMousePressEvent(this->mouseEvent);
 
   return true;
 }
@@ -419,6 +518,8 @@ bool GLWidget::OnMouseRelease(const common::MouseEvent & /*_event*/)
   else if (this->state == "translate" || this->state == "rotate"
       || this->state == "scale")
     ModelManipulator::Instance()->OnMouseReleaseEvent(this->mouseEvent);
+  else if (this->state == "snap")
+    ModelSnap::Instance()->OnMouseReleaseEvent(this->mouseEvent);
 
   return true;
 }
@@ -434,6 +535,8 @@ bool GLWidget::OnMouseMove(const common::MouseEvent & /*_event*/)
   else if (this->state == "translate" || this->state == "rotate"
       || this->state == "scale")
     ModelManipulator::Instance()->OnMouseMoveEvent(this->mouseEvent);
+  else if (this->state == "snap")
+    ModelSnap::Instance()->OnMouseMoveEvent(this->mouseEvent);
 
   return true;
 }
@@ -441,7 +544,9 @@ bool GLWidget::OnMouseMove(const common::MouseEvent & /*_event*/)
 /////////////////////////////////////////////////
 bool GLWidget::OnMouseDoubleClick(const common::MouseEvent & /*_event*/)
 {
-  rendering::VisualPtr vis = this->userCamera->GetVisual(this->mouseEvent.pos);
+  rendering::VisualPtr vis =
+    this->userCamera->GetVisual(this->mouseEvent.Pos());
+
   if (vis && gui::get_entity_id(vis->GetRootVisual()->GetName()))
   {
     if (vis->IsPlane())
@@ -449,7 +554,7 @@ bool GLWidget::OnMouseDoubleClick(const common::MouseEvent & /*_event*/)
       math::Pose pose, camPose;
       camPose = this->userCamera->GetWorldPose();
       if (this->scene->GetFirstContact(this->userCamera,
-                                   this->mouseEvent.pos, pose.pos))
+            this->mouseEvent.Pos(), pose.pos))
       {
         this->userCamera->SetFocalPoint(pose.pos);
         math::Vector3 dir = pose.pos - camPose.pos;
@@ -475,7 +580,8 @@ void GLWidget::OnMousePressNormal()
   if (!this->userCamera)
     return;
 
-  rendering::VisualPtr vis = this->userCamera->GetVisual(this->mouseEvent.pos);
+  rendering::VisualPtr vis = this->userCamera->GetVisual(
+      this->mouseEvent.Pos());
 
   this->userCamera->HandleMouseEvent(this->mouseEvent);
 }
@@ -493,14 +599,14 @@ void GLWidget::wheelEvent(QWheelEvent *_event)
   if (!this->scene)
     return;
 
-  this->mouseEvent.scroll.y = _event->delta() > 0 ? -1 : 1;
-  this->mouseEvent.type = common::MouseEvent::SCROLL;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::LeftButton ?
-    common::MouseEvent::LEFT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::RightButton ?
-    common::MouseEvent::RIGHT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::MidButton ?
-    common::MouseEvent::MIDDLE : 0x0;
+  if (_event->delta() > 0)
+    this->mouseEvent.SetScroll(this->mouseEvent.Scroll().X(), -1);
+  else
+    this->mouseEvent.SetScroll(this->mouseEvent.Scroll().X(), 1);
+
+  this->mouseEvent.SetType(common::MouseEvent::SCROLL);
+
+  this->SetMouseEventButtons(_event->buttons());
 
   this->userCamera->HandleMouseEvent(this->mouseEvent);
 }
@@ -513,24 +619,20 @@ void GLWidget::mouseMoveEvent(QMouseEvent *_event)
 
   this->setFocus(Qt::MouseFocusReason);
 
-  this->mouseEvent.pos.Set(_event->pos().x(), _event->pos().y());
-  this->mouseEvent.type = common::MouseEvent::MOVE;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::LeftButton ?
-    common::MouseEvent::LEFT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::RightButton ?
-    common::MouseEvent::RIGHT : 0x0;
-  this->mouseEvent.buttons |= _event->buttons() & Qt::MidButton ?
-    common::MouseEvent::MIDDLE : 0x0;
+  this->mouseEvent.SetPos(_event->pos().x(), _event->pos().y());
+  this->mouseEvent.SetType(common::MouseEvent::MOVE);
+
+  this->SetMouseEventButtons(_event->buttons());
 
   if (_event->buttons())
-    this->mouseEvent.dragging = true;
+    this->mouseEvent.SetDragging(true);
   else
-    this->mouseEvent.dragging = false;
+    this->mouseEvent.SetDragging(false);
 
   // Process Mouse Events
   MouseEventHandler::Instance()->HandleMove(this->mouseEvent);
 
-  this->mouseEvent.prevPos = this->mouseEvent.pos;
+  this->mouseEvent.SetPrevPos(this->mouseEvent.Pos());
 }
 
 /////////////////////////////////////////////////
@@ -538,7 +640,7 @@ void GLWidget::OnMouseMoveMakeEntity()
 {
   if (this->entityMaker)
   {
-    if (this->mouseEvent.dragging)
+    if (this->mouseEvent.Dragging())
       this->entityMaker->OnMouseDrag(this->mouseEvent);
     else
       this->entityMaker->OnMouseMove(this->mouseEvent);
@@ -551,7 +653,8 @@ void GLWidget::OnMouseMoveNormal()
   if (!this->userCamera)
     return;
 
-  rendering::VisualPtr vis = this->userCamera->GetVisual(this->mouseEvent.pos);
+  rendering::VisualPtr vis = this->userCamera->GetVisual(
+      this->mouseEvent.Pos());
 
   if (vis && !vis->IsPlane())
     QApplication::setOverrideCursor(Qt::PointingHandCursor);
@@ -567,27 +670,15 @@ void GLWidget::mouseReleaseEvent(QMouseEvent *_event)
   if (!this->scene)
     return;
 
-  this->mouseEvent.pos.Set(_event->pos().x(), _event->pos().y());
-  this->mouseEvent.prevPos = this->mouseEvent.pos;
+  this->mouseEvent.SetPos(_event->pos().x(), _event->pos().y());
+  this->mouseEvent.SetPrevPos(this->mouseEvent.Pos());
 
-  if (_event->button() == Qt::LeftButton)
-    this->mouseEvent.button = common::MouseEvent::LEFT;
-  else if (_event->button() == Qt::RightButton)
-    this->mouseEvent.button = common::MouseEvent::RIGHT;
-  else if (_event->button() == Qt::MidButton)
-    this->mouseEvent.button = common::MouseEvent::MIDDLE;
+  this->SetMouseEventButton(_event->button());
 
-  this->mouseEvent.buttons = common::MouseEvent::NO_BUTTON;
-  this->mouseEvent.type = common::MouseEvent::RELEASE;
+  this->mouseEvent.SetButtons(common::MouseEvent::NO_BUTTON);
+  this->mouseEvent.SetType(common::MouseEvent::RELEASE);
 
-  this->mouseEvent.buttons |= _event->buttons() & Qt::LeftButton ?
-    common::MouseEvent::LEFT : 0x0;
-
-  this->mouseEvent.buttons |= _event->buttons() & Qt::RightButton ?
-    common::MouseEvent::RIGHT : 0x0;
-
-  this->mouseEvent.buttons |= _event->buttons() & Qt::MidButton ?
-    common::MouseEvent::MIDDLE : 0x0;
+  this->SetMouseEventButtons(_event->buttons());
 
   // Process Mouse Events
   MouseEventHandler::Instance()->HandleRelease(this->mouseEvent);
@@ -608,21 +699,83 @@ void GLWidget::OnMouseReleaseNormal()
   if (!this->userCamera)
     return;
 
-  if (!this->mouseEvent.dragging)
+  if (!this->mouseEvent.Dragging())
   {
     rendering::VisualPtr vis =
-      this->userCamera->GetVisual(this->mouseEvent.pos);
+      this->userCamera->GetVisual(this->mouseEvent.Pos());
+
     if (vis)
     {
-      if (this->mouseEvent.button == common::MouseEvent::RIGHT)
+      rendering::VisualPtr selectVis;
+      rendering::VisualPtr linkVis = vis->GetParent();
+      if (!linkVis)
       {
-        g_modelRightMenu->Run(vis->GetName(), QCursor::pos());
+        gzerr << "Link visual not found, this should not happen." << std::endl;
+        return;
       }
-      else if (this->mouseEvent.button == common::MouseEvent::LEFT)
+      rendering::VisualPtr modelVis = vis->GetRootVisual();
+      if (!modelVis)
       {
-        vis = vis->GetRootVisual();
-        this->SetSelectedVisual(vis);
-        event::Events::setSelectedEntity(vis->GetName(), "normal");
+        gzerr << "Model visual not found, this should not happen." << std::endl;
+        return;
+      }
+
+      // Flags to check if we should select a link or a model
+      bool rightButton = (this->mouseEvent.Button() ==
+          common::MouseEvent::RIGHT);
+      bool modelHighlighted = modelVis->GetHighlighted();
+      int linkCount = 0;
+      bool linkHighlighted = false;
+      for (unsigned int i = 0; i < modelVis->GetChildCount(); ++i)
+      {
+        // Find out if there's only one link in the model
+        uint32_t flags = modelVis->GetChild(i)->GetVisibilityFlags();
+        if ((flags != GZ_VISIBILITY_ALL) && (flags & GZ_VISIBILITY_GUI))
+        {
+          continue;
+        }
+        linkCount++;
+
+        // A link from the same model is currently selected
+        if (modelVis->GetChild(i)->GetHighlighted())
+        {
+          linkHighlighted = true;
+        }
+      }
+
+      // Select link
+      if (linkCount > 1 && !this->mouseEvent.Control() &&
+          ((modelHighlighted && !rightButton) || linkHighlighted))
+      {
+        selectVis = linkVis;
+        this->selectionLevel = SelectionLevels::LINK;
+      }
+      // Select model
+      else
+      {
+        // Can't select a link and a model at the same time
+        if (this->selectionLevel == SelectionLevels::LINK)
+          this->DeselectAllVisuals();
+
+        selectVis = modelVis;
+        this->selectionLevel = SelectionLevels::MODEL;
+      }
+      this->SetSelectedVisual(selectVis);
+      event::Events::setSelectedEntity(selectVis->GetName(), "normal");
+
+      // Open context menu
+      if (rightButton)
+      {
+        if (selectVis == modelVis)
+        {
+          g_modelRightMenu->Run(selectVis->GetName(), QCursor::pos(),
+              ModelRightMenu::EntityTypes::MODEL);
+        }
+        else if (selectVis == linkVis)
+        {
+          g_modelRightMenu->Run(selectVis->GetName(), QCursor::pos(),
+              ModelRightMenu::EntityTypes::LINK);
+        }
       }
     }
     else
@@ -669,9 +822,14 @@ void GLWidget::ViewScene(rendering::ScenePtr _scene)
     gzerr << "Unable to connect to a running Gazebo master.\n";
 
   if (_scene->GetUserCameraCount() == 0)
-    this->userCamera = _scene->CreateUserCamera(cameraName);
+  {
+    this->userCamera = _scene->CreateUserCamera(cameraName,
+        gazebo::gui::getINIProperty<int>("rendering.stereo", 0));
+  }
   else
+  {
     this->userCamera = _scene->GetUserCamera(0);
+  }
 
   gui::set_active_camera(this->userCamera);
   this->scene = _scene;
@@ -683,14 +841,8 @@ void GLWidget::ViewScene(rendering::ScenePtr _scene)
   double yaw = atan2(delta.y, delta.x);
 
   double pitch = atan2(-delta.z, sqrt(delta.x*delta.x + delta.y*delta.y));
-  this->userCamera->SetWorldPose(math::Pose(camPos,
+  this->userCamera->SetDefaultPose(math::Pose(camPos,
         math::Vector3(0, pitch, yaw)));
-
-  if (this->windowId >= 0)
-  {
-    rendering::RenderEngine::Instance()->GetWindowManager()->SetCamera(
-        this->windowId, this->userCamera);
-  }
 }
 
 /////////////////////////////////////////////////
@@ -706,11 +858,9 @@ void GLWidget::Clear()
   this->userCamera.reset();
   this->scene.reset();
   this->SetSelectedVisual(rendering::VisualPtr());
-  this->hoverVis.reset();
   this->keyModifiers = 0;
 }
 
-
 //////////////////////////////////////////////////
 rendering::UserCameraPtr GLWidget::GetCamera() const
 {
@@ -722,20 +872,20 @@ std::string GLWidget::GetOgreHandle() const
 {
   std::string ogreHandle;
 
-#if defined(WIN32) || defined(__APPLE__)
-  ogreHandle = boost::lexical_cast<std::string>(this->winId());
+#if defined(__APPLE__)
+  ogreHandle = std::to_string(this->winId());
+#elif defined(WIN32)
+  ogreHandle = std::to_string(
+      reinterpret_cast<uint32_t>(this->renderFrame->winId()));
 #else
   QX11Info info = x11Info();
   QWidget *q_parent = dynamic_cast<QWidget*>(this->renderFrame);
-  ogreHandle = boost::lexical_cast<std::string>(
-      reinterpret_cast<uint64_t>(info.display()));
-  ogreHandle += ":";
-  ogreHandle += boost::lexical_cast<std::string>(
-      static_cast<uint32_t>(info.screen()));
-  ogreHandle += ":";
   GZ_ASSERT(q_parent, "q_parent is null");
-  ogreHandle += boost::lexical_cast<std::string>(
-      static_cast<uint64_t>(q_parent->winId()));
+
+  ogreHandle =
+    std::to_string(reinterpret_cast<uint64_t>(info.display())) + ":" +
+    std::to_string(static_cast<uint32_t>(info.screen())) + ":" +
+    std::to_string(static_cast<uint64_t>(q_parent->winId()));
 #endif
 
   return ogreHandle;
@@ -753,14 +903,13 @@ void GLWidget::OnRemoveScene(const std::string &_name)
 /////////////////////////////////////////////////
 void GLWidget::OnCreateScene(const std::string &_name)
 {
-  this->hoverVis.reset();
   this->SetSelectedVisual(rendering::VisualPtr());
 
   this->ViewScene(rendering::get_scene(_name));
 
   ModelManipulator::Instance()->Init();
-
-  this->sceneCreated = true;
+  ModelSnap::Instance()->Init();
+  ModelAlign::Instance()->Init();
 }
 
 /////////////////////////////////////////////////
@@ -846,35 +995,80 @@ void GLWidget::OnOrbit()
 }
 
 /////////////////////////////////////////////////
-void GLWidget::OnSelectionMsg(ConstSelectionPtr &_msg)
+std::vector<rendering::VisualPtr> GLWidget::SelectedVisuals() const
 {
-  if (_msg->has_selected())
-  {
-    if (_msg->selected())
-    {
-      this->SetSelectedVisual(this->scene->GetVisual(_msg->name()));
-    }
-    else
-    {
-      this->SetSelectedVisual(rendering::VisualPtr());
-    }
-  }
+  return this->selectedVisuals;
 }
 
 /////////////////////////////////////////////////
 void GLWidget::SetSelectedVisual(rendering::VisualPtr _vis)
 {
-  if (this->selectedVis)
+  // deselect all if not in multi-selection mode.
+  if (!this->mouseEvent.Control())
   {
-    this->selectedVis->SetHighlighted(false);
+    this->DeselectAllVisuals();
   }
 
-  this->selectedVis = _vis;
+  boost::mutex::scoped_lock lock(this->selectedVisMutex);
 
-  if (this->selectedVis && !this->selectedVis->IsPlane())
+  msgs::Selection msg;
+
+  if (_vis && !_vis->IsPlane())
   {
-    this->selectedVis->SetHighlighted(true);
+    if (_vis == _vis->GetRootVisual())
+      this->selectionLevel = SelectionLevels::MODEL;
+    else
+      this->selectionLevel = SelectionLevels::LINK;
+
+    _vis->SetHighlighted(true);
+
+    // enable multi-selection if control is pressed
+    if (this->selectedVisuals.empty() || this->mouseEvent.Control())
+    {
+      std::vector<rendering::VisualPtr>::iterator it =
+        std::find(this->selectedVisuals.begin(),
+            this->selectedVisuals.end(), _vis);
+      if (it == this->selectedVisuals.end())
+        this->selectedVisuals.push_back(_vis);
+      else
+      {
+        // if element already exists, move to the back of vector
+        rendering::VisualPtr vis = (*it);
+        this->selectedVisuals.erase(it);
+        this->selectedVisuals.push_back(vis);
+      }
+    }
+    g_copyAct->setEnabled(true);
+
+    msg.set_id(_vis->GetId());
+    msg.set_name(_vis->GetName());
+    msg.set_selected(true);
+    this->selectionPub->Publish(msg);
   }
+  else if (g_copyAct)
+  {
+    g_copyAct->setEnabled(false);
+  }
+
+  if (g_alignAct)
+    g_alignAct->setEnabled(this->selectedVisuals.size() > 1);
+}
+
+/////////////////////////////////////////////////
+void GLWidget::DeselectAllVisuals()
+{
+  boost::mutex::scoped_lock lock(this->selectedVisMutex);
+
+  msgs::Selection msg;
+  for (unsigned int i = 0; i < this->selectedVisuals.size(); ++i)
+  {
+    this->selectedVisuals[i]->SetHighlighted(false);
+    msg.set_id(this->selectedVisuals[i]->GetId());
+    msg.set_name(this->selectedVisuals[i]->GetName());
+    msg.set_selected(false);
+    this->selectionPub->Publish(msg);
+  }
+  this->selectedVisuals.clear();
 }
 
 /////////////////////////////////////////////////
@@ -882,38 +1076,98 @@ void GLWidget::OnManipMode(const std::string &_mode)
 {
   this->state = _mode;
 
-  if (this->selectedVis)
-    ModelManipulator::Instance()->SetAttachedVisual(this->selectedVis);
+  if (!this->selectedVisuals.empty())
+  {
+    boost::mutex::scoped_lock lock(this->selectedVisMutex);
+    ModelManipulator::Instance()->SetAttachedVisual(
+        this->selectedVisuals.back());
+  }
 
   ModelManipulator::Instance()->SetManipulationMode(_mode);
+  ModelSnap::Instance()->Reset();
+
+  if (this->state != "select")
+  {
+    boost::mutex::scoped_lock lock(this->selectedVisMutex);
+    // only support multi-model selection in select mode for now.
+    // deselect 0 to n-1 models.
+    if (this->selectedVisuals.size() > 1)
+    {
+      for (std::vector<rendering::VisualPtr>::iterator it
+          = this->selectedVisuals.begin(); it != --this->selectedVisuals.end();)
+      {
+         (*it)->SetHighlighted(false);
+         it = this->selectedVisuals.erase(it);
+      }
+    }
+  }
 }
 
 /////////////////////////////////////////////////
-void GLWidget::Paste(const std::string &_object)
+void GLWidget::OnCopy()
 {
-  if (!_object.empty())
+  boost::mutex::scoped_lock lock(this->selectedVisMutex);
+  if (!this->selectedVisuals.empty() && !this->modelEditorEnabled)
   {
-    this->ClearSelection();
-    if (this->entityMaker)
-      this->entityMaker->Stop();
+    this->Copy(this->selectedVisuals.back()->GetName());
+  }
+}
 
-    // \todo Put this back in when pasting is enabled again
-    // this->modelMaker.InitFromModel(_object);
-    this->entityMaker = &this->modelMaker;
-    this->entityMaker->Start(this->userCamera);
-    gui::Events::manipMode("make_entity");
+/////////////////////////////////////////////////
+void GLWidget::OnPaste()
+{
+  if (!this->modelEditorEnabled)
+    this->Paste(this->copyEntityName);
+}
+
+/////////////////////////////////////////////////
+void GLWidget::Copy(const std::string &_name)
+{
+  this->copyEntityName = _name;
+  g_pasteAct->setEnabled(true);
+}
+
+/////////////////////////////////////////////////
+void GLWidget::Paste(const std::string &_name)
+{
+  if (!_name.empty())
+  {
+    bool isModel = false;
+    bool isLight = false;
+    if (scene->GetLight(_name))
+      isLight = true;
+    else if (scene->GetVisual(_name))
+      isModel = true;
+
+    if (isLight || isModel)
+    {
+      this->ClearSelection();
+      if (this->entityMaker)
+        this->entityMaker->Stop();
+
+      if (isLight && this->lightMaker.InitFromLight(_name))
+      {
+        this->entityMaker = &this->lightMaker;
+        this->entityMaker->Start(this->userCamera);
+        // this makes the entity appear at the mouse cursor
+        this->entityMaker->OnMouseMove(this->mouseEvent);
+        gui::Events::manipMode("make_entity");
+      }
+      else if (isModel && this->modelMaker.InitFromModel(_name))
+      {
+        this->entityMaker = &this->modelMaker;
+        this->entityMaker->Start(this->userCamera);
+        // this makes the entity appear at the mouse cursor
+        this->entityMaker->OnMouseMove(this->mouseEvent);
+        gui::Events::manipMode("make_entity");
+      }
+    }
   }
 }
 
 /////////////////////////////////////////////////
 void GLWidget::ClearSelection()
 {
-  if (this->hoverVis)
-  {
-    this->hoverVis->SetEmissive(common::Color(0, 0, 0));
-    this->hoverVis.reset();
-  }
-
   this->SetSelectedVisual(rendering::VisualPtr());
 
   this->scene->SelectVisual("", "normal");
@@ -922,23 +1176,30 @@ void GLWidget::ClearSelection()
 /////////////////////////////////////////////////
 void GLWidget::OnSetSelectedEntity(const std::string &_name,
                                    const std::string &_mode)
-
 {
   if (!_name.empty())
   {
     std::string name = _name;
     boost::replace_first(name, gui::get_world()+"::", "");
 
-    this->SetSelectedVisual(this->scene->GetVisual(name));
-    this->scene->SelectVisual(name, _mode);
+    rendering::VisualPtr selection = this->scene->GetVisual(name);
+
+    std::vector<rendering::VisualPtr>::iterator it =
+      std::find(this->selectedVisuals.begin(),
+          this->selectedVisuals.end(), selection);
+
+    // Shortcircuit the case when GLWidget already selected the visual.
+    if (it == this->selectedVisuals.end() || _name != (*it)->GetName())
+    {
+      this->SetSelectedVisual(selection);
+      this->scene->SelectVisual(name, _mode);
+    }
   }
-  else
+  else if (!this->selectedVisuals.empty())
   {
     this->SetSelectedVisual(rendering::VisualPtr());
     this->scene->SelectVisual("", _mode);
   }
-
-  this->hoverVis.reset();
 }
 
 /////////////////////////////////////////////////
@@ -961,7 +1222,7 @@ void GLWidget::PopHistory()
     msg.set_id(gui::get_entity_id(this->moveHistory.back().first));
     msg.set_name(this->moveHistory.back().first);
 
-    msgs::Set(msg.mutable_pose(), this->moveHistory.back().second);
+    msgs::Set(msg.mutable_pose(), this->moveHistory.back().second.Ign());
     this->scene->GetVisual(this->moveHistory.back().first)->SetWorldPose(
         this->moveHistory.back().second);
 
@@ -976,10 +1237,114 @@ void GLWidget::OnRequest(ConstRequestPtr &_msg)
 {
   if (_msg->request() == "entity_delete")
   {
-    if (this->selectedVis && this->selectedVis->GetName() == _msg->data())
+    boost::mutex::scoped_lock lock(this->selectedVisMutex);
+    if (!this->selectedVisuals.empty())
     {
-      this->selectedVis.reset();
-      this->SetSelectedVisual(rendering::VisualPtr());
+      for (std::vector<rendering::VisualPtr>::iterator it =
+          this->selectedVisuals.begin(); it != this->selectedVisuals.end();
+          ++it)
+      {
+        if ((*it)->GetName() == _msg->data())
+        {
+          ModelManipulator::Instance()->Detach();
+          this->selectedVisuals.erase(it);
+          break;
+        }
+      }
+    }
+
+    if (this->copyEntityName == _msg->data())
+    {
+      this->copyEntityName = "";
+      g_pasteAct->setEnabled(false);
     }
   }
 }
+
+/////////////////////////////////////////////////
+void GLWidget::OnAlignMode(const std::string &_axis, const std::string &_config,
+    const std::string &_target, bool _preview)
+{
+  ModelAlign::Instance()->AlignVisuals(this->selectedVisuals, _axis, _config,
+      _target, !_preview);
+}
+
+/////////////////////////////////////////////////
+void GLWidget::OnModelEditor(bool _checked)
+{
+  this->modelEditorEnabled = _checked;
+  g_arrowAct->trigger();
+  event::Events::setSelectedEntity("", "normal");
+
+  // Manually deselect, in case the editor was opened with Ctrl
+  this->DeselectAllVisuals();
+}
+
+/////////////////////////////////////////////////
+void GLWidget::OnOrtho()
+{
+  // Disable view control options when in ortho projection
+  g_fpsAct->setEnabled(false);
+  g_orbitAct->setEnabled(false);
+  this->userCamera->SetProjectionType("orthographic");
+}
+
+/////////////////////////////////////////////////
+void GLWidget::OnPerspective()
+{
+  // Enable view control options when in perspective projection
+  g_fpsAct->setEnabled(true);
+  g_orbitAct->setEnabled(true);
+  this->userCamera->SetProjectionType("perspective");
+}
+
+/////////////////////////////////////////////////
+QPaintEngine *GLWidget::paintEngine() const
+{
+  return NULL;
+}
+
+/////////////////////////////////////////////////
+void GLWidget::SetMouseEventButtons(const Qt::MouseButtons &_buttons)
+{
+  if (_buttons & Qt::LeftButton)
+  {
+    this->mouseEvent.SetButtons(
+        this->mouseEvent.Buttons() | common::MouseEvent::LEFT);
+  }
+  else
+  {
+    this->mouseEvent.SetButtons(this->mouseEvent.Buttons() | 0x0);
+  }
+
+  if (_buttons & Qt::RightButton)
+  {
+    this->mouseEvent.SetButtons(
+        this->mouseEvent.Buttons() | common::MouseEvent::RIGHT);
+  }
+  else
+  {
+    this->mouseEvent.SetButtons(this->mouseEvent.Buttons() | 0x0);
+  }
+
+  if (_buttons & Qt::MidButton)
+  {
+    this->mouseEvent.SetButtons(
+        this->mouseEvent.Buttons() | common::MouseEvent::MIDDLE);
+  }
+  else
+  {
+    this->mouseEvent.SetButtons(this->mouseEvent.Buttons() | 0x0);
+  }
+}
+
+/////////////////////////////////////////////////
+void GLWidget::SetMouseEventButton(const Qt::MouseButton &_button)
+{
+  if (_button == Qt::LeftButton)
+    this->mouseEvent.SetButton(common::MouseEvent::LEFT);
+  else if (_button == Qt::RightButton)
+    this->mouseEvent.SetButton(common::MouseEvent::RIGHT);
+  else if (_button == Qt::MidButton)
+    this->mouseEvent.SetButton(common::MouseEvent::MIDDLE);
+}
