@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,13 @@
  * limitations under the License.
  *
  */
+
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include "gazebo/transport/Node.hh"
 #include "gazebo/transport/TransportIface.hh"
 #include "gazebo/gui/GuiIface.hh"
@@ -28,7 +35,7 @@ JointForceControl::JointForceControl(const std::string &_name,
   : QWidget(_parent), name(_name)
 {
   this->forceSpin = new QDoubleSpinBox;
-  this->forceSpin->setRange(-1000.0, 1000.0);
+  this->forceSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
   this->forceSpin->setSingleStep(0.001);
   this->forceSpin->setDecimals(3);
   this->forceSpin->setValue(0.000);
@@ -63,25 +70,25 @@ JointPIDPosControl::JointPIDPosControl(const std::string &_name,
   : QWidget(_parent), name(_name)
 {
   this->posSpin = new QDoubleSpinBox;
-  this->posSpin->setRange(-1000, 1000);
+  this->posSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
   this->posSpin->setSingleStep(0.001);
   this->posSpin->setDecimals(3);
   this->posSpin->setValue(0.000);
 
   this->pGainSpin = new QDoubleSpinBox;
-  this->pGainSpin->setMinimum(0.0);
+  this->pGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
   this->pGainSpin->setSingleStep(0.01);
   this->pGainSpin->setDecimals(3);
   this->pGainSpin->setValue(1.000);
 
   this->iGainSpin = new QDoubleSpinBox;
-  this->iGainSpin->setMinimum(0.0);
+  this->iGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
   this->iGainSpin->setSingleStep(0.01);
   this->iGainSpin->setDecimals(3);
   this->iGainSpin->setValue(0.100);
 
   this->dGainSpin = new QDoubleSpinBox;
-  this->dGainSpin->setMinimum(0.0);
+  this->dGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
   this->dGainSpin->setSingleStep(0.01);
   this->dGainSpin->setDecimals(3);
   this->dGainSpin->setValue(0.010);
@@ -165,28 +172,28 @@ JointPIDVelControl::JointPIDVelControl(const std::string &_name,
   : QWidget(_parent), name(_name)
 {
   this->posSpin = new QDoubleSpinBox;
-  posSpin->setRange(-1000, 1000);
-  posSpin->setSingleStep(0.001);
-  posSpin->setDecimals(3);
-  posSpin->setValue(0.000);
+  this->posSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
+  this->posSpin->setSingleStep(0.001);
+  this->posSpin->setDecimals(3);
+  this->posSpin->setValue(0.000);
 
   this->pGainSpin = new QDoubleSpinBox;
-  pGainSpin->setMinimum(0.0);
-  pGainSpin->setSingleStep(0.01);
-  pGainSpin->setDecimals(3);
-  pGainSpin->setValue(1.000);
+  this->pGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
+  this->pGainSpin->setSingleStep(0.01);
+  this->pGainSpin->setDecimals(3);
+  this->pGainSpin->setValue(1.000);
 
   this->iGainSpin = new QDoubleSpinBox;
-  iGainSpin->setMinimum(0.0);
-  iGainSpin->setSingleStep(0.01);
-  iGainSpin->setDecimals(3);
-  iGainSpin->setValue(0.100);
+  this->iGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
+  this->iGainSpin->setSingleStep(0.01);
+  this->iGainSpin->setDecimals(3);
+  this->iGainSpin->setValue(0.100);
 
   this->dGainSpin = new QDoubleSpinBox;
-  dGainSpin->setMinimum(0.0);
-  dGainSpin->setSingleStep(0.01);
-  dGainSpin->setDecimals(3);
-  dGainSpin->setValue(0.010);
+  this->dGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
+  this->dGainSpin->setSingleStep(0.01);
+  this->dGainSpin->setDecimals(3);
+  this->dGainSpin->setValue(0.010);
 
   _layout->addWidget(posSpin, _index, 2);
   _layout->addWidget(pGainSpin, _index, 3);
