@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,13 @@
  * limitations under the License.
  *
  */
+
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <google/protobuf/descriptor.h>
 #include <google/protobuf/message.h>
 
@@ -93,6 +100,8 @@ ModelListWidget::ModelListWidget(QWidget *_parent)
   this->variantManager = new QtVariantPropertyManager();
   this->propTreeBrowser = new QtTreePropertyBrowser();
   this->propTreeBrowser->setObjectName("propTreeBrowser");
+  this->propTreeBrowser->setStyleSheet(
+      "QTreeView::branch:selected:active { background-color: transparent; }");
   this->variantFactory = new QtVariantEditorFactory();
   this->propTreeBrowser->setFactoryForManager(this->variantManager,
                                               this->variantFactory);
@@ -129,6 +138,10 @@ ModelListWidget::ModelListWidget(QWidget *_parent)
       gui::Events::ConnectModelUpdate(
         boost::bind(&ModelListWidget::OnModelUpdate, this, _1)));
 
+  this->connections.push_back(
+      gui::Events::ConnectLightUpdate(
+        boost::bind(&ModelListWidget::OnLightUpdate, this, _1)));
+
   this->connections.push_back(
       rendering::Events::ConnectCreateScene(
         boost::bind(&ModelListWidget::OnCreateScene, this, _1)));
@@ -185,7 +198,42 @@ void ModelListWidget::OnModelSelection(QTreeWidgetItem *_item, int /*_column*/)
                                              this->selectedEntityName);
       this->requestPub->Publish(*this->requestMsg);
     }
+    else if (name == "GUI")
+    {
+      QtVariantProperty *item = NULL;
 
+      rendering::UserCameraPtr cam = gui::get_active_camera();
+      if (!cam)
+        return;
+
+      // Create a camera item
+      QtProperty *topItem = this->variantManager->addProperty(
+          QtVariantPropertyManager::groupTypeId(), tr("camera"));
+      auto cameraBrowser = this->propTreeBrowser->addProperty(topItem);
+
+      // Create and set the gui camera name
+      std::string cameraName = cam->GetName();
+      item = this->variantManager->addProperty(QVariant::String, tr("name"));
+      item->setValue(cameraName.c_str());
+      topItem->addSubProperty(item);
+      item->setEnabled(false);
+
+      // Create and set the gui camera pose
+      item = this->variantManager->addProperty(
+          QtVariantPropertyManager::groupTypeId(), tr("pose"));
+      {
+        topItem->addSubProperty(item);
+        ignition::math::Pose3d cameraPose = cam->GetWorldPose().Ign();
+
+        this->FillPoseProperty(msgs::Convert(cameraPose), item);
+        // set expanded to true by default for easier viewing
+        this->propTreeBrowser->setExpanded(cameraBrowser, true);
+        for (auto browser : cameraBrowser->children())
+        {
+          this->propTreeBrowser->setExpanded(browser, true);
+        }
+      }
+    }
     else
     {
       this->propTreeBrowser->clear();
@@ -286,6 +334,15 @@ void ModelListWidget::OnModelUpdate(const msgs::Model &_msg)
   this->modelMsgs.push_back(msg);
 }
 
+/////////////////////////////////////////////////
+void ModelListWidget::OnLightUpdate(const msgs::Light &_msg)
+{
+  boost::mutex::scoped_lock lock(*this->receiveMutex);
+  msgs::Light msg;
+  msg.CopyFrom(_msg);
+  this->lightMsgs.push_back(msg);
+}
+
 /////////////////////////////////////////////////
 void ModelListWidget::ProcessModelMsgs()
 {
@@ -421,7 +478,7 @@ void ModelListWidget::OnResponse(ConstResponsePtr &_msg)
   }
   else if (_msg->has_type() && _msg->type() == "error")
   {
-    if (_msg->response() == "nonexistant")
+    if (_msg->response() == "nonexistent")
     {
       this->removeEntityList.push_back(this->selectedEntityName);
     }
@@ -496,7 +553,8 @@ void ModelListWidget::OnCustomContextMenu(const QPoint &_pt)
   if (i >= 0)
   {
     g_modelRightMenu->Run(item->text(0).toStdString(),
-                          this->modelTreeWidget->mapToGlobal(_pt));
+                          this->modelTreeWidget->mapToGlobal(_pt),
+                          ModelRightMenu::EntityTypes::MODEL);
     return;
   }
 
@@ -505,7 +563,16 @@ void ModelListWidget::OnCustomContextMenu(const QPoint &_pt)
   if (i >= 0)
   {
     g_modelRightMenu->Run(item->text(0).toStdString(),
-                          this->modelTreeWidget->mapToGlobal(_pt));
+                          this->modelTreeWidget->mapToGlobal(_pt),
+                          ModelRightMenu::EntityTypes::LIGHT);
+  }
+
+  // Check to see if the selected item is a link
+  if (item->data(3, Qt::UserRole).toString().toStdString() == "Link")
+  {
+    g_modelRightMenu->Run(item->data(0, Qt::UserRole).toString().toStdString(),
+                          this->modelTreeWidget->mapToGlobal(_pt),
+                          ModelRightMenu::EntityTypes::LINK);
   }
 }
 
@@ -542,6 +609,8 @@ void ModelListWidget::OnPropertyChanged(QtProperty *_item)
     this->ScenePropertyChanged(_item);
   else if (currentItem == this->physicsItem)
     this->PhysicsPropertyChanged(_item);
+  else if (currentItem == this->guiItem)
+    this->GUIPropertyChanged(_item);
 }
 
 /////////////////////////////////////////////////
@@ -558,7 +627,7 @@ void ModelListWidget::LightPropertyChanged(QtProperty * /*_item*/)
             (*iter)).toString().toStdString());
     else if ((*iter)->propertyName().toStdString() == "pose")
     {
-      math::Pose pose;
+      ignition::math::Pose3d pose;
       pose.Set(this->variantManager->value(
                  this->GetChildItem((*iter), "x")).toDouble(),
                this->variantManager->value(
@@ -605,6 +674,37 @@ void ModelListWidget::LightPropertyChanged(QtProperty * /*_item*/)
   this->lightPub->Publish(msg);
 }
 
+/////////////////////////////////////////////////
+void ModelListWidget::GUIPropertyChanged(QtProperty *_item)
+{
+  // Only camera pose editable for now
+  QtProperty *cameraProperty = this->GetChildItem("camera");
+  if (!cameraProperty)
+    return;
+
+  QtProperty *cameraPoseProperty = this->GetChildItem(cameraProperty, "pose");
+  if (!cameraPoseProperty)
+    return;
+
+  if (cameraPoseProperty)
+  {
+    std::string changedProperty = _item->propertyName().toStdString();
+    if (changedProperty == "x"
+      || changedProperty == "y"
+      || changedProperty == "z"
+      || changedProperty == "roll"
+      || changedProperty == "pitch"
+      || changedProperty == "yaw")
+    {
+      msgs::Pose poseMsg;
+      this->FillPoseMsg(cameraPoseProperty, &poseMsg, poseMsg.GetDescriptor());
+      rendering::UserCameraPtr cam = gui::get_active_camera();
+      if (cam)
+        cam->SetWorldPose(msgs::ConvertIgn(poseMsg));
+    }
+  }
+}
+
 /////////////////////////////////////////////////
 void ModelListWidget::PhysicsPropertyChanged(QtProperty * /*_item*/)
 {
@@ -616,6 +716,8 @@ void ModelListWidget::PhysicsPropertyChanged(QtProperty * /*_item*/)
   {
     if ((*iter)->propertyName().toStdString() == "gravity")
       this->FillVector3Msg((*iter), msg.mutable_gravity());
+    else if ((*iter)->propertyName().toStdString() == "magnetic field")
+      this->FillVector3Msg((*iter), msg.mutable_magnetic_field());
     else if ((*iter)->propertyName().toStdString() == "enable physics")
       msg.set_enable_physics(this->variantManager->value((*iter)).toBool());
     else if ((*iter)->propertyName().toStdString() == "solver")
@@ -652,7 +754,7 @@ void ModelListWidget::PhysicsPropertyChanged(QtProperty * /*_item*/)
 }
 
 /////////////////////////////////////////////////
-void ModelListWidget::ScenePropertyChanged(QtProperty * /*_item*/)
+void ModelListWidget::ScenePropertyChanged(QtProperty */*_item*/)
 {
   msgs::Scene msg;
 
@@ -1290,7 +1392,7 @@ QtProperty *ModelListWidget::GetChildItem(QtProperty *_item,
 
 /////////////////////////////////////////////////
 void ModelListWidget::FillPropertyTree(const msgs::SphericalCoordinates &_msg,
-    QtProperty *_parent)
+                                       QtProperty * /*_parent*/)
 {
   QtVariantProperty *item = NULL;
 
@@ -1316,68 +1418,79 @@ void ModelListWidget::FillPropertyTree(const msgs::SphericalCoordinates &_msg,
 
   item->setAttribute("enumNames", types);
   item->setValue(0);
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 
   item = this->variantManager->addProperty(QVariant::Double, tr("Latitude"));
   item->setValue(_msg.latitude_deg());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 
   item = this->variantManager->addProperty(QVariant::Double, tr("Longitude"));
   item->setValue(_msg.longitude_deg());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 
   item = this->variantManager->addProperty(QVariant::Double, tr("Elevation"));
   item->setValue(_msg.elevation());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 
   item = this->variantManager->addProperty(QVariant::Double, tr("Heading"));
   item->setValue(_msg.heading_deg());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 }
 
 /////////////////////////////////////////////////
 void ModelListWidget::FillPropertyTree(const msgs::Joint &_msg,
-                                       QtProperty *_parent)
+                                       QtProperty * /*_parent*/)
 {
   QtProperty *topItem = NULL;
   QtVariantProperty *item = NULL;
 
+  // joint name
   item = this->variantManager->addProperty(QVariant::String,
                                            tr("name"));
   item->setValue(_msg.name().c_str());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->propTreeBrowser->addProperty(item);
   item->setEnabled(false);
 
+  // joint type
+  if (_msg.has_type())
+  {
+    item = this->variantManager->addProperty(QVariant::String,
+                                             tr("type"));
+    std::string jointType = msgs::ConvertJointType(_msg.type());
+    item->setValue(jointType.c_str());
+    this->propTreeBrowser->addProperty(item);
+    item->setEnabled(false);
+  }
+
+  // parent link
+  if (_msg.has_parent())
+  {
+    item = this->variantManager->addProperty(QVariant::String,
+                                               tr("parent link"));
+    item->setValue(_msg.parent().c_str());
+    this->propTreeBrowser->addProperty(item);
+    item->setEnabled(false);
+  }
+
+  // child link
+  if (_msg.has_child())
+  {
+    item = this->variantManager->addProperty(QVariant::String,
+                                               tr("child link"));
+    item->setValue(_msg.child().c_str());
+    this->propTreeBrowser->addProperty(item);
+    item->setEnabled(false);
+  }
+
   // Pose value
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(), tr("pose"));
-  if (_parent)
-    _parent->addSubProperty(topItem);
-  else
-    this->propTreeBrowser->addProperty(topItem);
+  this->propTreeBrowser->addProperty(topItem);
   topItem->setEnabled(false);
 
   this->FillPoseProperty(_msg.pose(), topItem);
@@ -1389,10 +1502,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Joint &_msg,
     item = this->variantManager->addProperty(QVariant::String,
                                              QString::fromStdString(angleName));
     item->setValue(_msg.angle(i));
-    if (_parent)
-      _parent->addSubProperty(item);
-    else
-      this->propTreeBrowser->addProperty(item);
+    this->propTreeBrowser->addProperty(item);
     item->setEnabled(false);
   }
 
@@ -1418,10 +1528,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Joint &_msg,
       // Axis shape value
       topItem = this->variantManager->addProperty(
           QtVariantPropertyManager::groupTypeId(), tr(axisName.c_str()));
-      if (_parent)
-        _parent->addSubProperty(topItem);
-      else
-        this->propTreeBrowser->addProperty(topItem);
+      this->propTreeBrowser->addProperty(topItem);
       topItem->setEnabled(false);
 
       /// XYZ of the axis
@@ -1484,20 +1591,14 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
   QtBrowserItem *browserItem = NULL;
   item = this->variantManager->addProperty(QVariant::String, tr("id"));
   item->setValue(_msg.id());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->AddProperty(item, _parent);
   browserItem = this->propTreeBrowser->items(item)[0];
   this->propTreeBrowser->setItemVisible(browserItem, false);
 
   // name
   item = this->variantManager->addProperty(QVariant::String, tr("name"));
   item->setValue(_msg.name().c_str());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->AddProperty(item, _parent);
   // TODO: setting link name currently causes problems
   item->setEnabled(false);
 
@@ -1507,12 +1608,8 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     item->setValue(_msg.self_collide());
   else
     item->setValue(true);
-
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
-
+  this->AddProperty(item, _parent);
+  item->setEnabled(false);
 
   // gravity
   item = this->variantManager->addProperty(QVariant::Bool, tr("gravity"));
@@ -1520,11 +1617,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     item->setValue(_msg.gravity());
   else
     item->setValue(true);
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
-
+  this->AddProperty(item, _parent);
 
   // kinematic
   item = this->variantManager->addProperty(QVariant::Bool, tr("kinematic"));
@@ -1532,10 +1625,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     item->setValue(_msg.kinematic());
   else
     item->setValue(false);
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->AddProperty(item, _parent);
 
   // canonical
   item = this->variantManager->addProperty(QVariant::Bool, tr("canonical"));
@@ -1543,19 +1633,13 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     item->setValue(_msg.canonical());
   else
     item->setValue(false);
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  this->AddProperty(item, _parent);
   item->setEnabled(false);
 
   // pose
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(), tr("pose"));
-  if (_parent)
-    _parent->addSubProperty(topItem);
-  else
-    this->propTreeBrowser->addProperty(topItem);
+  this->AddProperty(topItem, _parent);
 
   this->FillPoseProperty(_msg.pose(), topItem);
   if (_msg.has_canonical() && _msg.canonical())
@@ -1564,10 +1648,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
   // Inertial
   inertialItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(), tr("inertial"));
-  if (_parent)
-    _parent->addSubProperty(inertialItem);
-  else
-    this->propTreeBrowser->addProperty(inertialItem);
+  this->AddProperty(inertialItem, _parent);
 
   // TODO: disable setting inertial properties until there are tests
   // in place to verify the functionality
@@ -1658,10 +1739,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     prop = this->variantManager->addProperty(
         QtVariantPropertyManager::groupTypeId(), tr("collision"));
     prop->setToolTip(tr(_msg.collision(i).name().c_str()));
-    if (_parent)
-      _parent->addSubProperty(prop);
-    else
-      this->propTreeBrowser->addProperty(prop);
+    this->AddProperty(prop, _parent);
 
     this->FillPropertyTree(_msg.collision(i), prop);
 
@@ -1676,10 +1754,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     prop = this->variantManager->addProperty(
         QtVariantPropertyManager::groupTypeId(), tr("visual"));
     prop->setToolTip(tr(_msg.visual(i).name().c_str()));
-    if (_parent)
-      _parent->addSubProperty(prop);
-    else
-      this->propTreeBrowser->addProperty(prop);
+    this->AddProperty(prop, _parent);
 
     this->FillPropertyTree(_msg.visual(i), prop);
 
@@ -1694,10 +1769,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
     prop = this->variantManager->addProperty(
         QtVariantPropertyManager::groupTypeId(), "sensor");
     prop->setToolTip(tr(_msg.sensor(i).name().c_str()));
-    if (_parent)
-      _parent->addSubProperty(prop);
-    else
-      this->propTreeBrowser->addProperty(prop);
+    this->AddProperty(prop, _parent);
 
     // this->FillPropertyTree(_msg.sensor(i), prop);
   }
@@ -1707,6 +1779,13 @@ void ModelListWidget::FillPropertyTree(const msgs::Link &_msg,
 void ModelListWidget::FillPropertyTree(const msgs::Collision &_msg,
                                        QtProperty *_parent)
 {
+  if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding collision elements."
+           << " This should never happen." << std::endl;
+    return;
+  }
+
   QtProperty *topItem = NULL;
   QtVariantProperty *item = NULL;
 
@@ -1714,10 +1793,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Collision &_msg,
   QtBrowserItem *browserItem = NULL;
   item = this->variantManager->addProperty(QVariant::String, tr("id"));
   item->setValue(_msg.id());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  _parent->addSubProperty(item);
   browserItem = this->propTreeBrowser->items(item)[0];
   this->propTreeBrowser->setItemVisible(browserItem, false);
 
@@ -1725,10 +1801,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Collision &_msg,
   item = this->variantManager->addProperty(QVariant::String,
                                            tr("name"));
   item->setValue(_msg.name().c_str());
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  _parent->addSubProperty(item);
 
   // Laser Retro value
   item = this->variantManager->addProperty(QVariant::Double,
@@ -1737,39 +1810,27 @@ void ModelListWidget::FillPropertyTree(const msgs::Collision &_msg,
     item->setValue(_msg.laser_retro());
   else
     item->setValue(0.0);
-  if (_parent)
-    _parent->addSubProperty(item);
-  else
-    this->propTreeBrowser->addProperty(item);
+  _parent->addSubProperty(item);
 
   // Pose value
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(),
       tr("pose"));
-  if (_parent)
-    _parent->addSubProperty(topItem);
-  else
-    this->propTreeBrowser->addProperty(topItem);
+  _parent->addSubProperty(topItem);
   this->FillPoseProperty(_msg.pose(), topItem);
 
   // Geometry shape value
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(),
       tr("geometry"));
-  if (_parent)
-    _parent->addSubProperty(topItem);
-  else
-    this->propTreeBrowser->addProperty(topItem);
+  _parent->addSubProperty(topItem);
   this->FillPropertyTree(_msg.geometry(), topItem);
 
   // Surface value
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(),
       tr("surface"));
-  if (_parent)
-    _parent->addSubProperty(topItem);
-  else
-    this->propTreeBrowser->addProperty(topItem);
+  _parent->addSubProperty(topItem);
   this->FillPropertyTree(_msg.surface(), topItem);
 }
 
@@ -1778,7 +1839,11 @@ void ModelListWidget::FillPropertyTree(const msgs::Surface &_msg,
                                        QtProperty *_parent)
 {
   if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding surface elements."
+           << " This should never happen." << std::endl;
     return;
+  }
 
   QtProperty *topItem = NULL;
   QtVariantProperty *item = NULL;
@@ -1874,7 +1939,11 @@ void ModelListWidget::FillPropertyTree(const msgs::Geometry &_msg,
                                        QtProperty *_parent)
 {
   if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding geometry elements."
+           << " This should never happen." << std::endl;
     return;
+  }
 
   QtVariantProperty *item = NULL;
 
@@ -1996,7 +2065,11 @@ void ModelListWidget::FillPropertyTree(const msgs::Visual &_msg,
                                        QtProperty *_parent)
 {
   if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding visual elements."
+           << " This should never happen." << std::endl;
     return;
+  }
 
   QtProperty *topItem = NULL;
   QtVariantProperty *item = NULL;
@@ -2074,6 +2147,14 @@ void ModelListWidget::FillPropertyTree(const msgs::Model &_msg,
   item->setEnabled(false);
   this->propTreeBrowser->addProperty(item);
 
+  item = this->variantManager->addProperty(QVariant::Bool, tr("self_collide"));
+  if (_msg.has_self_collide())
+    item->setValue(_msg.self_collide());
+  else
+    item->setValue(false);
+  item->setEnabled(false);
+  this->propTreeBrowser->addProperty(item);
+
   topItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(), tr("pose"));
   QtBrowserItem *bItem = this->propTreeBrowser->addProperty(topItem);
@@ -2099,11 +2180,15 @@ void ModelListWidget::FillVector3dProperty(const msgs::Vector3d &_msg,
                                            QtProperty *_parent)
 {
   if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding Vector3d elements."
+           << " This should never happen." << std::endl;
     return;
+  }
 
   QtVariantProperty *item;
-  math::Vector3 value;
-  value = msgs::Convert(_msg);
+  ignition::math::Vector3d value;
+  value = msgs::ConvertIgn(_msg);
   value.Round(6);
 
   // Add X value
@@ -2111,37 +2196,34 @@ void ModelListWidget::FillVector3dProperty(const msgs::Vector3d &_msg,
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "x");
-    if (_parent)
-      _parent->addSubProperty(item);
+    _parent->addSubProperty(item);
   }
   static_cast<QtVariantPropertyManager*>
     (this->variantFactory->propertyManager(item))->setAttribute(
         item, "decimals", 6);
-  item->setValue(value.x);
+  item->setValue(value.X());
 
   // Add Y value
   item = static_cast<QtVariantProperty*>(this->GetChildItem(_parent, "y"));
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "y");
-    if (_parent)
-      _parent->addSubProperty(item);
+    _parent->addSubProperty(item);
   }
   static_cast<QtVariantPropertyManager*>(this->variantFactory->propertyManager(
     item))->setAttribute(item, "decimals", 6);
-  item->setValue(value.y);
+  item->setValue(value.Y());
 
   // Add Z value
   item = static_cast<QtVariantProperty*>(this->GetChildItem(_parent, "z"));
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "z");
-    if (_parent)
-      _parent->addSubProperty(item);
+    _parent->addSubProperty(item);
   }
   static_cast<QtVariantPropertyManager*>(this->variantFactory->propertyManager(
     item))->setAttribute(item, "decimals", 6);
-  item->setValue(value.z);
+  item->setValue(value.Z());
 }
 
 /////////////////////////////////////////////////
@@ -2149,14 +2231,18 @@ void ModelListWidget::FillPoseProperty(const msgs::Pose &_msg,
                                        QtProperty *_parent)
 {
   if (!_parent)
+  {
+    gzwarn << "Null QtProperty parent, not adding pose elements."
+           << " This should never happen." << std::endl;
     return;
+  }
 
   QtVariantProperty *item;
-  math::Pose value;
-  value = msgs::Convert(_msg);
+  ignition::math::Pose3d value;
+  value = msgs::ConvertIgn(_msg);
   value.Round(6);
 
-  math::Vector3 rpy = value.rot.GetAsEuler();
+  ignition::math::Vector3d rpy = value.Rot().Euler();
   rpy.Round(6);
 
   this->FillVector3dProperty(_msg.position(), _parent);
@@ -2166,47 +2252,45 @@ void ModelListWidget::FillPoseProperty(const msgs::Pose &_msg,
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "roll");
-    if (_parent)
-    {
-      _parent->addSubProperty(item);
-    }
+    _parent->addSubProperty(item);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "decimals", 6);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "singleStep", 0.05);
   }
-  static_cast<QtVariantPropertyManager*>(this->variantFactory->propertyManager(
-    item))->setAttribute(item, "decimals", 6);
-  item->setValue(rpy.x);
+  item->setValue(rpy.X());
 
   // Add Pitch value
   item = static_cast<QtVariantProperty*>(this->GetChildItem(_parent, "pitch"));
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "pitch");
-    if (_parent)
-      _parent->addSubProperty(item);
+    _parent->addSubProperty(item);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "decimals", 6);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "singleStep", 0.05);
   }
-  static_cast<QtVariantPropertyManager*>(this->variantFactory->propertyManager(
-    item))->setAttribute(item, "decimals", 6);
-  item->setValue(rpy.y);
+  item->setValue(rpy.Y());
 
   // Add Yaw value
   item = static_cast<QtVariantProperty*>(this->GetChildItem(_parent, "yaw"));
   if (!item)
   {
     item = this->variantManager->addProperty(QVariant::Double, "yaw");
-    if (_parent)
-      _parent->addSubProperty(item);
+    _parent->addSubProperty(item);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "decimals", 6);
+    static_cast<QtVariantPropertyManager *>(
+        this->variantFactory->propertyManager(
+        item))->setAttribute(item, "singleStep", 0.05);
   }
-  static_cast<QtVariantPropertyManager*>(this->variantFactory->propertyManager(
-    item))->setAttribute(item, "decimals", 6);
-  item->setValue(rpy.z);
-}
-
-/////////////////////////////////////////////////
-void ModelListWidget::OnLightMsg(ConstLightPtr &_msg)
-{
-  boost::mutex::scoped_lock lock(*this->receiveMutex);
-  msgs::Light msg;
-  msg.CopyFrom(*_msg);
-  this->lightMsgs.push_back(msg);
+  item->setValue(rpy.Z());
 }
 
 /////////////////////////////////////////////////
@@ -2290,9 +2374,6 @@ void ModelListWidget::InitTransport(const std::string &_name)
 
   this->requestSub = this->node->Subscribe("~/request",
       &ModelListWidget::OnRequest, this, false);
-
-  this->lightSub = this->node->Subscribe("~/light",
-                                         &ModelListWidget::OnLightMsg, this);
 }
 
 /////////////////////////////////////////////////
@@ -2302,8 +2383,17 @@ void ModelListWidget::ResetTree()
 
   // Create the top level of items in the tree widget
   {
+    // GUI item
+    this->guiItem = new QTreeWidgetItem(
+        static_cast<QTreeWidgetItem*>(0),
+        QStringList(QString("%1").arg(tr("GUI"))));
+    this->guiItem->setData(0, Qt::UserRole, QVariant(tr("GUI")));
+    this->modelTreeWidget->addTopLevelItem(this->guiItem);
+
+    // Scene item
     this->ResetScene();
 
+    // Spherical coordinates item
     this->sphericalCoordItem = new QTreeWidgetItem(
         static_cast<QTreeWidgetItem*>(0),
         QStringList(QString("%1").arg(tr("Spherical Coordinates"))));
@@ -2311,18 +2401,21 @@ void ModelListWidget::ResetTree()
         Qt::UserRole, QVariant(tr("Spherical Coordinates")));
     this->modelTreeWidget->addTopLevelItem(this->sphericalCoordItem);
 
+    // Physics item
     this->physicsItem = new QTreeWidgetItem(
         static_cast<QTreeWidgetItem*>(0),
         QStringList(QString("%1").arg(tr("Physics"))));
     this->physicsItem->setData(0, Qt::UserRole, QVariant(tr("Physics")));
     this->modelTreeWidget->addTopLevelItem(this->physicsItem);
 
+    // Models item
     this->modelsItem = new QTreeWidgetItem(
         static_cast<QTreeWidgetItem*>(0),
         QStringList(QString("%1").arg(tr("Models"))));
     this->modelsItem->setData(0, Qt::UserRole, QVariant(tr("Models")));
     this->modelTreeWidget->addTopLevelItem(this->modelsItem);
 
+    // Lights item
     this->lightsItem = new QTreeWidgetItem(
         static_cast<QTreeWidgetItem*>(0),
         QStringList(QString("%1").arg(tr("Lights"))));
@@ -2499,6 +2592,20 @@ void ModelListWidget::FillPropertyTree(const msgs::Physics &_msg,
     this->FillVector3dProperty(xyz, gravityItem);
   }
 
+  QtProperty *magneticFieldItem = this->variantManager->addProperty(
+      QtVariantPropertyManager::groupTypeId(), tr("magnetic field"));
+  this->propTreeBrowser->addProperty(magneticFieldItem);
+  if (_msg.has_magnetic_field())
+    this->FillVector3dProperty(_msg.magnetic_field(), magneticFieldItem);
+  else
+  {
+    msgs::Vector3d xyz;
+    xyz.set_x(0.0);
+    xyz.set_y(0.0);
+    xyz.set_z(0.0);
+    this->FillVector3dProperty(xyz, magneticFieldItem);
+  }
+
   QtProperty *solverItem = this->variantManager->addProperty(
       QtVariantPropertyManager::groupTypeId(), tr("solver"));
   this->propTreeBrowser->addProperty(solverItem);
@@ -2576,7 +2683,7 @@ void ModelListWidget::FillPropertyTree(const msgs::Light &_msg,
   if (_msg.has_pose())
     this->FillPoseProperty(_msg.pose(), topItem);
   else
-    this->FillPoseProperty(msgs::Convert(math::Pose()), topItem);
+    this->FillPoseProperty(msgs::Convert(ignition::math::Pose3d()), topItem);
 
   // Create and set the diffuse color property
   item = this->variantManager->addProperty(QVariant::Color, tr("diffuse"));
@@ -2678,3 +2785,19 @@ void ModelListWidget::ProcessLightMsgs()
   }
   this->lightMsgs.clear();
 }
+
+/////////////////////////////////////////////////
+void ModelListWidget::AddProperty(QtProperty *_item, QtProperty *_parent)
+{
+  if (!_item)
+  {
+    gzwarn << "Null QtProperty item, not adding."
+           << " This should never happen." << std::endl;
+    return;
+  }
+
+  if (_parent)
+    _parent->addSubProperty(_item);
+  else
+    this->propTreeBrowser->addProperty(_item);
+}
