@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -385,9 +385,10 @@ void ModelListWidget_TEST::ModelProperties()
   QCOMPARE(modelItem->text(0), tr(modelName.c_str()));
 
   // Get propery browser widget
+  QObject *propTreeObj =
+    modelListWidget->findChild<QObject *>("propTreeBrowser");
   QtTreePropertyBrowser *propTreeBrowser =
-    modelListWidget->findChild<QtTreePropertyBrowser *>(
-      "propTreeBrowser");
+    dynamic_cast<QtTreePropertyBrowser *>(propTreeObj);
 
   QVERIFY(propTreeBrowser != NULL);
   QCOMPARE(propTreeBrowser->properties().size(), 0);
@@ -609,6 +610,7 @@ void ModelListWidget_TEST::LinkProperties()
   gazebo::gui::ModelListWidget *modelListWidget
       = new gazebo::gui::ModelListWidget;
   modelListWidget->show();
+  modelListWidget->setGeometry(0, 0, 400, 800);
   QCoreApplication::processEvents();
 
   this->Load("worlds/multilink_shape.world");
@@ -655,9 +657,10 @@ void ModelListWidget_TEST::LinkProperties()
   QCOMPARE(modelItem->text(0), tr(modelName.c_str()));
 
   // Get propery browser widget
+  QObject *propTreeObj =
+    modelListWidget->findChild<QObject *>("propTreeBrowser");
   QtTreePropertyBrowser *propTreeBrowser =
-    modelListWidget->findChild<QtTreePropertyBrowser *>(
-      "propTreeBrowser");
+    dynamic_cast<QtTreePropertyBrowser *>(propTreeObj);
 
   QVERIFY(propTreeBrowser != NULL);
   QCOMPARE(propTreeBrowser->properties().size(), 0);
@@ -711,6 +714,7 @@ void ModelListWidget_TEST::LinkProperties()
   QRect boxLinkRect = modelTreeWidget->visualItemRect(boxLinkItem);
   QTest::mouseClick(modelTreeWidget->viewport(), Qt::LeftButton, 0,
       boxLinkRect.center() );
+
   QCoreApplication::processEvents();
   sleep = 0;
   maxSleep = 5;
@@ -785,10 +789,12 @@ void ModelListWidget_TEST::LinkProperties()
   QTest::mouseClick(modelTreeWidget->viewport(), Qt::LeftButton, 0,
       sphereLinkRect.center() );
   QCoreApplication::processEvents();
+  QTest::qWait(100);
   sleep = 0;
   maxSleep = 5;
   while (!sphereLinkItem->isSelected() && sleep < maxSleep)
   {
+    QCoreApplication::processEvents();
     QTest::qWait(10);
     sleep++;
   }
