@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,12 @@
  *
 */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <tbb/parallel_for.h>
 #include <tbb/blocked_range.h>
 #include <float.h>
@@ -67,10 +73,20 @@ void Model::Load(sdf::ElementPtr _sdf)
   this->jointPub = this->node->Advertise<msgs::Joint>("~/joint");
 
   this->SetStatic(this->sdf->Get<bool>("static"));
-  this->sdf->GetElement("static")->GetValue()->SetUpdateFunc(
-      boost::bind(&Entity::IsStatic, this));
+  if (this->sdf->HasElement("static"))
+  {
+    this->sdf->GetElement("static")->GetValue()->SetUpdateFunc(
+        boost::bind(&Entity::IsStatic, this));
+  }
+
+  if (this->sdf->HasElement("self_collide"))
+  {
+    this->SetSelfCollide(this->sdf->Get<bool>("self_collide"));
+  }
+
+  if (this->sdf->HasElement("allow_auto_disable"))
+    this->SetAutoDisable(this->sdf->Get<bool>("allow_auto_disable"));
 
-  this->SetAutoDisable(this->sdf->Get<bool>("allow_auto_disable"));
   this->LoadLinks();
 
   // Load the joints if the world is already loaded. Otherwise, the World
@@ -318,9 +334,9 @@ void Model::Fini()
 {
   Entity::Fini();
 
+  this->plugins.clear();
   this->attachedModels.clear();
   this->joints.clear();
-  this->plugins.clear();
   this->links.clear();
   this->canonicalLink.reset();
 }
@@ -367,24 +383,32 @@ void Model::Reset()
 {
   Entity::Reset();
 
-  for (std::vector<ModelPluginPtr>::iterator iter = this->plugins.begin();
-       iter != this->plugins.end(); ++iter)
-  {
-    (*iter)->Reset();
-  }
-
-  // reset link velocities when resetting model
-  for (Link_V::iterator liter = this->links.begin();
-       liter != this->links.end(); ++liter)
-  {
-    (*liter)->ResetPhysicsStates();
-  }
+  this->ResetPhysicsStates();
 
   for (Joint_V::iterator jiter = this->joints.begin();
        jiter != this->joints.end(); ++jiter)
   {
     (*jiter)->Reset();
   }
+
+  // Reset plugins after links and joints,
+  // so that plugins can restore initial conditions
+  for (std::vector<ModelPluginPtr>::iterator iter = this->plugins.begin();
+       iter != this->plugins.end(); ++iter)
+  {
+    (*iter)->Reset();
+  }
+}
+
+//////////////////////////////////////////////////
+void Model::ResetPhysicsStates()
+{
+  // reset link velocities when resetting model
+  for (Link_V::iterator liter = this->links.begin();
+       liter != this->links.end(); ++liter)
+  {
+    (*liter)->ResetPhysicsStates();
+  }
 }
 
 //////////////////////////////////////////////////
@@ -558,20 +582,11 @@ JointPtr Model::GetJoint(const std::string &_name)
 
   for (iter = this->joints.begin(); iter != this->joints.end(); ++iter)
   {
-    if ((*iter)->GetScopedName() == _name ||
-        (*iter)->GetName() == _name)
+    if ((*iter)->GetScopedName() == _name || (*iter)->GetName() == _name)
     {
       result = (*iter);
       break;
     }
-    else if ((*iter)->GetName() == _name)
-    {
-      // check again without scoped names, deprecated, warn
-      result = (*iter);
-      gzwarn << "Calling Model::GetJoint(" << _name
-             << ") with un-scoped joint name is deprecated, please scope\n";
-      break;
-    }
   }
 
   return result;
@@ -584,7 +599,7 @@ LinkPtr Model::GetLinkById(unsigned int _id) const
 }
 
 //////////////////////////////////////////////////
-Link_V Model::GetLinks() const
+const Link_V &Model::GetLinks() const
 {
   return this->links;
 }
@@ -737,8 +752,27 @@ void Model::LoadPlugin(sdf::ElementPtr _sdf)
 {
   std::string pluginName = _sdf->Get<std::string>("name");
   std::string filename = _sdf->Get<std::string>("filename");
-  gazebo::ModelPluginPtr plugin =
-    gazebo::ModelPlugin::Create(filename, pluginName);
+
+  gazebo::ModelPluginPtr plugin;
+
+  try
+  {
+    plugin = gazebo::ModelPlugin::Create(filename, pluginName);
+  }
+  catch(...)
+  {
+    gzerr << "Exception occured in the constructor of plugin with name["
+      << pluginName << "] and filename[" << filename << "]. "
+      << "This plugin will not run.\n";
+
+    // Log the message. gzerr has problems with this in 1.9. Remove the
+    // gzlog command in gazebo2.
+    gzlog << "Exception occured in the constructor of plugin with name["
+      << pluginName << "] and filename[" << filename << "]. "
+      << "This plugin will not run." << std::endl;
+    return;
+  }
+
   if (plugin)
   {
     if (plugin->GetType() != MODEL_PLUGIN)
@@ -751,8 +785,43 @@ void Model::LoadPlugin(sdf::ElementPtr _sdf)
     }
 
     ModelPtr myself = boost::static_pointer_cast<Model>(shared_from_this());
-    plugin->Load(myself, _sdf);
-    plugin->Init();
+
+    try
+    {
+      plugin->Load(myself, _sdf);
+    }
+    catch(...)
+    {
+      gzerr << "Exception occured in the Load function of plugin with name["
+        << pluginName << "] and filename[" << filename << "]. "
+        << "This plugin will not run.\n";
+
+      // Log the message. gzerr has problems with this in 1.9. Remove the
+      // gzlog command in gazebo2.
+      gzlog << "Exception occured in the Load function of plugin with name["
+        << pluginName << "] and filename[" << filename << "]. "
+        << "This plugin will not run." << std::endl;
+      return;
+    }
+
+    try
+    {
+      plugin->Init();
+    }
+    catch(...)
+    {
+      gzerr << "Exception occured in the Init function of plugin with name["
+        << pluginName << "] and filename[" << filename << "]. "
+        << "This plugin will not run\n";
+
+      // Log the message. gzerr has problems with this in 1.9. Remove the
+      // gzlog command in gazebo2.
+      gzlog << "Exception occured in the Init function of plugin with name["
+        << pluginName << "] and filename[" << filename << "]. "
+        << "This plugin will not run." << std::endl;
+      return;
+    }
+
     this->plugins.push_back(plugin);
   }
 }
@@ -801,11 +870,12 @@ void Model::FillMsg(msgs::Model &_msg)
 {
   _msg.set_name(this->GetScopedName());
   _msg.set_is_static(this->IsStatic());
-  msgs::Set(_msg.mutable_pose(), this->GetWorldPose());
+  _msg.set_self_collide(this->GetSelfCollide());
+  msgs::Set(_msg.mutable_pose(), this->GetWorldPose().Ign());
   _msg.set_id(this->GetId());
-  msgs::Set(_msg.mutable_scale(), this->scale);
+  msgs::Set(_msg.mutable_scale(), this->scale.Ign());
 
-  msgs::Set(this->visualMsg->mutable_pose(), this->GetWorldPose());
+  msgs::Set(this->visualMsg->mutable_pose(), this->GetWorldPose().Ign());
   _msg.add_visual()->CopyFrom(*this->visualMsg);
 
   for (Link_V::iterator iter = this->links.begin(); iter != this->links.end();
@@ -839,7 +909,7 @@ void Model::ProcessMsg(const msgs::Model &_msg)
 
   this->SetName(this->world->StripWorldName(_msg.name()));
   if (_msg.has_pose())
-    this->SetWorldPose(msgs::Convert(_msg.pose()));
+    this->SetWorldPose(msgs::ConvertIgn(_msg.pose()));
   for (int i = 0; i < _msg.link_size(); i++)
   {
     LinkPtr link = this->GetLinkById(_msg.link(i).id());
@@ -851,7 +921,7 @@ void Model::ProcessMsg(const msgs::Model &_msg)
     this->SetStatic(_msg.is_static());
 
   if (_msg.has_scale())
-    this->SetScale(msgs::Convert(_msg.scale()));
+    this->SetScale(msgs::ConvertIgn(_msg.scale()));
 }
 
 //////////////////////////////////////////////////
@@ -1016,6 +1086,18 @@ bool Model::GetAutoDisable() const
   return this->sdf->Get<bool>("allow_auto_disable");
 }
 
+/////////////////////////////////////////////////
+void Model::SetSelfCollide(bool _self_collide)
+{
+  this->sdf->GetElement("self_collide")->Set(_self_collide);
+}
+
+/////////////////////////////////////////////////
+bool Model::GetSelfCollide() const
+{
+  return this->sdf->Get<bool>("self_collide");
+}
+
 /////////////////////////////////////////////////
 void Model::RemoveLink(const std::string &_name)
 {
