@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,14 @@
  * limitations under the License.
  *
 */
+
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include "gazebo/common/Console.hh"
 #include "gazebo/physics/Collision.hh"
 #include "gazebo/physics/Shape.hh"
 
@@ -45,3 +53,18 @@ math::Vector3 Shape::GetScale() const
 {
   return this->scale;
 }
+
+//////////////////////////////////////////////////
+double Shape::ComputeVolume() const
+{
+  if (!this->collisionParent)
+  {
+    gzerr << "Cannot discern shape type, returning 0 volume" << std::endl;
+    return 0;
+  }
+  gzwarn << "ComputeVolume not fully implemented for this shape type, returning"
+         << " bounding box approximation" << std::endl;
+
+  math::Vector3 size = this->collisionParent->GetBoundingBox().GetSize();
+  return size.x * size.y * size.z;
+}
