@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 
 #include <string>
 
+#include "gazebo/common/Assert.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Console.hh"
 
@@ -48,6 +49,9 @@ BulletJoint::BulletJoint(BasePtr _parent)
 BulletJoint::~BulletJoint()
 {
   delete this->constraint;
+  this->constraint = NULL;
+  delete this->feedback;
+  this->feedback = NULL;
   this->bulletWorld = NULL;
 }
 
@@ -63,10 +67,6 @@ void BulletJoint::Load(sdf::ElementPtr _sdf)
     {
       sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
 
-      if (dynamicsElem->HasElement("damping"))
-      {
-        this->SetDamping(0, dynamicsElem->Get<double>("damping"));
-      }
       if (dynamicsElem->HasElement("friction"))
       {
         sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
@@ -82,10 +82,6 @@ void BulletJoint::Load(sdf::ElementPtr _sdf)
     {
       sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
 
-      if (dynamicsElem->HasElement("damping"))
-      {
-        this->SetDamping(1, dynamicsElem->Get<double>("damping"));
-      }
       if (dynamicsElem->HasElement("friction"))
       {
         sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
@@ -151,6 +147,15 @@ void BulletJoint::Detach()
   if (this->constraint && this->bulletWorld)
     this->bulletWorld->removeConstraint(this->constraint);
   delete this->constraint;
+  this->constraint = NULL;
+}
+
+//////////////////////////////////////////////////
+void BulletJoint::SetProvideFeedback(bool _enable)
+{
+  Joint::SetProvideFeedback(_enable);
+
+  this->SetupJointFeedback();
 }
 
 //////////////////////////////////////////////////
@@ -202,6 +207,7 @@ void BulletJoint::CacheForceTorque()
   else
   {
     /// \TODO: implement for other joint types
+    // note that for fixed joint no further modification is needed
     // gzerr << "force torque for joint type [" << this->GetType()
     //       << "] not implemented, returns false results!!\n";
   }
@@ -340,6 +346,7 @@ void BulletJoint::CacheForceTorque()
 //////////////////////////////////////////////////
 JointWrench BulletJoint::GetForceTorque(unsigned int /*_index*/)
 {
+  GZ_ASSERT(this->constraint != NULL, "constraint should be valid");
   return this->wrench;
 }
 
@@ -348,19 +355,19 @@ void BulletJoint::SetupJointFeedback()
 {
   if (this->provideFeedback)
   {
-    this->feedback = new btJointFeedback;
-    this->feedback->m_appliedForceBodyA = btVector3(0, 0, 0);
-    this->feedback->m_appliedForceBodyB = btVector3(0, 0, 0);
-    this->feedback->m_appliedTorqueBodyA = btVector3(0, 0, 0);
-    this->feedback->m_appliedTorqueBodyB = btVector3(0, 0, 0);
+    if (this->feedback == NULL)
+    {
+      this->feedback = new btJointFeedback;
+      this->feedback->m_appliedForceBodyA = btVector3(0, 0, 0);
+      this->feedback->m_appliedForceBodyB = btVector3(0, 0, 0);
+      this->feedback->m_appliedTorqueBodyA = btVector3(0, 0, 0);
+      this->feedback->m_appliedTorqueBodyB = btVector3(0, 0, 0);
+    }
 
     if (this->constraint)
       this->constraint->setJointFeedback(this->feedback);
     else
-    {
       gzerr << "Bullet Joint [" << this->GetName() << "] ID is invalid\n";
-      getchar();
-    }
   }
 }
 
@@ -533,21 +540,6 @@ math::Vector3 BulletJoint::GetLinkTorque(unsigned int /*_index*/) const
   return math::Vector3();
 }
 
-//////////////////////////////////////////////////
-void BulletJoint::SetAttribute(Attribute, unsigned int /*_index*/,
-    double /*_value*/)
-{
-  gzdbg << "Not implement in Bullet\n";
-}
-
-//////////////////////////////////////////////////
-void BulletJoint::SetAttribute(const std::string &_key,
-    unsigned int _index,
-    const boost::any &_value)
-{
-  this->SetParam(_key, _index, _value);
-}
-
 //////////////////////////////////////////////////
 bool BulletJoint::SetParam(const std::string &/*_key*/,
     unsigned int /*_index*/,
@@ -558,18 +550,10 @@ bool BulletJoint::SetParam(const std::string &/*_key*/,
 }
 
 //////////////////////////////////////////////////
-double BulletJoint::GetParam(const std::string &/*_key*/,
-    unsigned int /*_index*/)
-{
-  gzdbg << "Not implement in Bullet\n";
-  return 0;
-}
-
-//////////////////////////////////////////////////
-double BulletJoint::GetAttribute(const std::string &_key,
+double BulletJoint::GetParam(const std::string &_key,
     unsigned int _index)
 {
-  return this->GetParam(_key, _index);
+  return Joint::GetParam(_key, _index);
 }
 
 //////////////////////////////////////////////////
@@ -584,3 +568,8 @@ math::Angle BulletJoint::GetLowStop(unsigned int _index)
   return this->GetLowerLimit(_index);
 }
 
+//////////////////////////////////////////////////
+bool BulletJoint::SetPosition(unsigned int _index, double _position)
+{
+  return Joint::SetPositionMaximal(_index, _position);
+}
