@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@
 #include "gazebo/physics/bullet/BulletBoxShape.hh"
 #include "gazebo/physics/bullet/BulletCylinderShape.hh"
 #include "gazebo/physics/bullet/BulletMeshShape.hh"
+#include "gazebo/physics/bullet/BulletPolylineShape.hh"
 #include "gazebo/physics/bullet/BulletRayShape.hh"
 
 #include "gazebo/physics/bullet/BulletHingeJoint.hh"
@@ -37,6 +38,7 @@
 #include "gazebo/physics/bullet/BulletSliderJoint.hh"
 #include "gazebo/physics/bullet/BulletHinge2Joint.hh"
 #include "gazebo/physics/bullet/BulletScrewJoint.hh"
+#include "gazebo/physics/bullet/BulletFixedJoint.hh"
 
 #include "gazebo/transport/Publisher.hh"
 
@@ -398,7 +400,10 @@ void BulletPhysics::OnRequest(ConstRequestPtr &_msg)
     physicsMsg.set_contact_surface_layer(
       boost::any_cast<double>(this->GetParam("contact_surface_layer")));
 
-    physicsMsg.mutable_gravity()->CopyFrom(msgs::Convert(this->GetGravity()));
+    physicsMsg.mutable_gravity()->CopyFrom(
+      msgs::Convert(this->GetGravity().Ign()));
+    physicsMsg.mutable_magnetic_field()->CopyFrom(
+        msgs::Convert(this->MagneticField()));
     physicsMsg.set_real_time_update_rate(this->realTimeUpdateRate);
     physicsMsg.set_real_time_factor(this->targetRealTimeFactor);
     physicsMsg.set_max_step_size(this->maxStepSize);
@@ -412,6 +417,11 @@ void BulletPhysics::OnRequest(ConstRequestPtr &_msg)
 /////////////////////////////////////////////////
 void BulletPhysics::OnPhysicsMsg(ConstPhysicsPtr &_msg)
 {
+  // Parent class handles many generic parameters
+  // This should be done first so that the profile settings
+  // can be over-ridden by other message parameters.
+  PhysicsEngine::OnPhysicsMsg(_msg);
+
   if (_msg->has_min_step_size())
     this->SetParam("min_step_size", _msg->min_step_size());
 
@@ -437,7 +447,7 @@ void BulletPhysics::OnPhysicsMsg(ConstPhysicsPtr &_msg)
     this->SetParam("contact_surface_layer", _msg->contact_surface_layer());
 
   if (_msg->has_gravity())
-    this->SetGravity(msgs::Convert(_msg->gravity()));
+    this->SetGravity(msgs::ConvertIgn(_msg->gravity()));
 
   if (_msg->has_real_time_factor())
     this->SetTargetRealTimeFactor(_msg->real_time_factor());
@@ -454,9 +464,6 @@ void BulletPhysics::OnPhysicsMsg(ConstPhysicsPtr &_msg)
 
   /// Make sure all models get at least one update cycle.
   this->world->EnableAllModels();
-
-  // Parent class handles many generic parameters
-  PhysicsEngine::OnPhysicsMsg(_msg);
 }
 
 //////////////////////////////////////////////////
@@ -504,59 +511,6 @@ void BulletPhysics::SetSORPGSIters(unsigned int _iters)
       "solver")->GetElement("iters")->Set(_iters);
 }
 
-
-//////////////////////////////////////////////////
-bool BulletPhysics::SetParam(BulletParam _param, const boost::any &_value)
-{
-  if (!this->dynamicsWorld)
-    return false;
-
-  gzwarn << "Deprecated by "
-         << "bool SetParam(const std::string&, const boost::any&).\n";
-
-  switch (_param)
-  {
-    case SOLVER_TYPE:
-    {
-      return this->SetParam("solver_type", _value);
-    }
-    case GLOBAL_CFM:
-    {
-      return this->SetParam("cfm", _value);
-    }
-    case GLOBAL_ERP:
-    {
-      return this->SetParam("erp", _value);
-    }
-    case PGS_ITERS:
-    {
-      return this->SetParam("iters", _value);
-    }
-    case SOR:
-    {
-      return this->SetParam("sor", _value);
-    }
-    case CONTACT_SURFACE_LAYER:
-    {
-      return this->SetParam("contact_surface_layer", _value);
-    }
-    case MAX_CONTACTS:
-    {
-      return this->SetParam("max_contacts", _value);
-    }
-    case MIN_STEP_SIZE:
-    {
-      return this->SetParam("min_step_size", _value);
-    }
-    default:
-    {
-      gzwarn << "Param not supported in bullet" << std::endl;
-      return false;
-    }
-  }
-  return true;
-}
-
 //////////////////////////////////////////////////
 bool BulletPhysics::SetParam(const std::string &_key, const boost::any &_value)
 {
@@ -565,202 +519,86 @@ bool BulletPhysics::SetParam(const std::string &_key, const boost::any &_value)
 
   btContactSolverInfo& info = this->dynamicsWorld->getSolverInfo();
 
-  if (_key == "type")
+  try
   {
-    gzwarn << "keyword `type` for GetParam/SetParam is deprecated, please"
-           << " use `solver_type`.\n";
-    return this->SetParam("solver_type", _value);
-  }
-  else if (_key == "solver_type")
-  {
-    std::string value;
-    try
+    if (_key == "solver_type")
     {
-      value = boost::any_cast<std::string>(_value);
+      std::string value = boost::any_cast<std::string>(_value);
+      if (value == "sequential_impulse")
+      {
+        bulletElem->GetElement("solver")->GetElement("type")->Set(value);
+        this->solverType = value;
+      }
+      else
+      {
+        gzwarn << "Currently only 'sequential_impulse' solver is supported"
+               << std::endl;
+        return false;
+      }
     }
-    catch(const boost::bad_any_cast &e)
+    else if (_key == "cfm")
     {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("constraints")->GetElement("cfm")->Set(value);
+      info.m_globalCfm = value;
     }
-    if (value == "sequential_impulse")
+    else if (_key == "erp")
     {
-      bulletElem->GetElement("solver")->GetElement("type")->Set(value);
-      this->solverType = value;
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("constraints")->GetElement("erp")->Set(value);
+      info.m_erp = value;
+    }
+    else if (_key == "iters")
+    {
+      int value = boost::any_cast<int>(_value);
+      bulletElem->GetElement("solver")->GetElement("iters")->Set(value);
+      info.m_numIterations = value;
+    }
+    else if (_key == "sor")
+    {
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("solver")->GetElement("sor")->Set(value);
+      info.m_sor = value;
+    }
+    else if (_key == "contact_surface_layer")
+    {
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("constraints")->GetElement(
+          "contact_surface_layer")->Set(value);
+    }
+    else if (_key == "split_impulse")
+    {
+      bool value = boost::any_cast<bool>(_value);
+      bulletElem->GetElement("constraints")->GetElement(
+          "split_impulse")->Set(value);
+    }
+    else if (_key == "split_impulse_penetration_threshold")
+    {
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("constraints")->GetElement(
+          "split_impulse_penetration_threshold")->Set(value);
+    }
+    else if (_key == "max_contacts")
+    {
+      /// TODO: Implement max contacts param
+      int value = boost::any_cast<int>(_value);
+      this->sdf->GetElement("max_contacts")->GetValue()->Set(value);
+    }
+    else if (_key == "min_step_size")
+    {
+      /// TODO: Implement min step size param
+      double value = boost::any_cast<double>(_value);
+      bulletElem->GetElement("solver")->GetElement("min_step_size")->Set(value);
     }
     else
     {
-      gzwarn << "Currently only 'sequential_impulse' solver is supported"
-             << std::endl;
-      return false;
+      return PhysicsEngine::SetParam(_key, _value);
     }
   }
-  else if (_key == "cfm")
+  catch(boost::bad_any_cast &e)
   {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("constraints")->GetElement("cfm")->Set(value);
-    info.m_globalCfm = value;
-  }
-  else if (_key == "erp")
-  {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("constraints")->GetElement("erp")->Set(value);
-    info.m_erp = value;
-  }
-  else if (_key == "iters")
-  {
-    int value;
-    try
-    {
-      try
-      {
-        value = boost::any_cast<int>(_value);
-      }
-      catch(const boost::bad_any_cast &e)
-      {
-        value = boost::any_cast<unsigned int>(_value);
-      }
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("solver")->GetElement("iters")->Set(value);
-    info.m_numIterations = value;
-  }
-  else if (_key == "sor")
-  {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("solver")->GetElement("sor")->Set(value);
-    info.m_sor = value;
-  }
-  else if (_key == "contact_surface_layer")
-  {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("constraints")->GetElement(
-        "contact_surface_layer")->Set(value);
-  }
-  else if (_key == "split_impulse")
-  {
-    bool value;
-    try
-    {
-      value = boost::any_cast<bool>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("constraints")->GetElement(
-        "split_impulse")->Set(value);
-  }
-  else if (_key == "split_impulse_penetration_threshold")
-  {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("constraints")->GetElement(
-        "split_impulse_penetration_threshold")->Set(value);
-  }
-  else if (_key == "max_contacts")
-  {
-    /// TODO: Implement max contacts param
-    int value;
-    try
-    {
-      try
-      {
-        value = boost::any_cast<int>(_value);
-      }
-      catch(const boost::bad_any_cast &e)
-      {
-        value = boost::any_cast<unsigned int>(_value);
-      }
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    this->sdf->GetElement("max_contacts")->GetValue()->Set(value);
-  }
-  else if (_key == "min_step_size")
-  {
-    /// TODO: Implement min step size param
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    bulletElem->GetElement("solver")->GetElement("min_step_size")->Set(value);
-  }
-  else if (_key == "max_step_size")
-  {
-    double value;
-    try
-    {
-      value = boost::any_cast<double>(_value);
-    }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-    this->SetMaxStepSize(value);
-  }
-  else
-  {
-    gzwarn << _key << " is not supported in bullet" << std::endl;
+    gzerr << "BulletPhysics::SetParam(" << _key << ") boost::any_cast error: "
+          << e.what() << std::endl;
     return false;
   }
 
@@ -768,97 +606,51 @@ bool BulletPhysics::SetParam(const std::string &_key, const boost::any &_value)
 }
 
 //////////////////////////////////////////////////
-boost::any BulletPhysics::GetParam(BulletParam _param) const
+boost::any BulletPhysics::GetParam(const std::string &_key) const
 {
-  boost::any value = 0;
-  switch (_param)
-  {
-    case SOLVER_TYPE:
-    {
-      return this->GetParam("solver_type");
-    }
-    case GLOBAL_CFM:
-    {
-      return this->GetParam("cfm");
-    }
-    case GLOBAL_ERP:
-    {
-      return this->GetParam("erp");
-    }
-    case PGS_ITERS:
-    {
-      return this->GetParam("iters");
-    }
-    case SOR:
-    {
-      return this->GetParam("sor");
-    }
-    case CONTACT_SURFACE_LAYER:
-    {
-      return this->GetParam("contact_surface_layer");
-    }
-    case MAX_CONTACTS:
-    {
-      return this->GetParam("max_contacts");
-    }
-    case MIN_STEP_SIZE:
-    {
-      return this->GetParam("min_step_size");
-    }
-    default:
-    {
-      gzwarn << "Param not supported in bullet" << std::endl;
-      return value;
-    }
-  }
+  boost::any value;
+  this->GetParam(_key, value);
+  return value;
 }
 
 //////////////////////////////////////////////////
-boost::any BulletPhysics::GetParam(const std::string &_key) const
+bool BulletPhysics::GetParam(const std::string &_key, boost::any &_value) const
 {
   sdf::ElementPtr bulletElem = this->sdf->GetElement("bullet");
   GZ_ASSERT(bulletElem != NULL, "Bullet SDF element does not exist");
 
-  if (_key == "type")
-  {
-    gzwarn << "keyword `type` for GetParam/SetParam is deprecated, please"
-           << " use `solver_type`.\n";
-    return this->GetParam("solver_type");
-  }
-  else if (_key == "solver_type")
-    return bulletElem->GetElement("solver")->Get<std::string>("type");
+  if (_key == "solver_type")
+    _value = bulletElem->GetElement("solver")->Get<std::string>("type");
   else if (_key == "cfm")
-    return bulletElem->GetElement("constraints")->Get<double>("cfm");
+    _value = bulletElem->GetElement("constraints")->Get<double>("cfm");
   else if (_key == "erp")
-    return bulletElem->GetElement("constraints")->Get<double>("erp");
+    _value = bulletElem->GetElement("constraints")->Get<double>("erp");
   else if (_key == "iters")
-    return bulletElem->GetElement("solver")->Get<int>("iters");
+    _value = bulletElem->GetElement("solver")->Get<int>("iters");
   else if (_key == "sor")
-    return bulletElem->GetElement("solver")->Get<double>("sor");
+    _value = bulletElem->GetElement("solver")->Get<double>("sor");
   else if (_key == "contact_surface_layer")
-    return bulletElem->GetElement("constraints")->Get<double>(
+    _value = bulletElem->GetElement("constraints")->Get<double>(
         "contact_surface_layer");
   else if (_key == "split_impulse")
   {
-    return bulletElem->GetElement("constraints")->Get<bool>(
+    _value = bulletElem->GetElement("constraints")->Get<bool>(
       "split_impulse");
   }
   else if (_key == "split_impulse_penetration_threshold")
   {
-    return bulletElem->GetElement("constraints")->Get<double>(
+    _value = bulletElem->GetElement("constraints")->Get<double>(
       "split_impulse_penetration_threshold");
   }
   else if (_key == "max_contacts")
-    return this->sdf->GetElement("max_contacts")->Get<int>();
+    _value = this->sdf->GetElement("max_contacts")->Get<int>();
   else if (_key == "min_step_size")
-    return bulletElem->GetElement("solver")->Get<double>("min_step_size");
-  else if (_key == "max_step_size")
-    return this->GetMaxStepSize();
+    _value = bulletElem->GetElement("solver")->Get<double>("min_step_size");
   else
   {
-    gzwarn << _key << " is not supported in bullet" << std::endl;
-    return 0;
+    return PhysicsEngine::GetParam(_key, _value);
   }
+  return true;
 }
 
 //////////////////////////////////////////////////
@@ -902,6 +694,8 @@ ShapePtr BulletPhysics::CreateShape(const std::string &_type,
     shape.reset(new BulletCylinderShape(collision));
   else if (_type == "mesh" || _type == "trimesh")
     shape.reset(new BulletMeshShape(collision));
+  else if (_type == "polyline")
+    shape.reset(new BulletPolylineShape(collision));
   else if (_type == "heightmap")
     shape.reset(new BulletHeightmapShape(collision));
   else if (_type == "multiray")
@@ -938,6 +732,8 @@ JointPtr BulletPhysics::CreateJoint(const std::string &_type, ModelPtr _parent)
     joint.reset(new BulletHinge2Joint(this->dynamicsWorld, _parent));
   else if (_type == "screw")
     joint.reset(new BulletScrewJoint(this->dynamicsWorld, _parent));
+  else if (_type == "fixed")
+    joint.reset(new BulletFixedJoint(this->dynamicsWorld, _parent));
   else
     gzthrow("Unable to create joint of type[" << _type << "]");
 
