@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 Open Source Robotics Foundation
+ * Copyright (C) 2014-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,20 +27,17 @@
 #include "gazebo/physics/dart/DARTModel.hh"
 #include "gazebo/physics/dart/DARTJoint.hh"
 
+#include "gazebo/physics/dart/DARTJointPrivate.hh"
+
 using namespace gazebo;
 using namespace physics;
 
 //////////////////////////////////////////////////
 DARTJoint::DARTJoint(BasePtr _parent)
   : Joint(_parent),
-    dtJoint(NULL),
-    dtChildBodyNode(NULL)
+    dataPtr(new DARTJointPrivate(boost::dynamic_pointer_cast<DARTPhysics>(
+      this->GetWorld()->GetPhysicsEngine())))
 {
-  this->dartPhysicsEngine = boost::dynamic_pointer_cast<DARTPhysics>(
-                              this->GetWorld()->GetPhysicsEngine());
-
-  this->forceApplied[0] = 0.0;
-  this->forceApplied[1] = 0.0;
 }
 
 //////////////////////////////////////////////////
@@ -48,8 +45,7 @@ DARTJoint::~DARTJoint()
 {
   this->Detach();
 
-  if (dtJoint)
-    delete dtJoint;
+  delete this->dataPtr;
 }
 
 //////////////////////////////////////////////////
@@ -67,7 +63,7 @@ void DARTJoint::Init()
 
   // Name
   std::string jointName = this->GetName();
-  this->dtJoint->setName(jointName.c_str());
+  this->dataPtr->dtJoint->setName(jointName.c_str());
 
   // Parent and child link information
   DARTLinkPtr dartParentLink =
@@ -86,8 +82,8 @@ void DARTJoint::Init()
   {
     dtTransformChildBodyNode =
         DARTTypes::ConvPose(dartChildLink->GetWorldPose());
-    this->dtChildBodyNode = dartChildLink->GetDARTBodyNode();
-    this->dtChildBodyNode->setParentJoint(this->dtJoint);
+    this->dataPtr->dtChildBodyNode = dartChildLink->GetDARTBodyNode();
+    this->dataPtr->dtChildBodyNode->setParentJoint(this->dataPtr->dtJoint);
   }
   dtTransformChildLinkToJoint = DARTTypes::ConvPose(this->anchorPose);
 
@@ -95,9 +91,9 @@ void DARTJoint::Init()
   {
     dtTransformParentBodyNode =
         DARTTypes::ConvPose(dartParentLink->GetWorldPose());
-    dart::dynamics::BodyNode* dtParentBodyNode =
+    dart::dynamics::BodyNode *dtParentBodyNode =
       dartParentLink->GetDARTBodyNode();
-    dtParentBodyNode->addChildBodyNode(this->dtChildBodyNode);
+    dtParentBodyNode->addChildBodyNode(this->dataPtr->dtChildBodyNode);
   }
 
   dtTransformParentLinkToJoint = dtTransformParentBodyNode.inverse() *
@@ -105,50 +101,10 @@ void DARTJoint::Init()
                                  dtTransformChildLinkToJoint;
 
   // We assume that the joint angles are all zero.
-  this->dtJoint->setTransformFromParentBodyNode(dtTransformParentLinkToJoint);
-  this->dtJoint->setTransformFromChildBodyNode(dtTransformChildLinkToJoint);
-
-  //----------------------------------------------------------------------------
-  // TODO: Currently, dampingCoefficient seems not to be initialized when
-  //       this joint is loaded. Therefore, we need below code...
-  //----------------------------------------------------------------------------
-  if (this->sdf->HasElement("axis"))
-  {
-    sdf::ElementPtr axisElem = this->sdf->GetElement("axis");
-    if (axisElem->HasElement("dynamics"))
-    {
-      sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
-
-      if (dynamicsElem->HasElement("damping"))
-      {
-        this->SetDamping(0, dynamicsElem->Get<double>("damping"));
-      }
-      if (dynamicsElem->HasElement("friction"))
-      {
-        sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
-        gzlog << "joint friction not implemented in DART.\n";
-      }
-    }
-  }
-
-  if (this->sdf->HasElement("axis2"))
-  {
-    sdf::ElementPtr axisElem = this->sdf->GetElement("axis");
-    if (axisElem->HasElement("dynamics"))
-    {
-      sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
-
-      if (dynamicsElem->HasElement("damping"))
-      {
-        this->SetDamping(1, dynamicsElem->Get<double>("damping"));
-      }
-      if (dynamicsElem->HasElement("friction"))
-      {
-        sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
-        gzlog << "joint friction not implemented in DART.\n";
-      }
-    }
-  }
+  this->dataPtr->dtJoint->setTransformFromParentBodyNode(
+        dtTransformParentLinkToJoint);
+  this->dataPtr->dtJoint->setTransformFromChildBodyNode(
+        dtTransformChildLinkToJoint);
 }
 
 //////////////////////////////////////////////////
@@ -233,25 +189,16 @@ void DARTJoint::SetAnchor(unsigned int /*_index*/,
 //////////////////////////////////////////////////
 void DARTJoint::SetDamping(unsigned int _index, double _damping)
 {
-  this->dampingCoefficient = _damping;
-
-  if (this->GetAngleCount() > 2)
+  if (_index < this->GetAngleCount())
   {
-     gzerr << "Incompatible joint type, GetAngleCount() = "
-           << this->GetAngleCount() << " > 2\n";
-     return;
+    this->SetStiffnessDamping(_index, this->stiffnessCoefficient[_index],
+                              _damping);
   }
-
-  // \TODO: implement on a per axis basis (requires additional sdf parameters)
-
-  /// \TODO:  this check might not be needed?  attaching an object to a static
-  /// body should not affect damping application.
-  bool parentStatic = this->GetParent() ? this->GetParent()->IsStatic() : false;
-  bool childStatic = this->GetChild() ? this->GetChild()->IsStatic() : false;
-
-  if (!parentStatic && !childStatic)
+  else
   {
-    this->dtJoint->setDampingCoefficient(_index, _damping);
+    gzerr << "Index[" << _index << "] is out of bounds (GetAngleCount() = "
+          << this->GetAngleCount() << ").\n";
+    return;
   }
 }
 
@@ -282,24 +229,38 @@ void DARTJoint::SetStiffnessDamping(unsigned int _index,
     this->dissipationCoefficient[_index] = _damping;
     this->springReferencePosition[_index] = _reference;
 
-    /// \TODO: set joint stiffness coefficient
+    /// \TODO: this check might not be needed?  attaching an object to a static
+    /// body should not affect damping application.
+    bool parentStatic =
+        this->GetParent() ? this->GetParent()->IsStatic() : false;
+    bool childStatic =
+        this->GetChild() ? this->GetChild()->IsStatic() : false;
 
-    /// setting joint damping
-    bool parentStatic = this->GetParent() ?
-      this->GetParent()->IsStatic() : false;
-    bool childStatic = this->GetChild() ? this->GetChild()->IsStatic() : false;
-
-    if (!parentStatic && !childStatic)
+    if (!this->applyDamping)
     {
-      this->dtJoint->setDampingCoefficient(_index, _damping);
+      if (!parentStatic && !childStatic)
+      {
+        this->dataPtr->dtJoint->setSpringStiffness(
+              static_cast<int>(_index), _stiffness);
+        this->dataPtr->dtJoint->setRestPosition(
+              static_cast<int>(_index), _reference);
+        this->dataPtr->dtJoint->setDampingCoefficient(
+              static_cast<int>(_index), _damping);
+        this->applyDamping = physics::Joint::ConnectJointUpdate(
+          boost::bind(&DARTJoint::ApplyDamping, this));
+      }
+      else
+      {
+        gzwarn << "Spring Damper for Joint[" << this->GetName()
+               << "] is not initialized because either parent[" << parentStatic
+               << "] or child[" << childStatic << "] is static.\n";
+      }
     }
-
-    /// \TODO: add spring force element
-    gzdbg << "Joint [" << this->GetName()
-           << "] stiffness not implement in DART\n";
   }
   else
+  {
     gzerr << "SetStiffnessDamping _index " << _index << " is too large.\n";
+  }
 }
 
 //////////////////////////////////////////////////
@@ -310,7 +271,7 @@ bool DARTJoint::SetHighStop(unsigned int _index, const math::Angle &_angle)
     case 0:
     case 1:
     case 2:
-      this->dtJoint->getGenCoord(_index)->set_qMax(_angle.Radian());
+      this->dataPtr->dtJoint->setPositionUpperLimit(_index, _angle.Radian());
       return true;
     default:
       gzerr << "Invalid index[" << _index << "]\n";
@@ -326,7 +287,7 @@ bool DARTJoint::SetLowStop(unsigned int _index, const math::Angle &_angle)
   case 0:
   case 1:
   case 2:
-    this->dtJoint->getGenCoord(_index)->set_qMin(_angle.Radian());
+    this->dataPtr->dtJoint->setPositionLowerLimit(_index, _angle.Radian());
     return true;
   default:
     gzerr << "Invalid index[" << _index << "]\n";
@@ -342,7 +303,7 @@ math::Angle DARTJoint::GetHighStop(unsigned int _index)
   case 0:
   case 1:
   case 2:
-    return this->dtJoint->getGenCoord(_index)->get_qMax();
+    return this->dataPtr->dtJoint->getPositionUpperLimit(_index);
   default:
     gzerr << "Invalid index[" << _index << "]\n";
   };
@@ -358,7 +319,7 @@ math::Angle DARTJoint::GetLowStop(unsigned int _index)
   case 0:
   case 1:
   case 2:
-    return this->dtJoint->getGenCoord(_index)->get_qMin();
+    return this->dataPtr->dtJoint->getPositionLowerLimit(_index);
   default:
     gzerr << "Invalid index[" << _index << "]\n";
   };
@@ -371,7 +332,7 @@ math::Vector3 DARTJoint::GetLinkForce(unsigned int _index) const
 {
   math::Vector3 result;
 
-  if (!this->dtJoint)
+  if (!this->dataPtr->dtJoint)
   {
     gzerr << "DART joint is invalid\n";
     return result;
@@ -385,7 +346,7 @@ math::Vector3 DARTJoint::GetLinkForce(unsigned int _index) const
 
   Eigen::Vector6d F1 = Eigen::Vector6d::Zero();
   Eigen::Vector6d F2 = Eigen::Vector6d::Zero();
-  Eigen::Isometry3d T12 = dtJoint->getLocalTransform();
+  Eigen::Isometry3d T12 = this->dataPtr->dtJoint->getLocalTransform();
 
   // JointWrench.body1Force contains the
   // force applied by the parent Link on the Joint specified in
@@ -394,8 +355,9 @@ math::Vector3 DARTJoint::GetLinkForce(unsigned int _index) const
   {
     dart::dynamics::BodyNode *dartChildBody = theChildLink->GetDARTBodyNode();
     GZ_ASSERT(dartChildBody, "dartChildBody pointer is NULL");
-    F2 = -dart::math::dAdT(dtJoint->getTransformFromChildBodyNode(),
-                           dartChildBody->getBodyForce());
+    F2 = -dart::math::dAdT(
+          this->dataPtr->dtJoint->getTransformFromChildBodyNode(),
+          dartChildBody->getBodyForce());
   }
 
   // JointWrench.body2Force contains
@@ -416,7 +378,7 @@ math::Vector3 DARTJoint::GetLinkTorque(unsigned int _index) const
 {
   math::Vector3 result;
 
-  if (!this->dtJoint)
+  if (!this->dataPtr->dtJoint)
   {
     gzerr << "DART joint is invalid\n";
     return result;
@@ -428,7 +390,7 @@ math::Vector3 DARTJoint::GetLinkTorque(unsigned int _index) const
 
   Eigen::Vector6d F1 = Eigen::Vector6d::Zero();
   Eigen::Vector6d F2 = Eigen::Vector6d::Zero();
-  Eigen::Isometry3d T12 = dtJoint->getLocalTransform();
+  Eigen::Isometry3d T12 = this->dataPtr->dtJoint->getLocalTransform();
 
   // JointWrench.body1Force contains the
   // force applied by the parent Link on the Joint specified in
@@ -438,7 +400,8 @@ math::Vector3 DARTJoint::GetLinkTorque(unsigned int _index) const
     dart::dynamics::BodyNode *dartChildBody = theChildLink->GetDARTBodyNode();
     GZ_ASSERT(dartChildBody, "dartChildBody pointer is NULL");
     F2 = -dart::math::dAdT(
-      dtJoint->getTransformFromChildBodyNode(), dartChildBody->getBodyForce());
+      this->dataPtr->dtJoint->getTransformFromChildBodyNode(),
+      dartChildBody->getBodyForce());
   }
 
   // JointWrench.body2Force contains
@@ -454,89 +417,67 @@ math::Vector3 DARTJoint::GetLinkTorque(unsigned int _index) const
   return result;
 }
 
-//////////////////////////////////////////////////
-void DARTJoint::SetAttribute(const std::string &_key, unsigned int _index,
-                             const boost::any &_value)
-{
-  this->SetParam(_key, _index, _value);
-}
-
 //////////////////////////////////////////////////
 bool DARTJoint::SetParam(const std::string &_key, unsigned int _index,
-                             const boost::any &_value)
+                         const boost::any &_value)
 {
-  if (_key == "hi_stop")
+  // try because boost::any_cast can throw
+  try
   {
-    try
+    if (_key == "hi_stop")
     {
       this->SetHighStop(_index, boost::any_cast<double>(_value));
     }
-    catch(const boost::bad_any_cast &e)
-    {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
-      return false;
-    }
-  }
-  else if (_key == "lo_stop")
-  {
-    try
+    else if (_key == "lo_stop")
     {
       this->SetLowStop(_index, boost::any_cast<double>(_value));
     }
-    catch(const boost::bad_any_cast &e)
+    else if (_key == "friction")
     {
-      gzerr << "boost any_cast error:" << e.what() << "\n";
+      this->dataPtr->dtJoint->setCoulombFriction(_index,
+                                        boost::any_cast<double>(_value));
+    }
+    else
+    {
+      gzerr << "Unable to handle joint attribute[" << _key << "]\n";
       return false;
     }
   }
-  else
+  catch(const boost::bad_any_cast &e)
   {
-    gzerr << "Unable to handle joint attribute[" << _key << "]\n";
+    gzerr << "SetParam(" << _key << ")"
+          << " boost any_cast error:" << e.what()
+          << std::endl;
     return false;
   }
+
   return true;
 }
 
 //////////////////////////////////////////////////
-double DARTJoint::GetAttribute(const std::string& _key,
-                               unsigned int _index)
+double DARTJoint::GetParam(const std::string &_key, unsigned int _index)
 {
-  return this->GetParam(_key, _index);
+  try
+  {
+    if (_key == "friction")
+    {
+      return this->dataPtr->dtJoint->getCoulombFriction(_index);
+    }
+  }
+  catch(const common::Exception &e)
+  {
+    gzerr << "GetParam(" << _key << ") error:"
+          << e.GetErrorStr()
+          << std::endl;
+    return 0;
+  }
+  return Joint::GetParam(_key, _index);
 }
 
 //////////////////////////////////////////////////
-double DARTJoint::GetParam(const std::string& _key,
-                               unsigned int _index)
+void DARTJoint::CacheForceTorque()
 {
-  if (_key == "hi_stop")
-  {
-    try
-    {
-      return this->GetHighStop(_index).Radian();
-    }
-    catch(common::Exception &e)
-    {
-      gzerr << "GetParam error:" << e.GetErrorStr() << "\n";
-      return 0;
-    }
-  }
-  else if (_key == "lo_stop")
-  {
-    try
-    {
-      return this->GetLowStop(_index).Radian();
-    }
-    catch(common::Exception &e)
-    {
-      gzerr << "GetParam error:" << e.GetErrorStr() << "\n";
-      return 0;
-    }
-  }
-  else
-  {
-    gzerr << "Unable to get joint attribute[" << _key << "]\n";
-    return 0;
-  }
+  // Does nothing for now, will add when recovering pull request #1721
 }
 
 //////////////////////////////////////////////////
@@ -552,22 +493,25 @@ JointWrench DARTJoint::GetForceTorque(unsigned int /*_index*/)
 
   Eigen::Vector6d F1 = Eigen::Vector6d::Zero();
   Eigen::Vector6d F2 = Eigen::Vector6d::Zero();
-  Eigen::Isometry3d T12 = dtJoint->getLocalTransform();
+  Eigen::Isometry3d T12 = this->dataPtr->dtJoint->getLocalTransform();
 
-  // JointWrench.body1Force contains the
-  // force applied by the parent Link on the Joint specified in
-  // the parent Link frame.
+  // JointWrench.body2Force (F2) contains
+  // the force applied by the child Link on the parent link specified
+  // in the child Link orientation frame and with respect to the joint origin
   if (theChildLink != NULL)
   {
     dart::dynamics::BodyNode *dartChildBody = theChildLink->GetDARTBodyNode();
     GZ_ASSERT(dartChildBody, "dartChildBody pointer is NULL");
-    F2 = -dart::math::dAdT(dtJoint->getTransformFromChildBodyNode(),
+    Eigen::Isometry3d TJ2 = Eigen::Isometry3d::Identity();
+    TJ2.translation() =
+        this->dataPtr->dtJoint->getTransformFromChildBodyNode().translation();
+    F2 = -dart::math::dAdT(TJ2,
                            dartChildBody->getBodyForce());
   }
 
-  // JointWrench.body2Force contains
-  // the force applied by the child Link on the Joint specified
-  // in the child Link frame.
+  // JointWrench.body1Force (F1) contains the
+  // force applied by the parent Link on the child Link specified in
+  // the parent Link orientation frame and with respect to the joint origin
   F1 = -dart::math::dAdInvR(T12, F2);
 
   // kind of backwards here, body1 (parent) corresponds go f2, t2
@@ -599,7 +543,7 @@ double DARTJoint::GetForce(unsigned int _index)
 {
   if (_index < this->GetAngleCount())
   {
-    return this->forceApplied[_index];
+    return this->dataPtr->forceApplied[_index];
   }
   else
   {
@@ -614,7 +558,7 @@ unsigned int DARTJoint::GetAngleCount() const
 {
   unsigned int angleCount = 0;
 
-  angleCount = this->dtJoint->getNumGenCoords();
+  angleCount = this->dataPtr->dtJoint->getNumDofs();
 
   return angleCount;
 }
@@ -622,7 +566,14 @@ unsigned int DARTJoint::GetAngleCount() const
 /////////////////////////////////////////////////
 void DARTJoint::ApplyDamping()
 {
-  // DART applying damping force inside of DART.
+  // rename ApplyDamping to ApplyStiffnessDamping (below) in gazebo 4.0.
+  // public: virtual void ApplyStiffnessDamping();
+
+  // DART applies stiffness and damping force implicitly itself by setting
+  // the stiffness coefficient and the damping coefficient using
+  // dart::dynamics::Joint::setSpringStiffness(index, stiffnessCoeff) and
+  // dart::dynamics::Joint::setDampingCoefficient(index, dampingCoeff).
+  // Therefore, we do nothing here.
 }
 
 /////////////////////////////////////////////////
@@ -634,7 +585,7 @@ DARTModelPtr DARTJoint::GetDARTModel() const
 /////////////////////////////////////////////////
 dart::dynamics::Joint *DARTJoint::GetDARTJoint()
 {
-  return this->dtJoint;
+  return this->dataPtr->dtJoint;
 }
 
 /////////////////////////////////////////////////
@@ -644,14 +595,14 @@ void DARTJoint::SaveForce(unsigned int _index, double _force)
   // it simply records the forces commanded inside forceApplied.
   if (_index < this->GetAngleCount())
   {
-    if (this->forceAppliedTime < this->GetWorld()->GetSimTime())
+    if (this->dataPtr->forceAppliedTime < this->GetWorld()->GetSimTime())
     {
       // reset forces if time step is new
-      this->forceAppliedTime = this->GetWorld()->GetSimTime();
-      this->forceApplied[0] = this->forceApplied[1] = 0.0;
+      this->dataPtr->forceAppliedTime = this->GetWorld()->GetSimTime();
+      this->dataPtr->forceApplied[0] = this->dataPtr->forceApplied[1] = 0.0;
     }
 
-    this->forceApplied[_index] += _force;
+    this->dataPtr->forceApplied[_index] += _force;
   }
   else
     gzerr << "Something's wrong, joint [" << this->GetName()
