@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 Open Source Robotics Foundation
+ * Copyright (C) 2014-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,32 +15,38 @@
  *
 */
 
-#include "gazebo/common/Mesh.hh"
-#include "gazebo/common/Exception.hh"
+#include "gazebo/common/Assert.hh"
 #include "gazebo/common/Console.hh"
+#include "gazebo/common/Exception.hh"
+#include "gazebo/common/Mesh.hh"
 
+#include "gazebo/physics/dart/DARTMesh.hh"
 #include "gazebo/physics/dart/DARTCollision.hh"
-#include "gazebo/physics/dart/DARTPhysics.hh"
 #include "gazebo/physics/dart/DARTMeshShape.hh"
+#include "gazebo/physics/dart/DARTPhysics.hh"
+
+#include "gazebo/physics/dart/DARTMeshShapePrivate.hh"
 
 using namespace gazebo;
 using namespace physics;
 
-
 //////////////////////////////////////////////////
-DARTMeshShape::DARTMeshShape(CollisionPtr _parent) : MeshShape(_parent)
+DARTMeshShape::DARTMeshShape(CollisionPtr _parent)
+  : MeshShape(_parent),
+    dataPtr(new DARTMeshShapePrivate())
 {
 }
 
 //////////////////////////////////////////////////
 DARTMeshShape::~DARTMeshShape()
 {
+  delete this->dataPtr;
 }
 
 //////////////////////////////////////////////////
 void DARTMeshShape::Update()
 {
-  gzwarn << "Not implemented!\n";
+  MeshShape::Update();
 }
 
 //////////////////////////////////////////////////
@@ -52,5 +58,18 @@ void DARTMeshShape::Load(sdf::ElementPtr _sdf)
 //////////////////////////////////////////////////
 void DARTMeshShape::Init()
 {
-  gzwarn << "Not implemented!\n";
+  MeshShape::Init();
+
+  if (this->submesh)
+  {
+    this->dataPtr->dartMesh->Init(this->submesh,
+        boost::dynamic_pointer_cast<DARTCollision>(this->collisionParent),
+        this->sdf->Get<math::Vector3>("scale"));
+  }
+  else
+  {
+    this->dataPtr->dartMesh->Init(this->mesh,
+        boost::dynamic_pointer_cast<DARTCollision>(this->collisionParent),
+        this->sdf->Get<math::Vector3>("scale"));
+  }
 }
