@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -67,11 +67,10 @@ void SimbodyJoint::Load(sdf::ElementPtr _sdf)
     {
       sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
 
-      /// \TODO: switch to GetElement so default values apply
-      /// \TODO: check all physics engines
-      if (dynamicsElem->HasElement("damping"))
+      if (dynamicsElem && dynamicsElem->HasElement("friction"))
       {
-        this->dissipationCoefficient[0] = dynamicsElem->Get<double>("damping");
+        sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
+        gzlog << "joint friction not implemented\n";
       }
     }
   }
@@ -83,11 +82,10 @@ void SimbodyJoint::Load(sdf::ElementPtr _sdf)
     {
       sdf::ElementPtr dynamicsElem = axisElem->GetElement("dynamics");
 
-      /// \TODO: switch to GetElement so default values apply
-      /// \TODO: check all physics engines
-      if (dynamicsElem->HasElement("damping"))
+      if (dynamicsElem && dynamicsElem->HasElement("friction"))
       {
-        this->dissipationCoefficient[1] = dynamicsElem->Get<double>("damping");
+        sdf::ElementPtr frictionElem = dynamicsElem->GetElement("friction");
+        gzlog << "joint friction not implemented\n";
       }
     }
   }
@@ -431,18 +429,21 @@ void SimbodyJoint::SetStiffnessDamping(unsigned int _index,
     this->dissipationCoefficient[_index] = _damping;
     this->springReferencePosition[_index] = _reference;
 
-    // set damper coefficient
-    this->damper[_index].setDamping(
-      this->simbodyPhysics->integ->updAdvancedState(),
-      _damping);
+    if (this->physicsInitialized)
+    {
+      // set damper coefficient
+      this->damper[_index].setDamping(
+        this->simbodyPhysics->integ->updAdvancedState(),
+        _damping);
 
-    // set spring stiffness and reference position
-    this->spring[_index].setStiffness(
-      this->simbodyPhysics->integ->updAdvancedState(),
-      _stiffness);
-    this->spring[_index].setQZero(
-      this->simbodyPhysics->integ->updAdvancedState(),
-      _reference);
+      // set spring stiffness and reference position
+      this->spring[_index].setStiffness(
+        this->simbodyPhysics->integ->updAdvancedState(),
+        _stiffness);
+      this->spring[_index].setQZero(
+        this->simbodyPhysics->integ->updAdvancedState(),
+        _reference);
+    }
   }
   else
     gzerr << "SetStiffnessDamping _index too large.\n";
@@ -469,20 +470,6 @@ math::Vector3 SimbodyJoint::GetLinkTorque(unsigned int /*_index*/) const
   return math::Vector3();
 }
 
-//////////////////////////////////////////////////
-void SimbodyJoint::SetAttribute(Attribute, unsigned int /*_index*/,
-    double /*_value*/)
-{
-  gzerr << "Not implement in Simbody\n";
-}
-
-//////////////////////////////////////////////////
-void SimbodyJoint::SetAttribute(const std::string &_key,
-    unsigned int _index, const boost::any &_value)
-{
-  this->SetParam(_key, _index, _value);
-}
-
 //////////////////////////////////////////////////
 bool SimbodyJoint::SetParam(const std::string &/*_key*/,
     unsigned int /*_index*/, const boost::any &/*_value*/)
@@ -492,16 +479,9 @@ bool SimbodyJoint::SetParam(const std::string &/*_key*/,
 }
 
 //////////////////////////////////////////////////
-double SimbodyJoint::GetAttribute(const std::string &_key, unsigned int _index)
+double SimbodyJoint::GetParam(const std::string &_key, unsigned int _index)
 {
-  return this->GetParam(_key, _index);
-}
-//////////////////////////////////////////////////
-double SimbodyJoint::GetParam(const std::string &/*_key*/,
-    unsigned int /*_index*/)
-{
-  gzerr << "Not implement in Simbody\n";
-  return 0;
+  return Joint::GetParam(_key, _index);
 }
 
 //////////////////////////////////////////////////
@@ -513,9 +493,18 @@ bool SimbodyJoint::SetHighStop(unsigned int _index, const math::Angle &_angle)
   {
     if (this->physicsInitialized)
     {
-      this->limitForce[_index].setBounds(
-        this->simbodyPhysics->integ->updAdvancedState(),
-        this->GetLowStop(_index).Radian(), _angle.Radian());
+      if (!this->limitForce[_index].isEmptyHandle())
+      {
+        this->limitForce[_index].setBounds(
+          this->simbodyPhysics->integ->updAdvancedState(),
+          this->GetLowStop(_index).Radian(), _angle.Radian());
+      }
+      else
+      {
+        gzerr << "child link is NULL, force element not initialized, "
+              << "SetHighStop failed. Please file a report on issue tracker.\n";
+        return false;
+      }
     }
     else
     {
@@ -540,10 +529,19 @@ bool SimbodyJoint::SetLowStop(unsigned int _index, const math::Angle &_angle)
   {
     if (this->physicsInitialized)
     {
-      this->limitForce[_index].setBounds(
-        this->simbodyPhysics->integ->updAdvancedState(),
-        _angle.Radian(),
-        this->GetHighStop(_index).Radian());
+      if (!this->limitForce[_index].isEmptyHandle())
+      {
+        this->limitForce[_index].setBounds(
+          this->simbodyPhysics->integ->updAdvancedState(),
+          _angle.Radian(),
+          this->GetHighStop(_index).Radian());
+      }
+      else
+      {
+        gzerr << "child link is NULL, force element not initialized, "
+              << "SetLowStop failed. Please file a report on issue tracker.\n";
+        return false;
+      }
     }
     else
     {
