@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,22 +15,30 @@
  *
 */
 
-#ifdef  __APPLE__
+#include <string>
+#include <iostream>
+#include <boost/filesystem.hpp>
+#include <sys/types.h>
+
+#ifdef __APPLE__
 # include <QtCore/qglobal.h>
 #endif
 
-#ifndef Q_OS_MAC  // Not Apple
+// Not Apple or Windows
+#if not defined( Q_OS_MAC) && not defined(_WIN32)
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <GL/glx.h>
 #endif
 
-#include <sys/types.h>
-#include <dirent.h>
-#include <string>
-#include <iostream>
-
-#include <boost/filesystem.hpp>
+#ifndef _WIN32
+  #include <dirent.h>
+#else
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+  #include "gazebo/common/win_dirent.h"
+#endif
 
 #include "gazebo/rendering/ogre_gazebo.h"
 
@@ -66,6 +74,10 @@ RenderEngine::RenderEngine()
   this->logManager = NULL;
   this->root = NULL;
 
+#if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0))
+  this->overlaySystem = NULL;
+#endif
+
   this->dummyDisplay = NULL;
 
   this->initialized = false;
@@ -116,10 +128,12 @@ void RenderEngine::Load()
       gzthrow("Unable to create an Ogre rendering environment, no Root ");
     }
 
-#if OGRE_VERSION_MAJR > 1 || OGRE_VERSION_MINOR >= 9
-    // Must be created after this->root, but before this->root is
-    // initialized.
-    this->overlaySystem = new Ogre::OverlaySystem();
+#if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0))
+    // OgreOverlay is a component on its own in ogre 1.9 so must manually
+    // initialize it. Must be created after this->root, but before this->root
+    // is initialized.
+    if (!this->overlaySystem)
+      this->overlaySystem = new Ogre::OverlaySystem();
 #endif
 
     // Load all the plugins
@@ -135,10 +149,12 @@ void RenderEngine::Load()
     this->SetupResources();
   }
 
-  std::stringstream stream;
-  stream << (int32_t)this->dummyWindowId;
+  // Create a 1x1 render window so that we can grab a GL context. Based on
+  // testing, this is a hard requirement by Apple. We also need it to
+  // properly initialize GLWidget and UserCameras. See the GLWidget
+  // constructor.
+  this->windowManager->CreateWindow(std::to_string(this->dummyWindowId), 1, 1);
 
-  this->windowManager->CreateWindow(stream.str(), 1, 1);
   this->CheckSystemCapabilities();
 }
 
@@ -268,7 +284,9 @@ void RenderEngine::Render()
 //////////////////////////////////////////////////
 void RenderEngine::PostRender()
 {
-  // _fireFrameRenderingQueued needs to be here for CEGUI to work
+  // _fireFrameRenderingQueued was here for CEGUI to work. Leaving because
+  // it shouldn't harm anything, and we don't want to introduce
+  // a regression.
   this->root->_fireFrameRenderingQueued();
   this->root->_fireFrameEnded();
 }
@@ -277,7 +295,12 @@ void RenderEngine::PostRender()
 void RenderEngine::Init()
 {
   if (this->renderPathType == NONE)
+  {
+    gzwarn << "Cannot initialize render engine since "
+           << "render path type is NONE. Ignore this warning if"
+           << "rendering has been turned off on purpose.\n";
     return;
+  }
 
   this->initialized = false;
 
@@ -326,6 +349,11 @@ void RenderEngine::Fini()
     this->RemoveScene(this->scenes.front()->GetName());
   }
 
+#if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0))
+  delete this->overlaySystem;
+  this->overlaySystem = NULL;
+#endif
+
   // TODO: this was causing a segfault. Need to debug, and put back in
   if (this->root)
   {
@@ -358,7 +386,8 @@ void RenderEngine::Fini()
     this->scenes[i].reset();
   this->scenes.clear();
 
-#ifndef Q_OS_MAC
+  // Not Apple or Windows
+# if not defined( Q_OS_MAC) && not defined(_WIN32)
   if (this->dummyDisplay)
   {
     glXDestroyContext(static_cast<Display*>(this->dummyDisplay),
@@ -368,7 +397,7 @@ void RenderEngine::Fini()
     XCloseDisplay(static_cast<Display*>(this->dummyDisplay));
     this->dummyDisplay = NULL;
   }
-#endif
+# endif
 
   this->initialized = false;
 }
@@ -408,6 +437,10 @@ void RenderEngine::LoadPlugins()
     plugins.push_back(path+"/"+prefix+"Plugin_BSPSceneManager");
     plugins.push_back(path+"/"+prefix+"Plugin_OctreeSceneManager");
 
+#ifdef HAVE_OCULUS
+    plugins.push_back(path+"/Plugin_CgProgramManager");
+#endif
+
     for (piter = plugins.begin(); piter != plugins.end(); ++piter)
     {
       try
@@ -654,7 +687,7 @@ bool RenderEngine::CreateContext()
 {
   bool result = true;
 
-#ifdef Q_OS_MAC
+#if defined Q_OS_MAC || _WIN32
   this->dummyDisplay = 0;
 #else
   try
@@ -768,7 +801,7 @@ WindowManagerPtr RenderEngine::GetWindowManager() const
   return this->windowManager;
 }
 
-#if OGRE_VERSION_MAJR > 1 || OGRE_VERSION_MINOR >= 9
+#if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0))
 /////////////////////////////////////////////////
 Ogre::OverlaySystem *RenderEngine::GetOverlaySystem() const
 {
