@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,12 @@
  *
 */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include "gazebo/common/MeshManager.hh"
 
 #include "gazebo/gui/GuiIface.hh"
@@ -36,10 +42,12 @@ SelectionObj::SelectionObj(const std::string &_name, VisualPtr _vis)
   SelectionObjPrivate *dPtr =
       reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
 
+  dPtr->type = VT_GUI;
+
   dPtr->state = SELECTION_NONE;
   dPtr->mode = SELECTION_NONE;
 
-  dPtr->maxScale = 3.0;
+  dPtr->maxScale = 2.5;
   dPtr->minScale = 0.5;
 
   dPtr->xAxisMatOverlay = "Gazebo/RedTransparentOverlay";
@@ -62,24 +70,69 @@ SelectionObj::~SelectionObj()
 /////////////////////////////////////////////////
 void SelectionObj::Load()
 {
-  SelectionObjPrivate *dPtr =
-      reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
-
   Visual::Load();
 
   this->CreateRotateVisual();
   this->CreateTranslateVisual();
   this->CreateScaleVisual();
 
-  this->SetVisibilityFlags(GZ_VISIBILITY_GUI);
+  this->SetVisibilityFlags(GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
 
-  dPtr->transVisual->SetVisible(false);
-  dPtr->rotVisual->SetVisible(false);
-  dPtr->scaleVisual->SetVisible(false);
+  this->SetHandleVisible(TRANS, false);
+  this->SetHandleVisible(ROT, false);
+  this->SetHandleVisible(SCALE, false);
 
   this->GetSceneNode()->setInheritScale(false);
 }
 
+/////////////////////////////////////////////////
+void SelectionObj::Fini()
+{
+  SelectionObjPrivate *dPtr =
+      reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
+
+  // Destroy objects and nodes created by this visual
+  if (!dPtr->scene)
+    return;
+  Ogre::SceneManager *manager = dPtr->scene->GetManager();
+  if (!manager)
+    return;
+
+  // transVisual / rotVisual / scaleVisual
+  for (unsigned int i = 0; i < this->GetChildCount(); ++i)
+  {
+    if (!this->GetChild(i))
+      continue;
+
+    // transXVisual / transYVisual / transZVisual
+    for (unsigned int j = 0; j < this->GetChild(i)->GetChildCount(); ++j)
+    {
+      if (!this->GetChild(i)->GetChild(j) ||
+          !this->GetChild(i)->GetChild(j)->GetSceneNode())
+      {
+        continue;
+      }
+
+      // transShaftXNode / transHeadXNode
+      for (auto k = this->GetChild(i)->GetChild(j)->GetSceneNode()->
+          numChildren()-1; k >= 0; --k)
+      {
+        Ogre::SceneNode *toDestroy =
+            reinterpret_cast<Ogre::SceneNode *>(
+            this->GetChild(i)->
+            GetChild(j)->
+            GetSceneNode()->
+            getChild(k));
+
+        manager->destroyMovableObject(toDestroy->getAttachedObject(0));
+        manager->destroySceneNode(toDestroy);
+      }
+    }
+  }
+
+  Visual::Fini();
+}
+
 /////////////////////////////////////////////////
 void SelectionObj::Attach(rendering::VisualPtr _vis)
 {
@@ -123,15 +176,15 @@ void SelectionObj::UpdateSize()
   // overlays for big objects.
   if (math::equal(max, dPtr->maxScale))
   {
-    dPtr->rotXVisual->SetMaterial(dPtr->xAxisMatOverlay, false);
-    dPtr->rotYVisual->SetMaterial(dPtr->yAxisMatOverlay, false);
-    dPtr->rotZVisual->SetMaterial(dPtr->zAxisMatOverlay, false);
+    this->SetHandleMaterial(ROT_X, dPtr->xAxisMatOverlay, false);
+    this->SetHandleMaterial(ROT_Y, dPtr->yAxisMatOverlay, false);
+    this->SetHandleMaterial(ROT_Z, dPtr->zAxisMatOverlay, false);
   }
   else
   {
-    dPtr->rotXVisual->SetMaterial(dPtr->xAxisMat, false);
-    dPtr->rotYVisual->SetMaterial(dPtr->yAxisMat, false);
-    dPtr->rotZVisual->SetMaterial(dPtr->zAxisMat, false);
+    this->SetHandleMaterial(ROT_X, dPtr->xAxisMat, false);
+    this->SetHandleMaterial(ROT_Y, dPtr->yAxisMat, false);
+    this->SetHandleMaterial(ROT_Z, dPtr->zAxisMat, false);
   }
   this->SetScale(math::Vector3(max, max, max));
 }
@@ -173,16 +226,11 @@ void SelectionObj::SetMode(SelectionMode _mode)
 
   dPtr->mode = _mode;
 
-  dPtr->transVisual->SetVisible(false);
-  dPtr->rotVisual->SetVisible(false);
-  dPtr->scaleVisual->SetVisible(false);
+  this->SetHandleVisible(TRANS, false);
+  this->SetHandleVisible(ROT, false);
+  this->SetHandleVisible(SCALE, false);
 
-  if (dPtr->mode == TRANS)
-    dPtr->transVisual->SetVisible(true);
-  else if (dPtr->mode == ROT)
-    dPtr->rotVisual->SetVisible(true);
-  else if (dPtr->mode == SCALE)
-    dPtr->scaleVisual->SetVisible(true);
+  this->SetHandleVisible(dPtr->mode, true);
 }
 
 /////////////////////////////////////////////////
@@ -317,6 +365,7 @@ void SelectionObj::CreateTranslateVisual()
   dPtr->transVisual.reset(new rendering::Visual(
       this->GetName() + "__SELECTION_OBJ_TRANS__",
       shared_from_this()));
+  dPtr->transVisual->Load();
 
   dPtr->transXVisual.reset(
       new rendering::Visual(
@@ -353,8 +402,10 @@ void SelectionObj::CreateTranslateVisual()
   transHeadXNode->attachObject(headXObj);
   transHeadXNode->setScale(0.5, 0.5, 0.5);
   transHeadXNode->setPosition(0, 0, 0.22);
-  shaftXObj->setUserAny(Ogre::Any(std::string("trans_x")));
-  headXObj->setUserAny(Ogre::Any(std::string("trans_x")));
+  shaftXObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_x")));
+  headXObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_x")));
   shaftXObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   headXObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -376,8 +427,10 @@ void SelectionObj::CreateTranslateVisual()
   transHeadYNode->attachObject(headYObj);
   transHeadYNode->setScale(0.5, 0.5, 0.5);
   transHeadYNode->setPosition(0, 0, 0.22);
-  shaftYObj->setUserAny(Ogre::Any(std::string("trans_y")));
-  headYObj->setUserAny(Ogre::Any(std::string("trans_y")));
+  shaftYObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_y")));
+  headYObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_y")));
   shaftYObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   headYObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -399,8 +452,10 @@ void SelectionObj::CreateTranslateVisual()
   transHeadZNode->attachObject(headZObj);
   transHeadZNode->setScale(0.5, 0.5, 0.5);
   transHeadZNode->setPosition(0, 0, 0.22);
-  shaftZObj->setUserAny(Ogre::Any(std::string("trans_z")));
-  headZObj->setUserAny(Ogre::Any(std::string("trans_z")));
+  shaftZObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_z")));
+  headZObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("trans_z")));
   shaftZObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   headZObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -409,9 +464,9 @@ void SelectionObj::CreateTranslateVisual()
   dPtr->transYVisual->SetRotation(
       math::Quaternion(math::Vector3(1, 0, 0), GZ_DTOR(-90)));
 
-  dPtr->transXVisual->SetMaterial(dPtr->xAxisMatOverlay);
-  dPtr->transYVisual->SetMaterial(dPtr->yAxisMatOverlay);
-  dPtr->transZVisual->SetMaterial(dPtr->zAxisMatOverlay);
+  this->SetHandleMaterial(TRANS_X, dPtr->xAxisMatOverlay);
+  this->SetHandleMaterial(TRANS_Y, dPtr->yAxisMatOverlay);
+  this->SetHandleMaterial(TRANS_Z, dPtr->zAxisMatOverlay);
 
   dPtr->transVisual->SetScale(math::Vector3(5.0, 5.0, 5.0));
 
@@ -421,11 +476,6 @@ void SelectionObj::CreateTranslateVisual()
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
   dPtr->transZVisual->SetVisibilityFlags(
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
-
-  // Add to scene so they are selectable by the mouse
-  dPtr->scene->AddVisual(dPtr->transXVisual);
-  dPtr->scene->AddVisual(dPtr->transYVisual);
-  dPtr->scene->AddVisual(dPtr->transZVisual);
 }
 
 /////////////////////////////////////////////////
@@ -438,6 +488,7 @@ void SelectionObj::CreateRotateVisual()
   dPtr->rotVisual.reset(new rendering::Visual(
       this->GetName() + "__SELECTION_OBJ_ROT__",
       shared_from_this()));
+  dPtr->rotVisual->Load();
 
   dPtr->rotXVisual.reset(
       new rendering::Visual(
@@ -458,7 +509,8 @@ void SelectionObj::CreateRotateVisual()
       dPtr->rotXVisual->GetSceneNode()->createChildSceneNode(
       "__SELECTION_OBJ__ROT_NODE_X__"  + this->GetName());
   xNode->attachObject(rotXObj);
-  rotXObj->setUserAny(Ogre::Any(std::string("rot_x")));
+  rotXObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("rot_x")));
   rotXObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
   Ogre::MovableObject *rotYObj =
@@ -468,7 +520,8 @@ void SelectionObj::CreateRotateVisual()
       dPtr->rotYVisual->GetSceneNode()->createChildSceneNode(
       "__SELECTION_OBJ__ROT_NODE_Y__"  + this->GetName());
   yNode->attachObject(rotYObj);
-  rotYObj->setUserAny(Ogre::Any(std::string("rot_y")));
+  rotYObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("rot_y")));
   rotYObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
   Ogre::MovableObject *rotZObj =
@@ -478,7 +531,8 @@ void SelectionObj::CreateRotateVisual()
       dPtr->rotZVisual->GetSceneNode()->createChildSceneNode(
       "__SELECTION_OBJ__ROT_NODE_Z__"  + this->GetName());
   zNode->attachObject(rotZObj);
-  rotZObj->setUserAny(Ogre::Any(std::string("rot_z")));
+  rotZObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("rot_z")));
   rotZObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
   dPtr->rotXVisual->Load();
@@ -493,9 +547,9 @@ void SelectionObj::CreateRotateVisual()
   // By default the visuals are not overlays like translation or scale visuals.
   // This is so that the rings does not block the object it's attached too,
   // and also gives with better depth perception.
-  dPtr->rotXVisual->SetMaterial(dPtr->xAxisMat);
-  dPtr->rotYVisual->SetMaterial(dPtr->yAxisMat);
-  dPtr->rotZVisual->SetMaterial(dPtr->zAxisMat);
+  this->SetHandleMaterial(ROT_X, dPtr->xAxisMat);
+  this->SetHandleMaterial(ROT_Y, dPtr->yAxisMat);
+  this->SetHandleMaterial(ROT_Z, dPtr->zAxisMat);
 
   dPtr->rotVisual->SetScale(math::Vector3(1.0, 1.0, 1.0));
 
@@ -505,11 +559,6 @@ void SelectionObj::CreateRotateVisual()
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
   dPtr->rotZVisual->SetVisibilityFlags(
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
-
-  // Add to scene so they are selectable by the mouse
-  dPtr->scene->AddVisual(dPtr->rotXVisual);
-  dPtr->scene->AddVisual(dPtr->rotYVisual);
-  dPtr->scene->AddVisual(dPtr->rotZVisual);
 }
 
 /////////////////////////////////////////////////
@@ -522,6 +571,7 @@ void SelectionObj::CreateScaleVisual()
   dPtr->scaleVisual.reset(new rendering::Visual(
       this->GetName() + "__SELECTION_OBJ_SCALE__",
       shared_from_this()));
+  dPtr->scaleVisual->Load();
 
   dPtr->scaleXVisual.reset(
       new rendering::Visual(
@@ -557,8 +607,10 @@ void SelectionObj::CreateScaleVisual()
   scaleHeadXNode->attachObject(scaleHeadXObj);
   scaleHeadXNode->setScale(0.02, 0.02, 0.02);
   scaleHeadXNode->setPosition(0, 0, 0.21);
-  scaleShaftXObj->setUserAny(Ogre::Any(std::string("scale_x")));
-  scaleHeadXObj->setUserAny(Ogre::Any(std::string("scale_x")));
+  scaleShaftXObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_x")));
+  scaleHeadXObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_x")));
   scaleShaftXObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   scaleHeadXObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -580,8 +632,10 @@ void SelectionObj::CreateScaleVisual()
   scaleHeadYNode->attachObject(scaleHeadYObj);
   scaleHeadYNode->setScale(0.02, 0.02, 0.02);
   scaleHeadYNode->setPosition(0, 0, 0.21);
-  scaleShaftYObj->setUserAny(Ogre::Any(std::string("scale_y")));
-  scaleHeadYObj->setUserAny(Ogre::Any(std::string("scale_y")));
+  scaleShaftYObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_y")));
+  scaleHeadYObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_y")));
   scaleShaftYObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   scaleHeadYObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -603,8 +657,10 @@ void SelectionObj::CreateScaleVisual()
   scaleHeadZNode->attachObject(scaleHeadZObj);
   scaleHeadZNode->setScale(0.02, 0.02, 0.02);
   scaleHeadZNode->setPosition(0, 0, 0.21);
-  scaleShaftZObj->setUserAny(Ogre::Any(std::string("scale_z")));
-  scaleHeadZObj->setUserAny(Ogre::Any(std::string("scale_z")));
+  scaleShaftZObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_z")));
+  scaleHeadZObj->getUserObjectBindings().setUserAny(
+      Ogre::Any(std::string("scale_z")));
   scaleShaftZObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
   scaleHeadZObj->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
 
@@ -613,9 +669,9 @@ void SelectionObj::CreateScaleVisual()
   dPtr->scaleYVisual->SetRotation(
       math::Quaternion(math::Vector3(1, 0, 0), GZ_DTOR(-90)));
 
-  dPtr->scaleXVisual->SetMaterial(dPtr->xAxisMatOverlay);
-  dPtr->scaleYVisual->SetMaterial(dPtr->yAxisMatOverlay);
-  dPtr->scaleZVisual->SetMaterial(dPtr->zAxisMatOverlay);
+  this->SetHandleMaterial(SCALE_X, dPtr->xAxisMatOverlay);
+  this->SetHandleMaterial(SCALE_Y, dPtr->yAxisMatOverlay);
+  this->SetHandleMaterial(SCALE_Z, dPtr->zAxisMatOverlay);
 
   dPtr->scaleVisual->SetScale(math::Vector3(5.0, 5.0, 5.0));
 
@@ -625,9 +681,85 @@ void SelectionObj::CreateScaleVisual()
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
   dPtr->scaleZVisual->SetVisibilityFlags(
       GZ_VISIBILITY_GUI | GZ_VISIBILITY_SELECTABLE);
+}
 
-  // Add to scene so they are selectable by the mouse
-  dPtr->scene->AddVisual(dPtr->scaleXVisual);
-  dPtr->scene->AddVisual(dPtr->scaleYVisual);
-  dPtr->scene->AddVisual(dPtr->scaleZVisual);
+/////////////////////////////////////////////////
+void SelectionObj::SetHandleVisible(SelectionMode _mode, bool _visible)
+{
+  SelectionObjPrivate *dPtr =
+      reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
+
+  if (_mode == TRANS || _mode == TRANS_X)
+    dPtr->transXVisual->SetVisible(_visible);
+  if (_mode == TRANS || _mode == TRANS_Y)
+    dPtr->transYVisual->SetVisible(_visible);
+  if (_mode == TRANS || _mode == TRANS_Z)
+    dPtr->transZVisual->SetVisible(_visible);
+  if (_mode == ROT || _mode == ROT_X)
+    dPtr->rotXVisual->SetVisible(_visible);
+  if (_mode == ROT || _mode == ROT_Y)
+    dPtr->rotYVisual->SetVisible(_visible);
+  if (_mode == ROT || _mode == ROT_Z)
+    dPtr->rotZVisual->SetVisible(_visible);
+  if (_mode == SCALE || _mode == SCALE_X)
+    dPtr->scaleXVisual->SetVisible(_visible);
+  if (_mode == SCALE || _mode == SCALE_Y)
+    dPtr->scaleYVisual->SetVisible(_visible);
+  if (_mode == SCALE || _mode == SCALE_Z)
+    dPtr->scaleZVisual->SetVisible(_visible);
+}
+
+/////////////////////////////////////////////////
+bool SelectionObj::GetHandleVisible(SelectionMode _mode) const
+{
+  SelectionObjPrivate *dPtr =
+      reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
+
+  if (_mode == TRANS || _mode == TRANS_X)
+    return dPtr->transXVisual->GetVisible();
+  if (_mode == TRANS_Y)
+    return dPtr->transYVisual->GetVisible();
+  if (_mode == TRANS_Z)
+    return dPtr->transZVisual->GetVisible();
+  if (_mode == ROT || _mode == ROT_X)
+    return dPtr->rotXVisual->GetVisible();
+  if (_mode == ROT_Y)
+    return dPtr->rotYVisual->GetVisible();
+  if (_mode == ROT_Z)
+    return dPtr->rotZVisual->GetVisible();
+  if (_mode == SCALE || _mode == SCALE_X)
+    return dPtr->scaleXVisual->GetVisible();
+  if (_mode == SCALE_Y)
+    return dPtr->scaleYVisual->GetVisible();
+  if (_mode == SCALE_Z)
+    return dPtr->scaleZVisual->GetVisible();
+
+  return false;
+}
+
+/////////////////////////////////////////////////
+void SelectionObj::SetHandleMaterial(SelectionMode _mode, const std::string
+    &_material, bool _unique)
+{
+  SelectionObjPrivate *dPtr =
+      reinterpret_cast<SelectionObjPrivate *>(this->dataPtr);
+
+  if (_mode == TRANS || _mode == TRANS_X)
+    dPtr->transXVisual->SetMaterial(_material, _unique);
+  if (_mode == TRANS || _mode == TRANS_Y)
+    dPtr->transYVisual->SetMaterial(_material, _unique);
+  if (_mode == TRANS || _mode == TRANS_Z)
+    dPtr->transZVisual->SetMaterial(_material, _unique);
+  if (_mode == ROT || _mode == ROT_X)
+    dPtr->rotXVisual->SetMaterial(_material, _unique);
+  if (_mode == ROT || _mode == ROT_Y)
+    dPtr->rotYVisual->SetMaterial(_material, _unique);
+  if (_mode == ROT || _mode == ROT_Z)
+    dPtr->rotZVisual->SetMaterial(_material, _unique);
+  if (_mode == SCALE || _mode == SCALE_X)
+    dPtr->scaleXVisual->SetMaterial(_material, _unique);
+  if (_mode == SCALE || _mode == SCALE_Y)
+    dPtr->scaleYVisual->SetMaterial(_material, _unique);
+  if (_mode == SCALE || _mode == SCALE_Z)
+    dPtr->scaleZVisual->SetMaterial(_material, _unique);
 }
