@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,10 +14,11 @@
  * limitations under the License.
  *
 */
-/* Desc: Ray proximity sensor
- * Author: Mihai Emanuel Dolha
- * Date: 29 March 2012
-*/
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
 
 #include "gazebo/physics/World.hh"
 #include "gazebo/physics/Entity.hh"
@@ -26,8 +27,6 @@
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Events.hh"
 
-#include "gazebo/math/Rand.hh"
-
 #include "gazebo/transport/transport.hh"
 
 #include "gazebo/rendering/Scene.hh"
@@ -106,9 +105,9 @@ void GpuRaySensor::Load(const std::string &_worldName)
   // Handle noise model settings.
   if (rayElem->HasElement("noise"))
   {
-    this->noises.push_back(
+    this->noises[GPU_RAY_NOISE] =
         NoiseFactory::NewNoiseModel(rayElem->GetElement("noise"),
-        this->GetType()));
+        this->GetType());
   }
 
   this->parentEntity = this->world->GetEntity(this->parentName);
@@ -160,16 +159,15 @@ void GpuRaySensor::Init()
     this->laserCam->SetNearClip(this->GetRangeMin());
     this->laserCam->SetFarClip(this->GetRangeMax());
 
-    this->laserCam->SetHorzFOV(
-      (this->GetAngleMax() - this->GetAngleMin()).Radian());
-    this->laserCam->SetVertFOV((this->GetVerticalAngleMax()
-            - this->GetVerticalAngleMin()).Radian());
+    this->laserCam->SetHorzFOV((this->AngleMax() - this->AngleMin()).Radian());
+    this->laserCam->SetVertFOV(
+        (this->VerticalAngleMax() - this->VerticalAngleMin()).Radian());
 
     this->laserCam->SetHorzHalfAngle(
-      (this->GetAngleMax() + this->GetAngleMin()).Radian() / 2.0);
+      (this->AngleMax() + this->AngleMin()).Radian() / 2.0);
 
-    this->laserCam->SetVertHalfAngle((this->GetVerticalAngleMax()
-            + this->GetVerticalAngleMin()).Radian() / 2.0);
+    this->laserCam->SetVertHalfAngle((this->VerticalAngleMax()
+            + this->VerticalAngleMin()).Radian() / 2.0);
 
     if (this->GetHorzFOV() > 2 * M_PI)
       this->laserCam->SetHorzFOV(2*M_PI);
@@ -378,6 +376,12 @@ double GpuRaySensor::GetRangeCountRatio() const
 
 //////////////////////////////////////////////////
 math::Angle GpuRaySensor::GetAngleMin() const
+{
+  return this->AngleMin();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Angle GpuRaySensor::AngleMin() const
 {
   return this->horzElem->Get<double>("min_angle");
 }
@@ -390,6 +394,12 @@ void GpuRaySensor::SetAngleMin(double _angle)
 
 //////////////////////////////////////////////////
 math::Angle GpuRaySensor::GetAngleMax() const
+{
+  return this->AngleMax();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Angle GpuRaySensor::AngleMax() const
 {
   return this->horzElem->Get<double>("max_angle");
 }
@@ -415,7 +425,7 @@ double GpuRaySensor::GetRangeMax() const
 /////////////////////////////////////////////////
 double GpuRaySensor::GetAngleResolution() const
 {
-  return (this->GetAngleMax() - this->GetAngleMin()).Radian() /
+  return (this->AngleMax() - this->AngleMin()).Radian() /
     (this->GetRangeCount()-1);
 }
 
@@ -465,11 +475,17 @@ int GpuRaySensor::GetVerticalRangeCount() const
 
 //////////////////////////////////////////////////
 math::Angle GpuRaySensor::GetVerticalAngleMin() const
+{
+  return this->VerticalAngleMin();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Angle GpuRaySensor::VerticalAngleMin() const
 {
   if (this->scanElem->HasElement("vertical"))
     return this->vertElem->Get<double>("min_angle");
   else
-    return math::Angle(0);
+    return ignition::math::Angle(0);
 }
 
 //////////////////////////////////////////////////
@@ -481,17 +497,23 @@ void GpuRaySensor::SetVerticalAngleMin(double _angle)
 
 //////////////////////////////////////////////////
 math::Angle GpuRaySensor::GetVerticalAngleMax() const
+{
+  return this->VerticalAngleMax();
+}
+
+//////////////////////////////////////////////////
+ignition::math::Angle GpuRaySensor::VerticalAngleMax() const
 {
   if (this->scanElem->HasElement("vertical"))
     return this->vertElem->Get<double>("max_angle");
   else
-    return math::Angle(0);
+    return ignition::math::Angle(0);
 }
 
 //////////////////////////////////////////////////
 double GpuRaySensor::GetVerticalAngleResolution() const
 {
-  return (this->GetVerticalAngleMax() - this->GetVerticalAngleMin()).Radian() /
+  return (this->VerticalAngleMax() - this->VerticalAngleMin()).Radian() /
     (this->GetVerticalRangeCount()-1);
 }
 
@@ -570,14 +592,14 @@ bool GpuRaySensor::UpdateImpl(bool /*_force*/)
 
   // Store the latest laser scans into laserMsg
   msgs::Set(scan->mutable_world_pose(),
-      this->pose + this->parentEntity->GetWorldPose());
-  scan->set_angle_min(this->GetAngleMin().Radian());
-  scan->set_angle_max(this->GetAngleMax().Radian());
+      this->pose + this->parentEntity->GetWorldPose().Ign());
+  scan->set_angle_min(this->AngleMin().Radian());
+  scan->set_angle_max(this->AngleMax().Radian());
   scan->set_angle_step(this->GetAngleResolution());
   scan->set_count(this->GetRayCount());
 
-  scan->set_vertical_angle_min(this->GetVerticalAngleMin().Radian());
-  scan->set_vertical_angle_max(this->GetVerticalAngleMax().Radian());
+  scan->set_vertical_angle_min(this->VerticalAngleMin().Radian());
+  scan->set_vertical_angle_max(this->VerticalAngleMax().Radian());
   scan->set_vertical_angle_step(this->GetVerticalAngleResolution());
   scan->set_vertical_count(this->GetVerticalRayCount());
 
@@ -594,13 +616,23 @@ bool GpuRaySensor::UpdateImpl(bool /*_force*/)
       int index = j * this->GetRayCount() + i;
       double range = this->laserCam->GetLaserData()[index * 3];
 
-      if (!this->noises.empty())
+      // Mask ranges outside of min/max to +/- inf, as per REP 117
+      if (range >= this->GetRangeMax())
       {
-        range = this->noises[0]->Apply(range);
-        range = math::clamp(range, this->GetRangeMin(), this->GetRangeMax());
+        range = GZ_DBL_INF;
+      }
+      else if (range <= this->GetRangeMin())
+      {
+        range = -GZ_DBL_INF;
+      }
+      else if (this->noises.find(GPU_RAY_NOISE) != this->noises.end())
+      {
+        range = this->noises[GPU_RAY_NOISE]->Apply(range);
+        range = ignition::math::clamp(range,
+            this->GetRangeMin(), this->GetRangeMax());
       }
 
-      range = math::isnan(range) ? this->GetRangeMax() : range;
+      range = ignition::math::isnan(range) ? this->GetRangeMax() : range;
 
       if (add)
       {
