@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,11 +14,12 @@
  * limitations under the License.
  *
 */
-/*
- * Desc: Class to manager all sensors
- * Author: Nate Koenig
- * Date: 18 Dec 2009
- */
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Time.hh"
 
@@ -223,18 +224,6 @@ void SensorManager::GetSensorTypes(std::vector<std::string> &_types) const
   sensors::SensorFactory::GetSensorTypes(_types);
 }
 
-//////////////////////////////////////////////////
-std::string SensorManager::CreateSensor(sdf::ElementPtr _elem,
-                                        const std::string &_worldName,
-                                        const std::string &_parentName)
-{
-  SensorPtr parentSensor = sensors::get_sensor(_parentName);
-  GZ_ASSERT(parentSensor, "Unable to get parent sensor");
-
-  return this->CreateSensor(_elem, _worldName, _parentName,
-      parentSensor->GetId());
-}
-
 //////////////////////////////////////////////////
 std::string SensorManager::CreateSensor(sdf::ElementPtr _elem,
                                         const std::string &_worldName,
@@ -434,7 +423,9 @@ void SensorManager::SensorContainer::Stop()
   this->runCondition.notify_all();
   if (this->runThread)
   {
-    this->runThread->interrupt();
+    // Note: calling interrupt seems to cause the thread to either block
+    // or throw an exception, so commenting it out for now.
+    // this->runThread->interrupt();
     this->runThread->join();
     delete this->runThread;
     this->runThread = NULL;
