@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,13 +14,14 @@
  * limitations under the License.
  *
 */
+#include <ignition/math/Rand.hh>
 
-#include "ServerFixture.hh"
+#include "gazebo/test/ServerFixture.hh"
 #include "gazebo/physics/physics.hh"
 #include "gazebo/sensors/sensors.hh"
 #include "gazebo/common/common.hh"
 #include "scans_cmp.h"
-#include "helper_physics_generator.hh"
+#include "gazebo/test/helper_physics_generator.hh"
 
 #define LASER_TOL 1e-5
 #define DOUBLE_TOL 1e-6
@@ -68,22 +69,21 @@ void LaserTest::Stationary_EmptyWorld(const std::string &_physicsEngine)
     boost::static_pointer_cast<sensors::RaySensor>(
         sensors::SensorManager::Instance()->GetSensor(raySensorName));
 
-  ASSERT_TRUE(laser);
+  ASSERT_TRUE(laser != NULL);
   laser->Init();
   laser->Update(true);
 
   EXPECT_EQ(640, laser->GetRayCount());
   EXPECT_EQ(640, laser->GetRangeCount());
-  EXPECT_NEAR(laser->GetAngleMin().Radian(), -2.27, DOUBLE_TOL);
-  EXPECT_NEAR(laser->GetAngleMax().Radian(), 2.27, DOUBLE_TOL);
+  EXPECT_NEAR(laser->AngleMin().Radian(), -2.27, DOUBLE_TOL);
+  EXPECT_NEAR(laser->AngleMax().Radian(), 2.27, DOUBLE_TOL);
   EXPECT_NEAR(laser->GetRangeMin(), 0, DOUBLE_TOL);
   EXPECT_NEAR(laser->GetRangeMax(), 10, DOUBLE_TOL);
   EXPECT_NEAR(laser->GetRangeResolution(), 0.01, DOUBLE_TOL);
 
-
   for (int i = 0; i < laser->GetRangeCount(); ++i)
   {
-    EXPECT_EQ(10, laser->GetRange(i));
+    EXPECT_DOUBLE_EQ(GZ_DBL_INF, laser->GetRange(i));
   }
 
   // Spawn a box and test for proper laser scan
@@ -116,7 +116,7 @@ void LaserTest::Stationary_EmptyWorld(const std::string &_physicsEngine)
   {
     common::Time prevTime;
     physics::WorldPtr world = physics::get_world("default");
-    ASSERT_TRUE(world);
+    ASSERT_TRUE(world != NULL);
 
     physics::ModelPtr model = world->GetModel(modelName);
 
@@ -247,9 +247,7 @@ void LaserTest::LaserUnitBox(const std::string &_physicsEngine)
   EXPECT_NEAR(raySensor->GetRange(mid), expectedRangeAtMidPoint, LASER_TOL);
   EXPECT_NEAR(raySensor->GetRange(0), expectedRangeAtMidPoint, LASER_TOL);
 
-  // WARNING: for readings of no return, gazebo returns max range rather
-  // than +inf. issue #124
-  EXPECT_NEAR(raySensor->GetRange(samples-1), maxRange, LASER_TOL);
+  EXPECT_DOUBLE_EQ(raySensor->GetRange(samples-1), GZ_DBL_INF);
 
   // Move all boxes out of range
   world->GetModel(box01)->SetWorldPose(
@@ -262,7 +260,7 @@ void LaserTest::LaserUnitBox(const std::string &_physicsEngine)
 
   for (int i = 0; i < raySensor->GetRayCount(); ++i)
   {
-    EXPECT_NEAR(raySensor->GetRange(i), maxRange, LASER_TOL);
+    EXPECT_DOUBLE_EQ(raySensor->GetRange(i), GZ_DBL_INF);
   }
 }
 
@@ -347,11 +345,8 @@ void LaserTest::LaserVertical(const std::string &_physicsEngine)
 
     angleStep += vAngleStep;
 
-    // WARNING: for readings of no return, gazebo returns max range rather
-    // than +inf. issue #124
-    EXPECT_NEAR(raySensor->GetRange(i*samples), maxRange, LASER_TOL);
-    EXPECT_NEAR(raySensor->GetRange(i*samples + samples-1),
-        maxRange, LASER_TOL);
+    EXPECT_DOUBLE_EQ(raySensor->GetRange(i*samples), GZ_DBL_INF);
+    EXPECT_DOUBLE_EQ(raySensor->GetRange(i*samples + samples-1), GZ_DBL_INF);
   }
 
   // Move box out of range
@@ -364,8 +359,8 @@ void LaserTest::LaserVertical(const std::string &_physicsEngine)
   {
     for (int i = 0; i < raySensor->GetRayCount(); ++i)
     {
-      EXPECT_NEAR(raySensor->GetRange(j*raySensor->GetRayCount() + i),
-          maxRange, LASER_TOL);
+      EXPECT_DOUBLE_EQ(raySensor->GetRange(j*raySensor->GetRayCount() + i),
+          GZ_DBL_INF);
     }
   }
 }
@@ -586,20 +581,13 @@ void LaserTest::LaserUnitNoise(const std::string &_physicsEngine)
   physics::WorldPtr world = physics::get_world("default");
   ASSERT_TRUE(world != NULL);
 
-  // Expect at least one value to be non-max (empty world), and expect the
-  // mean to be close to the max+noiseMean
-  double total = 0.0;
   bool foundNoise = false;
   for (int i = 0; i < raySensor->GetRayCount(); ++i)
   {
     if (fabs(raySensor->GetRange(i) - maxRange) > LASER_TOL)
       foundNoise = true;
-    total += raySensor->GetRange(i);
   }
   EXPECT_TRUE(foundNoise);
-  double mean = total / raySensor->GetRayCount();
-  // The mean should be well within 3-sigma
-  EXPECT_NEAR(mean, maxRange + noiseMean, 3*noiseStdDev);
 }
 
 TEST_P(LaserTest, LaserNoise)
@@ -613,7 +601,7 @@ int main(int argc, char **argv)
 {
   // Set a specific seed to avoid occasional test failures due to
   // statistically unlikely, but possible results.
-  math::Rand::SetSeed(42);
+  ignition::math::Rand::Seed(42);
   ::testing::InitGoogleTest(&argc, argv);
   return RUN_ALL_TESTS();
 }
