@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2014 Open Source Robotics Foundation
+ * Copyright (C) 2013-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,8 +17,9 @@
 #include <boost/filesystem.hpp>
 #include <fstream>
 
-#include "ServerFixture.hh"
+#include "gazebo/test/ServerFixture.hh"
 
+using namespace gazebo;
 class OgreLog : public ServerFixture
 {
 };
