@@ -1,5 +1,5 @@
 /*
- * Copyright 2014 Open Source Robotics Foundation
+ * Copyright (C) 2014-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,14 @@
  * limitations under the License.
  *
 */
+
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <stdio.h>
 #include <signal.h>
 #include <boost/filesystem.hpp>
 #include <boost/algorithm/string.hpp>
@@ -137,7 +145,11 @@ bool Command::Run(int _argc, char **_argv)
   {
     int argvLen = strlen(_argv[i]) + 1;
     this->argv[i] = new char[argvLen];
+    #ifndef _WIN32
     snprintf(this->argv[i], argvLen, "%s", _argv[i]);
+    #else
+    sprintf_s(this->argv[i], argvLen, "%s", _argv[i]);
+    #endif
   }
 
   // The SDF find file callback.
@@ -289,7 +301,8 @@ PhysicsCommand::PhysicsCommand()
      "eg: -g 0,0,-9.8")
     ("step-size,s", po::value<double>(), "Maximum step size (seconds).")
     ("iters,i", po::value<double>(), "Number of iterations.")
-    ("update-rate,u", po::value<double>(), "Target real-time update rate.");
+    ("update-rate,u", po::value<double>(), "Target real-time update rate.")
+    ("profile,o", po::value<std::string>(), "Preset physics profile.");
 }
 
 /////////////////////////////////////////////////
@@ -351,6 +364,13 @@ bool PhysicsCommand::RunImpl()
     good = true;
   }
 
+  if (this->vm.count("profile") &&
+      this->vm["profile"].as<std::string>().size() > 0)
+  {
+    msg.set_profile_name(this->vm["profile"].as<std::string>());
+    good = true;
+  }
+
   if (good)
     pub->Publish(msg, true);
   else
@@ -370,6 +390,9 @@ ModelCommand::ModelCommand()
     ("delete,d", "Delete a model.")
     ("spawn-file,f", po::value<std::string>(), "Spawn model from SDF file.")
     ("spawn-string,s", "Spawn model from SDF string, pass by a pipe.")
+    ("info,i", "Output model state information to the terminal.")
+    ("pose,p",
+     "Output model pose as a space separated 6-tuple: x y z roll pitch yaw.")
     ("pose-x,x", po::value<double>(), "x value")
     ("pose-y,y", po::value<double>(), "y value")
     ("pose-z,z", po::value<double>(), "z value")
@@ -487,6 +510,28 @@ bool ModelCommand::RunImpl()
 
     return this->ProcessSpawn(sdf, modelName, pose, node);
   }
+  else if (this->vm.count("info") || this->vm.count("pose"))
+  {
+    boost::shared_ptr<msgs::Response> response = gazebo::transport::request(
+        worldName, "entity_info", modelName);
+    gazebo::msgs::Model modelMsg;
+
+    if (response->has_serialized_data() &&
+        !response->serialized_data().empty() &&
+        modelMsg.ParseFromString(response->serialized_data()))
+    {
+      if (this->vm.count("info"))
+        std::cout << modelMsg.DebugString() << std::endl;
+      else if (this->vm.count("pose"))
+        std::cout << gazebo::msgs::ConvertIgn(modelMsg.pose()) << std::endl;
+    }
+    else
+    {
+      std::string tmpWorldName = worldName.empty() ? "default" : worldName;
+      std::cout << "Unable to get info on model[" << modelName << "] in "
+        << "the world[" << tmpWorldName << "]\n";
+    }
+  }
   else
   {
     transport::PublisherPtr pub =
@@ -495,7 +540,7 @@ bool ModelCommand::RunImpl()
 
     msgs::Model msg;
     msg.set_name(modelName);
-    msgs::Set(msg.mutable_pose(), pose);
+    msgs::Set(msg.mutable_pose(), pose.Ign());
     pub->Publish(msg, true);
   }
 
@@ -506,7 +551,7 @@ bool ModelCommand::RunImpl()
 bool ModelCommand::ProcessSpawn(boost::shared_ptr<sdf::SDF> _sdf,
     const std::string &_name, const math::Pose &_pose, transport::NodePtr _node)
 {
-  sdf::ElementPtr modelElem = _sdf->root->GetElement("model");
+  sdf::ElementPtr modelElem = _sdf->Root()->GetElement("model");
 
   if (!modelElem)
   {
@@ -523,7 +568,7 @@ bool ModelCommand::ProcessSpawn(boost::shared_ptr<sdf::SDF> _sdf,
 
   msgs::Factory msg;
   msg.set_sdf(_sdf->ToString());
-  msgs::Set(msg.mutable_pose(), _pose);
+  msgs::Set(msg.mutable_pose(), _pose.Ign());
   pub->Publish(msg, true);
 
   return true;
@@ -904,12 +949,12 @@ bool SDFCommand::TransportRequired()
 /////////////////////////////////////////////////
 bool SDFCommand::RunImpl()
 {
-  sdf::SDF::version = SDF_VERSION;
+  sdf::SDF::Version(SDF_VERSION);
 
   try
   {
     // Initialize the informational logger. This will log warnings and errors.
-    gzLogInit("gzsdf.log");
+    gzLogInit("gz-", "gzsdf.log");
   }
   catch(gazebo::common::Exception &_e)
   {
@@ -923,8 +968,8 @@ bool SDFCommand::RunImpl()
   {
     try
     {
-      sdf::SDF::version = boost::lexical_cast<std::string>(
-          this->vm["version"].as<std::string>());
+      sdf::SDF::Version(boost::lexical_cast<std::string>(
+          this->vm["version"].as<std::string>()));
     }
     catch(...)
     {
@@ -973,7 +1018,7 @@ bool SDFCommand::RunImpl()
     TiXmlDocument xmlDoc;
     if (xmlDoc.LoadFile(path.string()))
     {
-      if (sdf::Converter::Convert(&xmlDoc, sdf::SDF::version, true))
+      if (sdf::Converter::Convert(&xmlDoc, sdf::SDF::Version(), true))
       {
         // Create an XML printer to control formatting
         TiXmlPrinter printer;
