@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2014 Open Source Robotics Foundation
+ * Copyright (C) 2013-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -201,8 +201,6 @@ TEST_F(gzTest, Joint)
 {
   init();
 
-  std::string expectedStr;
-
   std::string helpOutput = custom_exec_str("gz help joint");
   EXPECT_NE(helpOutput.find("gz joint"), std::string::npos);
 
@@ -291,7 +289,7 @@ TEST_F(gzTest, Model)
     gazebo::msgs::Model msg;
     msg.set_name("simple_arm");
     gazebo::msgs::Set(msg.mutable_pose(),
-        gazebo::math::Pose(1.1, 2.3, 4.5, 0.1, 1.2, 3.4));
+        ignition::math::Pose3d(1.1, 2.3, 4.5, 0.1, 1.2, 3.4));
     EXPECT_EQ(g_msgDebugOut, msg.DebugString());
   }
 
@@ -301,19 +299,17 @@ TEST_F(gzTest, Model)
 
     waitForMsg("gz model -w default -m my_box -f " + filename);
 
-    std::ifstream ifs(filename.c_str());
-    EXPECT_TRUE(ifs);
-
     boost::shared_ptr<sdf::SDF> sdf(new sdf::SDF());
     EXPECT_TRUE(sdf::init(sdf));
 
     EXPECT_TRUE(sdf::readFile(filename, sdf));
-    sdf::ElementPtr modelElem = sdf->root->GetElement("model");
+    sdf::ElementPtr modelElem = sdf->Root()->GetElement("model");
     modelElem->GetAttribute("name")->SetFromString("my_box");
 
     gazebo::msgs::Factory msg;
     msg.set_sdf(sdf->ToString());
-    gazebo::msgs::Set(msg.mutable_pose(), gazebo::math::Pose(0, 0, 0, 0, 0, 0));
+    gazebo::msgs::Set(msg.mutable_pose(),
+        ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
 
     EXPECT_EQ(g_msgDebugOut, msg.DebugString());
   }
@@ -326,23 +322,62 @@ TEST_F(gzTest, Model)
     cmd += filename + " | gz model -w default -m my_box -s";
     waitForMsg(cmd);
 
-    std::ifstream ifs(filename.c_str());
-    EXPECT_TRUE(ifs);
-
     boost::shared_ptr<sdf::SDF> sdf(new sdf::SDF());
     EXPECT_TRUE(sdf::init(sdf));
 
     EXPECT_TRUE(sdf::readFile(filename, sdf));
-    sdf::ElementPtr modelElem = sdf->root->GetElement("model");
+    sdf::ElementPtr modelElem = sdf->Root()->GetElement("model");
     modelElem->GetAttribute("name")->SetFromString("my_box");
 
     gazebo::msgs::Factory msg;
     msg.set_sdf(sdf->ToString());
-    gazebo::msgs::Set(msg.mutable_pose(), gazebo::math::Pose(0, 0, 0, 0, 0, 0));
+    gazebo::msgs::Set(msg.mutable_pose(),
+        ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
 
     EXPECT_EQ(g_msgDebugOut, msg.DebugString());
   }
 
+  // Test model info and pose
+  {
+    // Make sure the error message is output.
+    std::string modelInfo = custom_exec_str("gz model -m does_not_exist -i");
+    EXPECT_EQ(gazebo::common::get_sha1<std::string>(modelInfo),
+        "7b5a9ab178ce5fa6ae74c80a33a99b84183ae600");
+
+    // Get info for a model that exists.
+    modelInfo = custom_exec_str("gz model -m my_box -i");
+
+    // Check that a few values exist. We don't check the sha1 value
+    // because a few values, such as pose, are dynamic.
+    EXPECT_TRUE(modelInfo.find("name: \"my_box\"") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("id: 9") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("name: \"my_box::link::collision\"")
+        != std::string::npos);
+
+    // Get the pose of the model.
+    modelInfo = custom_exec_str("gz model -m my_box -p");
+    boost::algorithm::trim(modelInfo);
+
+    // Split the string into parts p.
+    std::vector<std::string> p;
+    boost::split(p, modelInfo, boost::is_any_of(" "));
+
+    // Make sure we have the right number of parts.
+    // Don't ASSERT_EQ, because we need to run fini at end of test
+    EXPECT_EQ(p.size(), 6u);
+
+    // Make sure the pose is correct.
+    if (p.size() == 6u)
+    {
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[0]), 0.0));
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[1]), 0.0));
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[2]), 0.5));
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[3]), 0.0));
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[4]), 0.0));
+      EXPECT_NO_THROW(EXPECT_DOUBLE_EQ(boost::lexical_cast<double>(p[5]), 0.0));
+    }
+  }
+
   // Test model delete
   {
     waitForMsg("gz model -w default -m simple_arm -d");
@@ -583,11 +618,11 @@ TEST_F(gzTest, SDF)
   // Regenerate each sum using:
   // gz sdf -d -v <major.minor> | sha1sum'
   std::map<std::string, std::string> descSums;
-  descSums["1.0"] = "5235eb8464a96505c2a31fe96327d704e45c9cc4";
-  descSums["1.2"] = "27973b2542d7a0f7582a615b245d81797718c89a";
-  descSums["1.3"] = "30ffce1c662c17185d23f30ef3af5c110d367e10";
-  descSums["1.4"] = "a917916d211b711c6cba42ffd6811f9a659fce75";
-  descSums["1.5"] = "2904518b31e9d2319e6a5b8737b29826218b5b54";
+  descSums["1.0"] = "a02fbc1275100569c99d860044563f669934c0fc";
+  descSums["1.2"] = "f524458ace57d6aabbbc2303da208f65af37ef53";
+  descSums["1.3"] = "74a3aa8d31f97328175f43d03410be55631fa0e1";
+  descSums["1.4"] = "057f26137669d9d7eeb5a8c6f51e4f4077d9ddcf";
+  descSums["1.5"] = "522285759f420eba3b774e610822c357a0a683e2";
 
   // Test each descSum
   for (std::map<std::string, std::string>::iterator iter = descSums.begin();
@@ -603,10 +638,10 @@ TEST_F(gzTest, SDF)
   // gz sdf -o -v <major.minor> | sha1sum'
   std::map<std::string, std::string> docSums;
   docSums["1.0"] = "4cf955ada785adf72503744604ffadcdf13ec0d2";
-  docSums["1.2"] = "f84c1cf1b1ba04ab4859e96f6aea881134fb5a9b";
-  docSums["1.3"] = "f3dd699687c8922710e4492aadedd1c038d678c1";
-  docSums["1.4"] = "8d136b204ea6428bd99ee2dc4fd5cf385a3e4c3d";
-  docSums["1.5"] = "6fc5f41d43e8c0a32cfb9a194488eb6ba0bc6281";
+  docSums["1.2"] = "27f9d91080ce8aa18eac27c9d899fde2d4b78785";
+  docSums["1.3"] = "ad80986d42eae97baf277118f52d7e8b951d8ea1";
+  docSums["1.4"] = "153ddd6ba6797c37c7fcddb2be5362c9969d97a1";
+  docSums["1.5"] = "73b73f2735debfb86ca8361009ca32e3e0712ed4";
 
   // Test each docSum
   for (std::map<std::string, std::string>::iterator iter = docSums.begin();
@@ -620,7 +655,7 @@ TEST_F(gzTest, SDF)
 
 
   path = TEST_PATH;
-  path /= "worlds/empty_different_name.world";
+  path /= "worlds/box_plane_low_friction_test.world";
 
   {
     // Check empty.world
@@ -637,7 +672,7 @@ TEST_F(gzTest, SDF)
     std::string output =
       custom_exec_str(std::string("gz sdf -p ") + path.string());
     std::string shasum = gazebo::common::get_sha1<std::string>(output);
-    EXPECT_EQ(shasum, "19898716e05fecb7bd1d78e43fe1294ccf403bbf");
+    EXPECT_EQ(shasum, "ea127f9858a5e07c40ef6d949ef6113236adddfa");
   }
 
   path = PROJECT_BINARY_PATH;
