@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,12 @@
  *
 */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <boost/algorithm/string/regex.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/posix_time/posix_time_io.hpp>
@@ -38,12 +44,18 @@ std::ostringstream &FilterBase::Out(std::ostringstream &_stream,
 {
   if (!this->xmlOutput && !this->stamp.empty())
   {
+    std::ios_base::fmtflags flags = _stream.flags();
+
+    _stream.setf(std::ios::fixed);
     if (this->stamp == "sim")
       _stream << _state.GetSimTime().Double() << " ";
     else if (this->stamp == "real")
       _stream << _state.GetRealTime().Double() << " ";
     else if (this->stamp == "wall")
       _stream << _state.GetWallTime().Double() << " ";
+    else if (this->stamp == "iterations")
+      _stream << _state.GetIterations() << " ";
+    _stream.setf(flags);
   }
 
   return _stream;
@@ -543,13 +555,14 @@ std::string StateFilter::Filter(const std::string &_stateString)
       << "<state world_name='" << state.GetName() << "'>\n"
       << "<sim_time>" << state.GetSimTime() << "</sim_time>\n"
       << "<real_time>" << state.GetRealTime() << "</real_time>\n"
-      << "<wall_time>" << state.GetWallTime() << "</wall_time>\n";
+      << "<wall_time>" << state.GetWallTime() << "</wall_time>\n"
+      << "<iterations>" << state.GetIterations() << "</iterations>\n";
   }
 
   result << this->filter.Filter(state);
 
   if (this->xmlOutput)
-    result << "</sdf></state>\n";
+    result << "</state></sdf>\n";
 
   this->prevTime = state.GetSimTime();
   return result.str();
@@ -904,16 +917,18 @@ std::string LogCommand::GetFileSizeStr(const std::string &_filename)
 /////////////////////////////////////////////////
 int LogCommand::GetChar()
 {
+# ifndef _WIN32
   struct termios oldt, newt;
-  int ch;
   tcgetattr(STDIN_FILENO, &oldt);
   newt = oldt;
   newt.c_lflag &= ~(ICANON | ECHO);
   tcsetattr(STDIN_FILENO, TCSANOW, &newt);
-  ch = getchar();
+  int ch = getchar();
   tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
-
   return ch;
+# else
+  return 'q';
+# endif
 }
 
 /////////////////////////////////////////////////
